Package: Rpdb
Type: Package
Title: Read, Write, Visualize and Manipulate PDB Files
Version: 2.4.5
Date: 2026-05-11
Authors@R: 
	c(
    person("Leonard", "Mada", email = "lmada@umft.ro", role = c("cre", "ctb")),
    person("Julien", "Idé", role = c("aut"))
	)
Depends: rgl (>= 1.3.36), stats
Description: Provides tools to read, write, visualize Protein Data Bank (PDB) files and
    perform some structural manipulations.
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
NeedsCompilation: no
Repository: CRAN
URL: https://github.com/discoleo/Rpdb
BugReports: https://github.com/discoleo/Rpdb/issues
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-12 12:57:50 UTC; leo.mada
Author: Leonard Mada [cre, ctb],
  Julien Idé [aut]
Maintainer: Leonard Mada <lmada@umft.ro>
Date/Publication: 2026-05-12 16:20:11 UTC
Built: R 4.5.2; ; 2026-05-12 17:51:23 UTC; unix

Package: Rpdb
Type: Package
Title: Read, Write, Visualize and Manipulate PDB Files
Version: 2.4.5
Date: 2026-05-11
Authors@R: 
	c(
    person("Leonard", "Mada", email = "lmada@umft.ro", role = c("cre", "ctb")),
    person("Julien", "Idé", role = c("aut"))
	)
Depends: rgl (>= 1.3.36), stats
Description: Provides tools to read, write, visualize Protein Data Bank (PDB) files and
    perform some structural manipulations.
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
NeedsCompilation: no
Repository: CRAN
URL: https://github.com/discoleo/Rpdb
BugReports: https://github.com/discoleo/Rpdb/issues
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-12 12:57:50 UTC; leo.mada
Author: Leonard Mada [cre, ctb],
  Julien Idé [aut]
Maintainer: Leonard Mada <lmada@umft.ro>
Date/Publication: 2026-05-12 16:20:11 UTC
Built: R 4.6.0; ; 2026-05-12 18:28:56 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("[",atoms)
S3method("basis<-",default)
S3method("basis<-",pdb)
S3method("coords<-",atoms)
S3method("coords<-",pdb)
S3method(Mab,coords)
S3method(Mab,pdb)
S3method(Mbc,coords)
S3method(Mbc,pdb)
S3method(Mca,coords)
S3method(Mca,pdb)
S3method(Mxy,coords)
S3method(Mxy,pdb)
S3method(Myz,coords)
S3method(Myz,pdb)
S3method(Mzx,coords)
S3method(Mzx,pdb)
S3method(R,coords)
S3method(R,pdb)
S3method(Rx,coords)
S3method(Rx,pdb)
S3method(Ry,coords)
S3method(Ry,pdb)
S3method(Rz,coords)
S3method(Rz,pdb)
S3method(Ta,coords)
S3method(Ta,pdb)
S3method(Tabc,coords)
S3method(Tabc,pdb)
S3method(Tb,coords)
S3method(Tb,pdb)
S3method(Tc,coords)
S3method(Tc,pdb)
S3method(Tx,coords)
S3method(Tx,pdb)
S3method(Txyz,coords)
S3method(Txyz,pdb)
S3method(Ty,coords)
S3method(Ty,pdb)
S3method(Tz,coords)
S3method(Tz,pdb)
S3method(abc2xyz,atoms)
S3method(abc2xyz,coords)
S3method(abc2xyz,distances)
S3method(abc2xyz,pdb)
S3method(addEleLab,atoms)
S3method(addEleLab,pdb)
S3method(addResLab,atoms)
S3method(addResLab,pdb)
S3method(angle,coords)
S3method(angle,pdb)
S3method(atoms,atoms)
S3method(atoms,default)
S3method(atoms,pdb)
S3method(basis,default)
S3method(basis,pdb)
S3method(bond,coords)
S3method(bond,pdb)
S3method(cell.coords,crystal)
S3method(cell.coords,default)
S3method(cell.coords,pdb)
S3method(cell.density,default)
S3method(cell.density,pdb)
S3method(cell.volume,crystal)
S3method(cell.volume,pdb)
S3method(centres,atoms)
S3method(centres,coords)
S3method(centres,pdb)
S3method(centres,ppRoll)
S3method(centres.ppRoll,atoms)
S3method(centres.ppRoll,default)
S3method(centres.ppRoll,pdb)
S3method(chains,atoms)
S3method(chains,pdb)
S3method(connect,atoms)
S3method(connect,character)
S3method(connect,coords)
S3method(connect,default)
S3method(connect,nucleic)
S3method(connect,pdb)
S3method(connect.nucleic,atoms)
S3method(connect.nucleic,pdb)
S3method(coords,atoms)
S3method(coords,coords)
S3method(coords,data.frame)
S3method(coords,default)
S3method(coords,matrix)
S3method(coords,pdb)
S3method(crystal,default)
S3method(crystal,pdb)
S3method(dihedral,coords)
S3method(dihedral,pdb)
S3method(dist.point,default)
S3method(distances,atoms)
S3method(distances,connect)
S3method(distances,coords)
S3method(distances,default)
S3method(distances,pdb)
S3method(format.pdb.title,character)
S3method(inertia,atoms)
S3method(inertia,coords)
S3method(inertia,pdb)
S3method(info3d,atoms)
S3method(info3d,pdb)
S3method(lattice.range,pdb)
S3method(masses,default)
S3method(masses,pdb)
S3method(measure,coords)
S3method(measure,default)
S3method(measure,pdb)
S3method(merge,atoms)
S3method(merge,coords)
S3method(merge,pdb)
S3method(merge,pdb.hetero)
S3method(merge,pdb.resolution)
S3method(merge,pdb.structure)
S3method(mirror,coords)
S3method(mirror,pdb)
S3method(natom,atoms)
S3method(natom,coords)
S3method(natom,pdb)
S3method(norm,connect)
S3method(norm,distances)
S3method(pdb,default)
S3method(print,coords)
S3method(proj.line3d,matrix)
S3method(proj.line3d,numeric)
S3method(range,atoms)
S3method(range,coords)
S3method(range,pdb)
S3method(range.lattice,pdb)
S3method(reindex,atoms)
S3method(reindex,pdb)
S3method(replicate,atoms)
S3method(replicate,coords)
S3method(replicate,pdb)
S3method(select,atoms)
S3method(select,pdb)
S3method(split,pdb)
S3method(subset,atoms)
S3method(subset,pdb)
S3method(toSymbols,character)
S3method(toSymbols,integer)
S3method(toSymbols,numeric)
S3method(unsplit,default)
S3method(unsplit,pdb)
S3method(visualize,atoms)
S3method(visualize,character)
S3method(visualize,coords)
S3method(visualize,data.frame)
S3method(visualize,matrix)
S3method(visualize,pdb)
S3method(wrap,atoms)
S3method(wrap,coords)
S3method(wrap,pdb)
S3method(xyz2abc,atoms)
S3method(xyz2abc,coords)
S3method(xyz2abc,distances)
S3method(xyz2abc,pdb)
export("basis<-")
export("coords<-")
export(Mab)
export(Mbc)
export(Mca)
export(Mxy)
export(Myz)
export(Mzx)
export(R)
export(Rx)
export(Ry)
export(Rz)
export(Ta)
export(Tabc)
export(Tb)
export(Tc)
export(Tx)
export(Txyz)
export(Ty)
export(Tz)
export(abc2xyz)
export(addABC)
export(addBBox)
export(addEleLab)
export(addPBCBox)
export(addResLab)
export(addXYZ)
export(angle)
export(as.coords)
export(atoms)
export(basis)
export(bbox.pdb)
export(bond)
export(cell.coords)
export(cell.density)
export(cell.volume)
export(centres)
export(chains)
export(connect)
export(connect.nucleic)
export(coords)
export(crystal)
export(dihedral)
export(dist.point)
export(distances)
export(dotProd)
export(format.pdb.title)
export(inertia)
export(info3d)
export(is.atoms)
export(is.connect)
export(is.coords)
export(is.crystal)
export(is.distances)
export(is.pdb)
export(lattice.range)
export(masses)
export(measure)
export(mirror)
export(natom)
export(norm)
export(pdb)
export(proj.line3d)
export(range.lattice)
export(read.pdb)
export(reindex)
export(replicate)
export(rotVect)
export(select)
export(toSymbols)
export(trans)
export(translate)
export(unsplit)
export(vectNorm)
export(vectProd)
export(viewAB)
export(viewAxis)
export(viewBC)
export(viewCA)
export(viewInertia)
export(viewXY)
export(viewYZ)
export(viewZX)
export(visualize)
export(wrap)
export(write.pdb)
export(xyz2abc)
import(rgl)
importFrom(grDevices,rgb)

# Rpdb: Read, Write, Visualize and Manipulate PDB Files

- Reanimated the Rpdb package from the CRAN archive;
- Original author: Julien Idé;

## Changes

- see [News.md] for all changes;

## Usage

```
library(Rpdb)
# or: devtools::load_all(_path_)

# Download some legacy PDB files:
x = read.pdb(file.choose())
visualize(x, type="l")
```

**Example:**
```
# Download legacy PDB files from:
# https://www.rcsb.org/
x = read.pdb("7K3G.pdb")

# NMR Structure:
# Name of function drop.h: not yet stable;
x0H = drop.h(x)
con = connect(x0H) # Brute-force connectivity;
# Visualisation:
visualize(atoms(x0H), connect = con, type="l", pbc = F); addBBox(x)

# Labels:
res = residues(x0H, "A")
cols = rep("#84A232", nrow(res))
cols[res$resname %in% c("SER", "THR", "ASN")] = "red"
mark.pdb(x0H, ch = "A", col = cols, lwd = 2)

```

Note:
- Visualisation is incomplete;
- Started redesigning the Bounding box vs PBC box;
- Some functions are not yet visible in Rpdb, but can be accessed using: Rpdb:::__function__(...);

<!-- badges: start -->
[![R-CMD-check](https://github.com/discoleo/Rpdb/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/discoleo/Rpdb/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

# Rpdb: Read, Write, Visualize and Manipulate PDB Files

- Reanimated the Rpdb package from the CRAN archive;
- Original author: Julien Idé;

## Changes

- [fixed] deprecated rgl functions;
- [fixed] some Roxygen bugs in centre.R;

## Usage

	library(Rpdb)
	# or: devtools::load_all(_path_)
	
	x = read.pdb(file.choose())
	visualize(x, type="p")

Note:
- the default visualize is broken;
- the protein is NOT centered in the bounding box;

<!-- badges: start -->
[![R-CMD-check](https://github.com/discoleo/Rpdb/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/discoleo/Rpdb/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

rpdb - remote debugger based on pdb
==================================

This is essentially a wrapper around pdb that will re-route stdin and stdout to
a socket handler. By default it opens the debugger on port 4444::

    import rpdb; rpdb.set_trace()

But you can change that by simply instantiating Rpdb manually::

    import rpdb
    debugger = rpdb.Rpdb(12345)
    debugger.set_trace()

It is known to work on Jython 2.5, Python 2.5 and Python 3.1. It was written
originally for Jython since this is pretty much the only way to debug it when
running it on Tomcat.

After that your script will "hang" when entering set_trace() and the only
way to get it to continue is to access rpdb using telnet, netcat, etc..::

    nc 127.0.0.1 4444

Installation on CPython (standard Python)
-----------------------------------------

    python setup.py install

Installation in a Tomcat webapp
-------------------------------

Just copy the rpdb directory (the one with the __init__.py file) in your
WEB-INF/lib/Lib folder along with the standard Jython library (required).

Known bugs
----------
  - The socket is now closed properly so you will need to ^C in netcat and ^\
    in telnet to exit after a continue.
  - For some obscure reason, you will need to run continue twice on Jython.

Author(s)
---------
Bertrand Janin <tamentis@neopulsar.org> - http://tamentis.com/

The idea comes from there:

    http://bugs.python.org/issue721464
    http://snippets.dzone.com/posts/show/7248

#' Helper Functions for Rotation of Atomic Coordinates
#' 
#' Rotation of atomic coordinates along a specific Cartesian vector.
#' 
#' These functions are helper functions to perform a rotation around a specific 
#' Cartesian vector. All of them call the \code{R} function.
#' 
#' @return An object of the same class as \code{x} with rotated coordinates.
#' 
#' @param x an R object containing atomic coordinates.
#' @param angle the angle of the rotation in degrees.
#' @param mask a logical vector indicating the set of coordinates to which the
#'   rotation has to be applied.
#' @param crystal an object of class \sQuote{crystal} used to convert fractional
#'   into Cartesian coordinates when needed.
#' @param \dots further arguments passed to or from other methods.
#'   
#' @seealso \code{\link{R}} and \code{\link{xyz2abc}}, \code{\link{abc2xyz}} for
#'   passing from Cartesian to fractional coordinates (or Vis Versa).
#' 
#' @examples 
#' # First lets read a pdb file
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' cell <- cell.coords(x)
#' visualize(x, mode = NULL)
#'
#' # Rotation of the structure around the z-axis
#' visualize(Rz(x, 90), mode = NULL)
#' # Rotation of the residue 1 around the c-axis
#' visualize(Rz(x, 90, mask = x$atoms$resid == 1), mode = NULL)
#' 
#' @keywords manip
#' 
#' @name rotationHelpers
#' @export
Rx <- function(...)
	UseMethod("Rx")

#' @rdname rotationHelpers
#' @export
Rx.coords <- function(x, angle = 0, mask = TRUE, crystal = NULL, ...)
	R(x, angle = angle, x = 1, y = 0, z = 0, mask = mask, crystal = crystal, ...)

#' @rdname rotationHelpers
#' @export
Rx.pdb <- function(x, angle = 0, mask = TRUE, crystal = x$crystal, ...)
	R(x, angle = angle, x = 1, y = 0, z = 0, mask = mask, crystal = crystal, ...)


#' @rdname rotationHelpers
#' @export
Ry <- function(...)
	UseMethod("Ry")

#' @rdname rotationHelpers
#' @export
Ry.coords <- function(x, angle = 0, mask = TRUE, crystal = NULL, ...)
	R(x, angle = angle, x = 0, y = 1, z = 0, mask = mask, crystal = crystal, ...)

#' @rdname rotationHelpers
#' @export
Ry.pdb <- function(x, angle = 0, mask = TRUE, crystal = x$crystal, ...)
	R(x, angle = angle, x = 0, y = 1, z = 0, mask = mask, crystal = crystal, ...)


#' @rdname rotationHelpers
#' @export
Rz <- function(...)
	UseMethod("Rz")

#' @rdname rotationHelpers
#' @export
Rz.coords <- function(x, angle = 0, mask = TRUE, crystal = NULL, ...)
	R(x, angle = angle, x = 0, y = 0, z = 1, mask = mask, crystal = crystal, ...)

#' @rdname rotationHelpers
#' @export
Rz.pdb <- function(x, angle = 0, mask = TRUE, crystal = x$crystal, ...)
	R(x, angle = angle, x = 0, y = 0, z = 1, mask = mask, crystal = crystal, ...)

### Various Helper Functions


### Check Basis
check.basis = function(x) {
	if( ! x %in% c("xyz","abc"))
		stop("'basis' must be equal to 'xyz' or 'abc'");
}

# id or Logical
check.idOrLogical = function(x, max, lbl) {
	if(any(round(x) != x))
		stop(paste0(lbl, " must be a logical or integer vector"));
    if(any(x > max))
      stop(paste0(lbl, " contains indices out of range"));
}

#' Universal Constants
#' 
#' This data set provides various universal constants
#' 
#' @format 
#' A data frame containing for each universal constant the following information.
#' \describe{
#'   \item{\code{Quantity}}{a character vector containing a short description of the constants.}
#'   \item{\code{Value}}{a numeric vector containing the value of the constants.}
#'   \item{\code{Unit}}{a character vector indicating the unit of the constants.}
#' }
#' 
#' @source 
#' http://www.ebyte.it/library/educards/constants/ConstantsOfPhysicsAndMath.html
#' 
#' @examples 
#' # Data for the speed of light
#' universalConstants["c",]
#' 
#' # Return the speed of light in m.s-1
#' universalConstants["c","Value"]
#' 
#' # Return the Planck constant in J.s
#' universalConstants["h","Value"]
#' 
#' @keywords datasets
#' 
#' @name universalConstants
NULL

#' Wrap Atomic Coordinates
#' 
#' Wraps atomic coordinates using periodic boundary conditions.
#' 
#' The \code{wrap} function translates all atoms out of the unit cell back into
#' the unit cell using periodic boundary conditions. To do so, the \code{wrap}
#' function first converts Cartesian into fractional coordinates. Then atoms
#' with fractional coordinates greater than 1 or lower than 0 are respectively
#' translated by -1 or +1. Finally, if the original atomic coordinates were
#' Cartesian coordinates their are reconverted into Cartesian coordinates.
#' 
#' @return Return an object of class \sQuote{pdb} with wrapped atomic
#'   coordinates.
#'   
#' @param x an R object containing atomic coordinates to be wrapped.
#' @param crystal an object of class \sQuote{crystal} containing periodic boundary
#'   conditions used for wrapping.
#' @param factor a factor used to wrap the atoms by groups.
#' @param \dots further arguments passed to or from other methods.
#' 
#' @seealso \code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}},
#' \code{\link{crystal}}, \code{\link{centres.pdb}}, \code{\link{xyz2abc}}
#' 
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' 
#' # Translation of the atoms along x-axis
#' x$atoms$x1 <- x$atoms$x1 + 10
#' 
#' # Wrapping the structure
#' y <- wrap(x)
#' 
#' @keywords manip
#' 
#' @name wrap
#' @export
wrap <- function(x, ...)
  UseMethod("wrap")

#' @rdname wrap
#' @export
wrap.coords <- function(x, crystal = NULL, factor = NULL, ...)
{
	if(is.null(crystal)) stop("Please specify a 'crystal' object")
	if(is.null(factor)) factor = 1:natom(x);
  
	if(! is.coords(x)) stop("'x' must be an object of class 'coords'");
	if(! is.crystal(crystal)) stop("'crystal' must be an object of class 'crystal'");
	
	b = basis(x);
	if(b == "xyz") x = xyz2abc(x, crystal);
	
	centers = centres.coords(x, factor = factor, unsplit = TRUE);
	# TODO: can coords be >= 2 or < -1?
	x[centers > 1] = x[centers > 1] - 1;
	x[centers < 0] = x[centers < 0] + 1;
	
	if(b == "xyz") x = abc2xyz.coords(x, crystal);
	
	return(x);
}

#' @rdname wrap
#' @export
wrap.atoms <- function(x, crystal = NULL, factor = NULL, ...)
{
	if(is.null(crystal)) stop("Please specify a 'crystal' object");
	if(is.null(factor)) factor = x$resid;
	
	if(! is.atoms(x)) stop("'x' must be an object of class 'atoms'");
	if(! is.crystal(crystal)) stop("'crystal' must be an object of class 'crystal'");
	
	coords(x) = wrap.coords(coords(x), crystal, factor);
	
	return(x);
}

#' @rdname wrap
#' @export
wrap.pdb <- function(x, crystal = x$crystal, factor = NULL, ...)
{
	if(is.null(factor)) factor <- x$atoms$resid;
	
	if(! is.pdb(x)) stop("'x' must be an object of class 'pdb'");
	if(! is.crystal(crystal)) stop("'crystal' must be an object of class 'crystal'");
	
	coords(x) = wrap.coords(coords(x), crystal, factor);
	
	return(x);
}

#' Read, Write, Visualize and Manipulate PDB Files
#' 
#' Provides tools to read, write, visualize PDB files, and perform structural manipulations.
#' 
#' This package enables users, e.g. computational chemists,
#' to manipulate molecular structures stored in PDB files.
#' It enables users ro read, write and visualize PDB files.
#' Various basic structural manipulations are also supported.
#' Conversion of Cartesian coordinates into fractional coordinates.
#' Spliting a molecular structure into fragments.
#' Computation of centers-of-geometry and centers-of-mass.
#' Wrapping molecular structure using periodical boundary conditions.
#' Translation, rotation and reflection of atomic coordinates.
#' Calculate atomic bond lengths, angles and dihedrals.
#'
#' @author Julien Idé \email{julien.ide.fr@gmail.com}\cr
#'
#' @references
#' More information on the PDB format can be found here:\cr
#' http://www.wwpdb.org/documentation/format33/v3.3.html
#'
#' @keywords package
#' 
#' @examples
#' ## Read a PDB file included in the package
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' 
#' ## Visualize the PDB file
#' visualize(x, mode = NULL)
#' 
#' ## From Cartesian to fractional coordinates and vice versa
#' x <- xyz2abc(x)
#' basis(x)
#' natom(x, x$atoms$resid)
#' range(x)
#' centres(x)
#' x <- abc2xyz(x)
#' basis(x)
#' natom(x, x$atoms$resid)
#' range(x)
#' centres(x)
#' 
#' ## Split and unsplit
#' F <- x$atoms$resid
#' x <-   split(x, F)
#' x <- unsplit(x, F)
#' 
#' ## Subset and merge
#' x.PCB.only <- subset(x, resname == "PCB")
#' x.DCB.only <- subset(x, resname == "DCB")
#' x <- merge(x.PCB.only, x.DCB.only)
#' 
#' ## Duplicate and wrap
#' x <- replicate(x, a.ind = -1:1, b.ind = -1:1, c.ind = -1:1)
#' x <- wrap(x)
#' 
#' ## Write the 'pdb' object 'x' to a temporary file.
#' write.pdb(x, file = tempfile())
#' 
#' @name Rpdb-package
NULL

#' Add Labels to the \sQuote{rgl} Scene
#' 
#' Add Labels to the current \sQuote{rgl} scene.
#' 
#' \code{addResLab} add residue labels to the scene. If \code{at.centre==TRUE} 
#' only one label per residue is added at the centre of the residue. Otherwise, 
#' residue labels are added at each atomic positions. \code{addEleLab} add 
#' element labels to the scene at each atomic positions. \code{info3d} activate 
#' an interactive mode to add labels by selecting atoms by \bold{right-clicking} 
#' on the current \sQuote{rgl} scene. To escape the interactive mode press the 
#' ESC key. The labels are as follow: "ResidResname:EleidElename"
#' 
#' @return \code{addResLab} and \code{addEleLab} return (using invisible) a
#' two-column data.frame containing the IDs and type indicators of the objects
#' added to the scene.
#' 
#' @param x an R object containing atomic coordinates.
#' @param at.centre a single element logical vector indicating if residue labels
#'   have to be added only at the position of the residue's centre-of-mass 
#'   instead of at each atomic position.
#' @param eleid a single element logical vector indicating if the element ids 
#'   have to be concatenated with the element names to prepare the labels.
#' @param col the colors used to display the labels.
#' @param adj one value specifying the horizontal adjustment, or two, specifying 
#'   horizontal and vertical adjustment respectively. See 
#'   \code{\link[rgl:text3d]{text3d}}
#' @param id vector of ID numbers of \sQuote{rgl} items, as returned by 
#'   \code{rgl.ids}. The vertexes of these items are used to display the labels.
#' @param verbose a logical value specifying if information have to be printed 
#'   to the terminal.
#' @param \dots further arguments passed to or from other methods.
#'   
#' 
#' @seealso \code{\link{pdb}}, \code{\link{visualize}}, \code{\link{measure}}
#'
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' visualize(x, type = "l", mode = NULL)
#' addResLab(x)
#' x <- read.pdb(system.file("examples/Pentacene.pdb", package="Rpdb"))
#' visualize(x, type = "l", mode = NULL)
#' addEleLab(x)
#' 
#' @keywords dynamic
#'       
#' @name addLabels
#' @export
addResLab <- function(x, ...)
  UseMethod("addResLab")

#' @rdname addLabels
#' @export
addResLab.atoms <- function(x, at.centre = TRUE, col = "black", ...){
  if(at.centre){
    labels <- paste0(x$resname, x$resid)
    labels <- labels[!duplicated(labels)]
    xyz <- centres(x)
  }
  else{
    labels <- paste0(x$resname, x$resid)
    xyz <- coords(x)
  }
  txt.ids <- rgl::text3d(xyz, text=labels, col = col, ...)
  invisible(txt.ids)
}

#' @rdname addLabels
#' @export
addResLab.pdb <- function(x, at.centre = TRUE, col = "black", ...)
  addResLab.atoms(x$atoms, at.centre = at.centre, col = col, ...)

#' @rdname addLabels
#' @export
addEleLab <- function(x, ...)
  UseMethod("addEleLab")

#' @rdname addLabels
#' @export
addEleLab.atoms <- function(x, eleid = FALSE, col = "black", ...){
  if(eleid){
    labels <- paste(x$elename, x$eleid, sep=":")
  }
  else{
    labels <- x$elename    
  }
  xyz <- coords(x)
  txt.ids <- rgl::text3d(xyz, text=labels, col = col, ...)
  invisible(txt.ids)
}

#' @rdname addLabels
#' @export
addEleLab.pdb <- function(x, eleid = FALSE, col = "black", ...)
  addEleLab.atoms(x$atoms, eleid = eleid, col = col, ...)

#' @rdname addLabels
#' @export
info3d <- function(...)
  UseMethod("info3d")

#' @rdname addLabels
#' @export
info3d.atoms <- function(x, id = rgl::rgl.ids(), col = "black", verbose = TRUE, adj = 0, ...){
  cat("Presse ESC when you have finish your selections.\n")
  ids <- id[id$type!="text",]$id
  if(basis(x) == "abc")
    stop("'x' must contain Cartesian coordinates")
  xyz <- coords(x, basis = "xyz")
  
  sph.ids <- NULL
  txt.ids <- NULL
  clean <- function(){
    if(!is.null(sph.ids))
      rgl::rgl.pop(id = sph.ids)
    if(!is.null(txt.ids))
      rgl::rgl.pop(id = txt.ids)
  }
  on.exit(clean())
  
  dist <- 0.0015
  sel <- NULL
  while(TRUE){
    # f <- rgl::rgl.select3d(button = "right", ...)
	f <- rgl::select3d(button = "right", ...)
    if(is.null(f)) 
      break
    e <- environment(f)
    rgl.info <- lapply(ids,
                       function(id){
                         verts <- rgl::rgl.attrib(id,"vertices")
                         info <- verts
                         return(info)
                       })
    info <- do.call(rbind, rgl.info)
    verts <- info[,1:3]
    hits <- f(verts)
    if(!any(hits)){
      wincoords <- rgl::rgl.user2window(verts, projection = e$proj)
      hits <- (0 <= wincoords[,3]) && (wincoords[,3] <= 1)
      if(any(hits)){
        dists <- (wincoords[, 1] - e$llx)^2 +
          (wincoords[, 2] - e$lly)^2
        hits <- (dists < dist) & (dists == min(dists))
      }
    }
    if(any(hits)){
      verts <- unique(verts[hits, , drop = FALSE])
      sel <- rbind(sel, verts)
      res.lab <- do.call(paste0, x[,c("resid","resname")])
      ele.lab <- do.call(paste0, x[,c("eleid","elename")])
      labels <- paste(res.lab, ele.lab, sep=":")
      M <-     xyz$x1%in%round(verts[,1],digits=3)
      M <- M & xyz$x2%in%round(verts[,2],digits=3)
      M <- M & xyz$x3%in%round(verts[,3],digits=3)
      if(verbose){
        if(length(which(M))!=0)
          print(subset(x, M))
      }
      if(length(which(M))!=0){
        txt.ids <- c(txt.ids, 
                     rgl::text3d(verts, text=labels[M], col = col, adj = adj, ...))
      }
    }
    else{
      clean()
      sph.ids <- NULL
      txt.ids <- NULL
      sel <- NULL
    }
  }
}

#' @rdname addLabels
#' @export
info3d.pdb <- function(x, id = rgl::rgl.ids(), col = "black", verbose = TRUE, adj = 0, ...)
  info3d.atoms(x$atoms, id = id, col = col, verbose = verbose, adj = adj, ...)

#' Mass of Chemical Elements
#' 
#' Determine the mass of chemical elements
#' 
#' \code{masses} is a generic function to determine the mass of chemical 
#' elements. \cr\cr For objects of class \sQuote{pdb}:
#' \itemize{ \item First the
#' element names are converted into element symbols using the \code{toSymbols} 
#' function. \item Then their masses are taken from the \code{elements} data 
#' set. } \code{NA} values are returned for unrecognized elements.
#' 
#' @return 
#' Return a numeric vector containing the mass of chemical elements.
#' 
#' @param \dots further arguments passed to or from other methods.
#' @param x either a character or an integer vector containing element symbols
#'   or atomic numbers, or an object of class \sQuote{pdb} from which element
#'   symbols are determined (see details).
#'   
#' @seealso 
#' \code{\link{toSymbols}}
#' 
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb",package="Rpdb"))
#' masses(x)
#' 
#' masses(c("C","Cl",NA,"AA","N"))
#' 
#' @keywords manip
#'       
#' @name masses
#' @export
masses <- function(...)
  UseMethod("masses")

#' @rdname masses
#' @export
masses.default <- function(x, ...) {
  if(is.character(x)) return(Rpdb::elements$mass[match(x, Rpdb::elements$symb)])
  else if(is.numeric(x) & x == round(x) ) return(Rpdb::elements$mass[match(x, Rpdb::elements$num)])
  else stop("Bad argument: 'x' must be a character or an integer vector")
}

#' @rdname masses
#' @export
masses.pdb <- function(x, ...) {
  x <- toSymbols(x$atoms$elename)
  return(masses(x))
}

#' Centres-of-Geometry: Rolling Window
#' 
#' Computes the geometric centres using a rolling window on a polypeptide chain
#' 
#' \sQuote{centres.ppRoll} is a generic function to compute the geometric centres
#' using a rolling window on an object containing atomic coordinates for
#' a polypeptide chain. Due to limitations by CRAN check, the function is further
#' referred as \sQuote{centres}.
#'
#' The function may be useful to visualize the overall protein structure,
#' but in a less cluttered way. Unlike the protein backbone, the centres
#' may capture better the bulk of the protein.
#'
#'
#'   
#' @param x an R object containing atomic coordinates (with additional class \code{ppRoll}).
#' @param window size of the rolling window, specified as number of amino-acids;
#' @param chain apply only to the respective chains, by default all chains;
#' @param na.rm a logical value indicating whether NA values should be stripped
#'   before the computation proceeds.
#' @param \dots further arguments passed to or from other methods.
#' 
#' @seealso \code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}}.
#'
#' @return Return an object of class \sQuote{coords} containing the coordinates
#'   of the centres.
#' 
#' @examples
#' ### Example 1: Toll-Like Receptor 2
#' # Download pdb file for TLR2: 2Z81;
#' # Read the pdb-file:
#' # x = read.pdb("pdb2z81.ent")
#' 
#' # Compute the centres:
#' # tmp = centres.ppRoll(x)
#'
#' # Visualize the structure:
#' # visualize(x, type="p", pbc.box = FALSE)
#' # lines3d(tmp, lwd = 5, col = "red")
#'
#' # Add Another "strand":
#' # tmp = centres.ppRoll(x, window = 8)
#' # lines3d(tmp, lwd = 5, col = "#FA3296")


#' @name centres.ppRoll
#' @method centres ppRoll
#' @exportS3Method centres ppRoll
centres.ppRoll <- function(x, window, ...) {
	UseMethod("centres.ppRoll");
}

#' @rdname centres.ppRoll
#' @method centres.ppRoll default
#' @exportS3Method centres.ppRoll default
centres.ppRoll.default = function(x, window = 34, na.rm = TRUE, ...) {
	ids = unique(x$resid[x$elename == "CA"]);
	len = length(ids);
	if(len == 0) {
		xyz = data.frame(x = numeric(0), y = numeric(0), z = numeric(0));
		return(as.coords(xyz));
	}
	if(len <= window) {
		xyz = data.frame(
			x = mean(x$x1, na.rm=na.rm),
			y = mean(x$x2, na.rm=na.rm),
			z = mean(x$x3, na.rm=na.rm));
		return(as.coords(xyz));
	}
	ids = sort(ids);
	maxID = ids[len] - window;
	xyz = lapply(seq(ids[1], maxID, by = 1), function(id) {
		idw = which(ids >= id & ids <= id + window);
		idw = ids[idw];
		isW = x$resid %in% idw;
		xyz = data.frame(
			x = mean(x$x1[isW], na.rm=na.rm),
			y = mean(x$x2[isW], na.rm=na.rm),
			z = mean(x$x3[isW], na.rm=na.rm));
	});
	xyz = do.call(rbind, xyz);
	return(invisible(as.coords(xyz)));
}

#' @rdname centres.ppRoll
#' @method centres.ppRoll atoms
#' @exportS3Method centres.ppRoll atoms
centres.ppRoll.atoms = function(x, window = 34, chain = NULL, na.rm = TRUE, ...) {
	if(is.null(chain)) {
		chain = unique(x$chainid);
	}
	FUN = function(chain) {
		xyz = x[x$chainid == chain, c("x1","x2","x3", "elename", "resid"),
			drop = FALSE];
		xyz = centres.ppRoll.default(xyz, window=window, na.rm=na.rm, ...);
		if(nrow(xyz) > 0) xyz$chainid = chain;
		return(xyz);	
	}
	xyz = lapply(chain, FUN);
	xyz = do.call(rbind, xyz);
	invisible(xyz);
}


#' @rdname centres.ppRoll
#' @method centres.ppRoll pdb
#' @exportS3Method centres.ppRoll pdb
centres.ppRoll.pdb = function(x, window = 34, chain = NULL, na.rm = TRUE, ...) {
	tmp = centres.ppRoll.atoms(x$atoms, window=window, chain=chain,
		na.rm = na.rm, ...);
	if(nrow(tmp) <= 2) return(tmp);
	invisible(tmp);
}

#' Create \sQuote{crystal} Object
#' 
#' Create an object of class \sQuote{crystal} containing
#'   the unit cell parameters and the name of the space group
#'   to associate with an object of class \sQuote{pdb}.
#' 
#' \code{crystal} is a generic function to create objects of class 
#' \sQuote{crystal}. The purpose of this class is to store CRYST1 records from 
#' PDB files which contain the unit cell parameters and the name of the space 
#' group of a molecular system stored in a PDB file.
#' The default method of the \code{crystal} function creates an object
#'   of class \sQuote{crystal} from its different components, i.e.:
#'   \code{abc}, \code{abg} and \code{sgroup}.
#'   At least \code{abc} has to be specified.\cr\cr
#' The S3 method for objects of class \sQuote{pdb} extracts
#'   the \code{crystal} structure from the respective object.\cr\cr
#' \code{is.crystal} tests if an object is of class \sQuote{crystal},
#'   i.e. if it has a \dQuote{class} attribute equal to \code{crystal}.
#' 
#' @return
#' Function \code{crystal} returns a list of class \sQuote{crystal}
#'   with the following components:
#' \item{abc}{a numeric vector of length 3 containing the norms of the lattice
#'   vectors a, b and c.}
#' \item{abg}{a numeric vector of length 3 containing the angles between the
#'   lattice vectors \eqn{\alpha}, \eqn{\beta} and \eqn{\gamma}.}
#' \item{sgroup}{a character string giving the Hermann-Mauguin symbol of the space group.}
#' 
#' Function \code{is.crystal} returns TRUE if \code{x} is an object of class \sQuote{crystal}
#'   and FALSE otherwise.
#' 
#' @param \dots further arguments passed to or from other methods.
#' @param abc a numeric vector of length 3 containing the norms of the lattice
#'   vectors a, b and c.
#' @param abg a numeric vector of length 3 containing the angles between
#'   the lattice vectors \eqn{\alpha}, \eqn{\beta} and \eqn{\gamma}.
#' @param sgroup a character string giving the Hermann-Mauguin symbol of
#'   the space group.
#' @param x an R object to be tested or from whom to extract the \code{crystal} structure.
#'   
#' @seealso  
#' \code{\link{cell.coords}}, \code{\link{pdb}}
#' 
#' @examples 
#' x <- crystal(abc = c(10, 10, 10), abg = c(90,90,90), sgroup = "P1")
#' is.crystal(x)
#'  
#' @keywords classes


#' @name crystal
#' @export
crystal <- function(...)
	UseMethod("crystal");


#' @rdname crystal
#' @export
crystal.pdb = function(x, ...) {
	return(x$crystal);
}

# Helper:
get.PDBCrystal = function(x) {
	if(is.pdb(x)) {
		x = x$crystal;
		if(is.null(x))
			stop("The PDB molecule does not contain crystal information!");
	} else if(! is.crystal(x)) {
		stop("'x' must be an object of class 'crystal'!");
	}
	return(x);
}

#' @rdname crystal
#' @export
crystal.default <- function(abc, abg = c(90, 90, 90), sgroup = "P1", ...)
{
	if(missing(abc)) stop("Please provide at leat 'abc'");
	if(is.crystal(abc)) {
		# TODO: What should be the default action?
		abc = abc$abc;
	}
	#
	to.return <- list(abc = abc, abg = abg, sgroup = sgroup)
	
	class(to.return) = c("crystal");
	return(to.return)
}


#' @rdname crystal
#' @export
is.crystal <- function(x)
{
	isCrystal = inherits(x, c("crystal"));
	return(isCrystal);
}


# Note: not yet exported;
as.crystal.character = function(x) {
	if(length(x) != 1) {
      warning("Multiple 'CRYSTAL' records have been found. Only the first record has been kept.")
      x <- x[1];
    }
    abc <- c(
      a = as.numeric(substr(x,  7, 15)),
      b = as.numeric(substr(x, 16, 24)),
      c = as.numeric(substr(x, 25, 33))
    )
    abg <- c(
      alpha = as.numeric(substr(x, 34, 40)),
      beta  = as.numeric(substr(x, 41, 47)),
      gamma = as.numeric(substr(x, 48, 54))
    )
    sgroup <- substr(x, 56, 66)
    
    if(any(is.na(abc))) warning("In 'crystal': 'abc' contains NA values");
    if(any(is.na(abg))) warning("In 'crystal': 'abg' contains NA values");
    if(sgroup == "") sgroup <- NULL

    crystal <- crystal.default(abc, abg, sgroup);
	return(crystal);
}

#' Visualize a Molecular Structure
#' 
#' Use the rgl library to visualize in 3D a molecular structure.
#' 
#' Three different visualization styles are allowed.
#' \itemize{
#'   \item   When \code{type="p"}: Points are drawn at each atomic positions (very light visualization mode).
#'   \item   When \code{type="l"}: Lines are drawn between bonded atoms. The connectivity of the system has to be specifyed.
#'   \item   When \code{type="s"}: Spheres are drawn at each atomic positions (heavy visualization mode).
#'   
#'   The radii of the spheres are given by \code{radii}.
#'   \itemize{
#'     \item   When \code{radii="rcov"}: Covalent radii, taken from the \code{elements} data set, are used.
#'     \item   When \code{radii="rvdw"}: Van der Waals radii, taken from the \code{elements} data set, are used.
#'     \item   When \code{radii} is a numeric vector: The numeric values are used to assign to each atom a radius. If \code{length(radii) != natom(pdb)} \code{radii} is recycled.
#'   }
#' }
#' When \code{xyz}, \code{abc} or \code{pbc.box} are \code{NULL}, the axis or pbc box are are added depending if a \sQuote{crystal} object can be found.\cr
#' Two different interactive visualization modes are avalable:
#'   \itemize{
#'     \item When \code{mode="measure"}: bond lengths, angles and dihedrals can be measured by \bold{right-clicing} on the atoms.
#'     \item When \code{mode="info"}: atomic labels can be added to the scene by \bold{right-clicing} on the atoms. The labels are as follow: "ResidResname:EleidElename"
#'   }
#' When \code{mode=NULL} the interactive mode is disabled. To escape the interactive mode press the ESC key.
#' 
#' @return 
#' Return (using invisible) a two-column data.frame containing the IDs
#' and type indicators of the objects added to the scene.
#' 
#' @param x an object or the name of a PDB file containing the molecular structure to visualize.
#' @param elename a character vector containing the atomic names used to chose atom colors and radii.
#' @param crystal an object of class \sQuote{crystal}. See \code{\link{crystal}}
#' @param connect an object of class \sQuote{connect}. See \code{\link{connect}}
#' @param mode a single element character vector indicating the visualization mode (See details).
#' @param type a character string indicating the visualization style (See details).
#' @param xyz a logical value indicating whether the x, y and z axes have to be added to the scene. See details.
#' @param abc a logical value indicating whether the a, b and c axes have to be added to the scene. See details.
#' @param pbc.box a logical value indicating whether the pbc box has to be added to the scene. See details.
#' @param alpha.box a logical value specifying the transparency of the box-lines;
#' @param lwd a numeric value indication the line width used to plot the axes, the pbc box and atomic bonds when \code{type = "l"} (see details).
#' @param lwd.xyz a numeric value indicating the line width used to plot the x, y and z axes.
#' @param lwd.abc a numeric value indicating the line width used to plot the a, b and c axes.
#' @param lwd.pbc.box a numeric value indicating the line width used to plot the pbc box.
#' @param cex.xyz a numeric value indicating the magnification used to plot the labels of the x, y and z axes.
#' @param cex.abc a numeric value indicating the magnification used to plot the labels of the a, b and c axes.
#' @param col a vector indicating the colors to use to plot each atom.
#' @param bg the color of the background
#' @param radii either a character string indicating the type of radii or a numeric vector specifying the radii of each atom to use to plot atoms as spheres (see details).
#' @param scale.atoms scalar value by which to scale the radii.
#' @param add a logical value indicating whether the plot has be to added to a existing scene (see \code{rgl.cur} and \code{open3d}).
#' @param windowRect a vector of four integers indicating the left, top, right and bottom of the displayed window in pixels (see \code{par3d}).
#' @param FOV the field of view. This controls the degree of parallax in the perspective view (see par3d).
#' @param userMatrix a 4 by 4 matrix describing user actions to display the scene (see \code{par3d}).
#' @param scale.box scalar value or numeric vector used to scale the PBC box.
#' @param scale.axis scalar value or numeric vector used to scale the axis-vectors.
#' @param \dots further arguments passed to or from other methods.
#' 
#' @seealso \code{\link{addXYZ}}, \code{\link{addABC}}, \code{\link{addPBCBox}}, \code{par3d}, \code{select3d}, \code{measure}, \code{info3d}
#' 
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' visualize(x, type = "l", mode = NULL)
#' visualize(x, type = "s", radii = "rcov", mode = NULL)
#' visualize(x, type = "s", radii = "rvdw", mode = NULL)
#' visualize(x, type = "p", mode = NULL)
#' visualize(subset(x, resid != 1), type = "l", mode = NULL)
#' visualize(subset(x, resid == 1), type = "s", add = TRUE, mode = NULL)
#' 
#' ### Protein BackBone + Surface
#' # x = some protein (1 chain);
#' # visualize(x, type="l", lwd=4)
#' # visualize(x, type="s", alpha = 0.05, add = TRUE)

#' @keywords dynamic
#' 

#' @name visualize
windowRect0 = function() c(8,24,800,600);

#' @name visualize
#' @export
visualize <- function(...)
  UseMethod("visualize")

#' @rdname visualize
#' @export
visualize.coords <- function(x, elename = NULL, crystal = NULL, connect = NULL, mode = NULL,
		type = "l", xyz = NULL, abc = NULL, pbc.box = NULL,
		lwd = 2, lwd.xyz = lwd, lwd.abc = lwd, lwd.pbc.box = lwd,
		cex.xyz = 2, cex.abc = 2, col = NULL, bg = "#FAFAD2",
		radii = "rvdw", scale.atoms = 1,
		add = FALSE, windowRect = c(0,0,800,600), FOV = 0,
		userMatrix = diag(4), scale.box = 1, scale.axis = scale.box,
		alpha.box = 0.25, ...) {
	# Checks:
	if(! is.coords(x)) stop("'x' must be an object of class 'coords'.")
	
	if(basis(x) == "abc") x = abc2xyz(x, crystal);
	
	# Unrecognized elements are considered as dummy atoms
	if(is.null(elename)) {
		warning("'elename' was not specified.",
			"All atom have been considered as dummy atoms.")
		elename = rep("Xx", natom(x));
	}
  
  # symb <- toSymbols(elename, na = "Xx");
  # symb[is.na(symb)] <- "Xx"
  # M <- match(symb, Rpdb::elements[, "symb"])
  M = match.element.character(elename, na = 1);
  
  # RGB:
  if(is.null(col)){
    col <- Rpdb::elements[M, c("red","green","blue")]
    col <- do.call(rgb, col)
  }
  if(length(col) != natom(x)) col <- rep(col, length = natom(x))
  
  # RGL:
  if(!add){
    rgl::open3d()
    rgl::par3d(windowRect = windowRect, userMatrix=userMatrix, FOV = FOV, ...)
    rgl::bg3d(color=bg)
  }
  ids <- rgl::ids3d()
  par.save <- rgl::par3d(skipRedraw=TRUE)
	
	### Axis & PBC-Box
	# Auto-selection:
	# TODO: Refactor
	if(is.null(xyz)) {
		if(is.null(crystal))
			xyz = TRUE
		else
			xyz = FALSE
	} # ELSE IF(! LOGICAL) ...
  
	if(is.null(abc)) {
		if(is.null(crystal))
			abc <- FALSE
		else
			abc <- TRUE
	} else if(! is.logical(abc)) {
		warning("Invalid abc-parameter!");
		abc = FALSE;
	}
	
	if(is.null(pbc.box)) {
		if(is.null(crystal))
			pbc.box <- FALSE
		else
			pbc.box <- TRUE
	} else if(! is.logical(pbc.box)) {
		warning("Invalid abc-parameter!");
		pbc.box = FALSE;
	}
  
  if(abc & is.null(crystal)) {
    warning("Cannot find periodical boundary conditions")
    abc <- FALSE
  }
  if(pbc.box & is.null(crystal)) {
    warning("Cannot find periodical boundary conditions")
    pbc.box <- FALSE
  }
  
	if(xyz) ids = rbind(ids, addXYZ(lwd = lwd.xyz,
		scale = scale.axis, cex = cex.xyz));
	if(abc) ids = rbind(ids, addABC(crystal, lwd = lwd.abc,
		scale = scale.axis, cex = cex.abc));
	if(pbc.box) ids = rbind(ids, addPBCBox(crystal, lwd = lwd.pbc.box,
		scale = scale.box, alpha = alpha.box));
  
  ### Molecule
  if(type == "l") {
		if(is.null(connect)) {
			warning("Undefined connectivity: Computing connectivity from coordinates...")
			connect = connect(x);
		}
		ind = t(connect);
		seg.id = rgl::segments3d(
			x$x1[ind],
			x$x2[ind],
			x$x3[ind],
			color = col[ind], lwd=lwd, ...
		)
		# TODO: store separate ids for each component/chain of the molecule;
		# => pop3d(id = id...);
		seg.id = data.frame(id = seg.id, type = "atom.seg")
		ids    = rbind(ids, seg.id)
  } else if(type == "p") {
    pts.id <- rgl::points3d(
      x$x1,
      x$x2,
      x$x3,
      color = col, ...)
    pts.id <- data.frame(id = pts.id, type = "atom.pts")
    ids <- rbind(ids, pts.id)
  }
  
  if(type == "s"){
    if(is.character(radii[1])){
      if(! radii[1] %in% c("rcov", "rbo", "rvdw") )
        stop("'radii' must be one of 'rcov', 'rbo', 'rvdw' or a numerical vector")
      radii = Rpdb::elements[M, radii[1]];
	  radii = radii * scale.atoms;
    }
    if(all(radii == 0)){
      warning("All atoms are dummy atoms. 'radii' have been set to 1")
      radii <- rep(1, natom(x))
    }
    sph.id <- rgl::spheres3d(
      x$x1,
      x$x2,
      x$x3,
      color = col, radius = radii, ...)
    sph.id <- data.frame(id = sph.id, type = "atom.sph")
    ids <- rbind(ids, sph.id)
  }

  rgl::par3d(par.save)
  if(!is.null(mode)){
    if(mode == "measure"){
      measure(x)
    }
    else if(mode == "info"){
      stop("No 'info' mode for object of class 'coords'")
    }
    else{
      stop("Unrecognized visualization mode")
    }
  }
  
  invisible(ids)
}

#' @rdname visualize
#' @export
visualize.data.frame <- function(x, elename = NULL, crystal = NULL, connect = NULL, mode = NULL,
		type = "l", xyz = NULL, abc = NULL, pbc.box = NULL,
		lwd = 2, lwd.xyz = lwd, lwd.abc = lwd, lwd.pbc.box = lwd,
		cex.xyz = 2, cex.abc = 2, col = NULL, bg = "#FAFAD2", 
		radii = "rvdw", add = FALSE, windowRect = c(0,0,800,600), FOV = 0,
		userMatrix = diag(4), ...) {
  
  if(is.null(basis(x))){
    basis(x) <- "xyz"
    warning("No basis attribute were found. Coordinates are assumed to Cartesian.")
  }
  
  visualize(coords(x), elename, crystal, connect, mode, type,
            xyz, abc, pbc.box, lwd, lwd.xyz, lwd.abc, lwd.pbc.box,
            cex.xyz, cex.abc, col, bg,  radii, add, windowRect, FOV, userMatrix, ...)
}

#' @rdname visualize
#' @export
visualize.matrix <- function(x, elename = NULL, crystal = NULL, connect = NULL,
		mode = NULL, type = "l", xyz = NULL, abc = NULL, pbc.box = NULL,
		lwd = 2, lwd.xyz = lwd, lwd.abc = lwd, lwd.pbc.box = lwd,
		cex.xyz = 2, cex.abc = 2, col = NULL, bg = "#FAFAD2",
		radii = "rvdw", add = FALSE, windowRect = c(0,0,800,600), FOV = 0,
		userMatrix = diag(4), ...){
  
  if(is.null(basis(x))){
    basis(x) <- "xyz"
    warning("No basis attribute were found. Coordinates are assumed to Cartesian.")
  }
  
  visualize(coords(x), elename, crystal, connect, mode, type,
            xyz, abc, pbc.box, lwd, lwd.xyz, lwd.abc, lwd.pbc.box,
            cex.xyz, cex.abc, col, bg,
			radii, add, windowRect, FOV, userMatrix, ...)
}

#' @rdname visualize
#' @export
visualize.atoms <- function(x, crystal = NULL, connect = NULL,
		mode = NULL, type = "l",
		xyz = NULL, abc = NULL, pbc.box = NULL,
		lwd = 2, lwd.xyz = lwd, lwd.abc = lwd, lwd.pbc.box = lwd,
		cex.xyz = 2, cex.abc = 2, col = NULL, bg = "#FAFAD2", 
		radii = "rvdw", scale.atoms = 1,
		add = FALSE, windowRect = NULL, FOV = 0,
		userMatrix = diag(4), alpha.box = 0.25, ...) {
	if(is.null(windowRect)) windowRect = windowRect0();
	# Atomic Symbols:
	atoms = x$symbol;
	if(is.null(atoms)) {
		atoms = x$elename;
	} else {
		attr(atoms, "isSymbol") = TRUE;
	}
	ids = visualize(coords(x), atoms, crystal, connect, mode=NULL, type,
            xyz, abc, pbc.box, lwd, lwd.xyz, lwd.abc, lwd.pbc.box,
            cex.xyz, cex.abc, col, bg, radii, scale.atoms = scale.atoms,
			add, windowRect, FOV, userMatrix, alpha.box = 0.25, ...)
  
  if(!is.null(mode)){
    if(mode == "measure"){
      measure(x)
    }
    else if(mode == "info"){
      info3d(x)
    }
    else{
      stop("Unrecognized visualization mode");
    }
  }
  
  invisible(ids)
}

#' @rdname visualize
#' @export
visualize.pdb <- function(x, mode = NULL, type = "l",
		xyz = NULL, abc = NULL, pbc.box = NULL,
		lwd = 2, lwd.xyz = lwd, lwd.abc = lwd, lwd.pbc.box = lwd,
		cex.xyz = 2, cex.abc = 2, col = NULL, bg = "#FAFAD2",
		radii = "rvdw", scale.atoms = 1,
		add = FALSE, windowRect = NULL, FOV = 0,
		userMatrix = diag(4), ...) {
	if(is.null(windowRect)) windowRect = windowRect0();
	### Proteins:
	connect = x$connect;
	isPr = isProtein(x);
	if(isPr) {
		bb = asBackbone(x);
		connect = rbind(connect, bb);
		connect = connect.default(connect);
	}
  # calls visualize.atoms;
  ids <- visualize(x$atoms, crystal = x$crystal, connect = connect, mode=NULL, type,
            xyz, abc, pbc.box,
			lwd, lwd.xyz, lwd.abc, lwd.pbc.box,
            cex.xyz, cex.abc, col, bg, radii, scale.atoms = scale.atoms,
			add, windowRect, FOV, userMatrix, alpha.box = 0.25, ...)
  
  if(!is.null(mode)){
    if(mode == "measure"){
      measure(x)
    }
    else if(mode == "info"){
      info3d(x)
    }
    else{
      stop("Unrecognized visualization mode")
    }
  }
  
  invisible(ids)
}

#' @rdname visualize
#' @export
visualize.character <- function(x, mode = NULL, type = "l", xyz = NULL, abc = NULL,
		pbc.box = NULL, lwd = 2, lwd.xyz = lwd, lwd.abc = lwd, lwd.pbc.box = lwd,
		cex.xyz = 2, cex.abc = 2, col = NULL, bg = "#FAFAD2",  radii = "rvdw",
		add = FALSE, windowRect = NULL, FOV = 0,
		userMatrix = diag(4), ...) {
	if(is.null(windowRect)) windowRect = windowRect0();
	x = read.pdb(x);
	visualize.pdb(x, mode = mode, type = type,
		xyz = xyz, abc = abc, pbc.box = pbc.box,
		lwd = lwd, lwd.xyz = lwd.xyz, lwd.abc = lwd.abc, lwd.pbc.box = lwd.pbc.box,
		cex.xyz = cex.xyz, cex.abc = cex.abc,
		col = col, bg = bg, radii = radii,
		add = add, windowRect = windowRect, FOV = FOV, userMatrix = userMatrix, ...);
}

#' Reassemble Groups
#' 
#' \code{unsplit} reverses the effect of \code{split}.
#' 
#' \code{unsplit} is a generic functions with a default method (Method dispatch 
#' takes place based on the class of the first element of \code{value}) working 
#' with lists of vectors or data frames (assumed to have compatible structure, 
#' as if created by \code{split}). It puts elements or rows back in the 
#' positions given by \code{f}. In the data frame case, row names are obtained 
#' by unsplitting the row name vectors from the elements of \code{value}. \cr 
#' \code{f} is recycled as necessary and if the length of \code{x} is not a 
#' multiple of the length of \code{f} a warning is printed. \cr Any missing 
#' values in \code{f} are dropped together with the corresponding values of 
#' \code{x}.
#' 
#' @return Returns a vector or data frame for which \code{split(x, f)} equals
#'   \code{value}
#'   
#' @param value a list of vectors or data frames compatible with a splitting of
#'   \code{x}. Recycling applies if the lengths do not match.
#' @param f a \sQuote{factor} in the sense that \code{\link{as.factor}(f)}
#'   defines the grouping, or a list of such factors in which case their
#'   interaction is used for the grouping.
#' @param drop logical indicating if levels that do not occur should be dropped
#'   (if \code{f} is a \code{factor} or a list).
#' @param \dots further potential arguments passed to methods.
#' 
#' @seealso 
#' \code{\link[base]{cut}} to categorize numeric values.
#' \cr
#' \code{\link[base]{strsplit}} to split strings.
#' 
#' @references
#' Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988) \emph{The New S Language}. Wadsworth & Brooks/Cole.
#' 
#' @examples
#' require(stats); require(graphics)
#' n <- 10; nn <- 100
#' g <- factor(round(n * runif(n * nn)))
#' x <- rnorm(n * nn) + sqrt(as.numeric(g))
#' xg <- split(x, g)
#' boxplot(xg, col = "lavender", notch = TRUE, varwidth = TRUE)
#' sapply(xg, length)
#' sapply(xg, mean)
#' 
#' ### Calculate 'z-scores' by group (standardize to mean zero, variance one)
#' z <- unsplit(lapply(split(x, g), scale), g)
#' 
#' # or
#' 
#' zz <- x
#' split(zz, g) <- lapply(split(x, g), scale)
#' 
#' # and check that the within-group std dev is indeed one
#' tapply(z, g, sd)
#' tapply(zz, g, sd)
#' 
#' 
#' ### Data frame variation
#' 
#' # Notice that assignment form is not used since a variable is being added
#' 
#' g <- airquality$Month
#' l <- split(airquality, g)
#' l <- lapply(l, transform, Oz.Z = scale(Ozone))
#' aq2 <- unsplit(l, g)
#' head(aq2)
#' with(aq2, tapply(Oz.Z,  Month, sd, na.rm=TRUE))
#' 
#' 
#' ### Split a matrix into a list by columns
#' ma <- cbind(x = 1:10, y = (-4:5)^2)
#' split(ma, col(ma))
#' 
#' split(1:10, 1:2)
#'
#' @keywords category
#' 
#' @name unsplit
#' @export        
unsplit <- function(value, f, drop = FALSE, ...)
  UseMethod("unsplit", value[[1]])

#' @rdname unsplit
#' @export
unsplit.default <- function(value, f, drop = FALSE, ...)
{
  len <- length(if (is.list(f)) f[[1L]] else f)
  if (is.data.frame(value[[1L]])) {
    x <- value[[1L]][rep(NA, len), , drop = FALSE]
    rownames(x) <- unsplit(lapply(value, rownames), f, drop = drop)
  }
  else x <- value[[1L]][rep(NA, len)]
  split(x, f, drop = drop) <- value
  x
}

#' Centres-of-Geometry and Centres-of-Mass
#' 
#' Computes centres-of-geometry and centres-of-mass of groups of atoms.
#' 
#' \code{centres} is a generic function to compute centres-of-geometry and 
#' centres-of-mass from an object containing atomic coordinates. For objects of 
#' class \sQuote{coords}, \sQuote{atoms} and \sQuote{pdb}, the coordinates of 
#' \code{x} are first splitted into groups defined by \code{factor} using the 
#' \code{\link{split}} function. For each group, the weighted mean of the 
#' \code{x1}, \code{x2} and \code{x3} components of \code{x} are calculated 
#' using \code{weights}. By default all atoms are assumed to have the same 
#' weight (calculation of centres-of-geometry). Finally, if \code{unplit = TRUE}
#' the coordinates of the centres are unsplitted using the \code{\link{unsplit}}
#' function to assign to each atom the coordinates of the centre to which they 
#' are attached (used for wrapping by groups). \cr \cr For objects of class 
#' \sQuote{atoms} and \sQuote{pdb} by default \code{factor} is set to 
#' \code{x$resid} and \code{x$coordinates$resid}, respectively, to compute the 
#' centre-of-geometry of the different resdiues. Notice that coordinates can be 
#' neglected for the calculation of the centres using NA values in 
#' \code{factor}.
#' 
#' @return Return an object of class \sQuote{coords} containing the coordinates
#'   of centres.
#'   
#' @param x an R object containing atomic coordinates.
#' @param factor a factor used to split the atomic coordinates by groups to
#'   compute multiple centres.
#' @param weights a numerical vector containing atomic weights used to compute
#'   centres-of-mass.
#' @param unsplit a logical value indicating whether the coordinates of the
#'   centres have to be unsplit to repeat their coordinates for each atom used
#'   for their calculation (used for wrapping by groups).
#' @param na.rm a logical value indicating whether NA values should be stripped
#'   before the computation proceeds.
#' @param \dots further arguments passed to or from other methods.
#' 
#' @seealso \code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}},
#' \code{\link{elements}} \cr\cr and \code{\link{split}}, \code{\link{unsplit}},
#' \code{\link[base]{factor}} for details about splitting data sets.
#' 
#' @examples
#' # First lets read a pdb file
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' 
#' # Centres-of-geometry of the residues
#' centres(x)
#' 
#' # Centre-of-geometry of the whole structure
#' centres(x, factor = rep(1, natom(x)))
#' # or
#' centres(coords(x))
#' 
#' # Centres-of-geometry of the PCB and DCB residues
#' centres(x, factor = x$atoms$resname)
#' 
#' # Knowing the name of the elements forming
#' # the C60 of the PCBM molecules (PCB residues)
#' # we can compute the centres-of-geometry of
#' # the C60 by neglecting the other atoms of the
#' # PCB residues.
#' C60.elename <- paste0("C", sprintf("%0.3d", 1:60))
#' 
#' is.PCB <- x$atoms$resname == "PCB" # Produce a mask to select only the PCB residues
#' is.C60 <- is.PCB & x$atoms$elename %in% C60.elename # Produce a mask to keep only the C60
#' 
#' F <- x$atoms$resid # We use the residue IDs to split the coordinates
#' F[!is.C60] <- NA # We keep only the atoms of the C60
#' 
#' C60.centres <- centres(x, factor = F)
#' 
#' # Lets check the position of the C60 centres
#' visualize(x , mode = NULL)
#' spheres3d(C60.centres)
#' text3d(Ty(C60.centres, 2), text=paste0("PCB_", rownames(C60.centres)), cex=2)
#' 
#' # Centres-of-mass of the resdiues
#' symb <- toSymbols(x$atoms$elename) # Convert elename into elemental symbols
#' # Find the mass of the element in the periodic table
#' w <- elements[match(symb, elements[,"symb"]), "mass"] 
#' centres(x, weights = w)
#' 
#' @keywords manip 
#' 
#' @name centres
#' @export
centres <- function(x, ...)
  UseMethod("centres")

#' @rdname centres
#' @export
centres.coords <- function(x, factor = NULL, weights = NULL, unsplit = FALSE, na.rm = FALSE, ...)
{
  if(!is.coords(x)) stop("'x' must be an object of class 'coords'")

  if(is.null(factor )) factor  <- rep("", natom(x))
  if(is.null(weights)) weights <- rep(1 , natom(x))

  w  <- split(weights, factor)
  x1 <- split(x$x1   , factor)
  x2 <- split(x$x2   , factor)
  x3 <- split(x$x3   , factor)
  
  w.mean <- function(x, w)
    sum(x*w/sum(w, na.rm = na.rm))
  
  x1.mean <- mapply(w.mean, x1, w, SIMPLIFY = FALSE)
  x2.mean <- mapply(w.mean, x2, w, SIMPLIFY = FALSE)
  x3.mean <- mapply(w.mean, x3, w, SIMPLIFY = FALSE)
  
  if(unsplit){
    x1.mean <- unsplit(x1.mean, factor)
    x2.mean <- unsplit(x2.mean, factor)
    x3.mean <- unsplit(x3.mean, factor)
  }
  
  x1.mean <- unlist(x1.mean)
  x2.mean <- unlist(x2.mean)
  x3.mean <- unlist(x3.mean)
  
  to.return <- coords.default(x1.mean, x2.mean, x3.mean, basis(x))
  
  return(to.return)
}

#' @rdname centres
#' @export
centres.atoms <- function(x, factor = NULL, weights = NULL, unsplit = FALSE, na.rm = FALSE, ...)
{
  if(!is.atoms(x)) stop("'x' must be an object of class 'atoms'")
  
  if(is.null(factor)) factor <- x$resid
  
  to.return <- centres.coords(coords(x), factor, weights, unsplit, na.rm)
  return(to.return)
}

#' @rdname centres
#' @export
centres.pdb <- function(x, factor = NULL, weights = NULL, unsplit = FALSE, na.rm = FALSE, ...)
{
  if(!is.pdb(x)) stop("'x' must be an object of class 'pdb'")
  
  to.return <- centres.atoms(x$atoms, factor, weights, unsplit, na.rm)
  return(to.return)
}

#' Translation of Atomic Coordinates
#' 
#' Translation of Cartesian or fractional coordinates.
#' 
#' \code{Txyz} and \code{Tabc} are generic functions. Method for objects of 
#' class \sQuote{coords} first convert the coordinates into Cartesian or 
#' fractional coordinates using \code{crystal} if needed to performed the 
#' translation. Once translated, the coordinates are reconverted back to the 
#' original basis set using again \code{crystal}. Method for objects of class 
#' \sQuote{pdb} first extract coordinates from the object using the function 
#' \code{coords}, perform the translation, and update the coordinates of the 
#' \sQuote{pdb} object using the function \code{coords<-}. The \code{thickness} 
#' argument can be use to translate selected atoms by a fraction of its 
#' thickness along the translation direction. This can be use when merging two 
#' fragments centered at the origin to build a dimer to avoid atomic overlap and
#' set the inter-fragment distance (see examples).
#' 
#' @return An object of the same class as \code{x} with translated coordinates.
#'   
#' @param obj an R object containing atomic coordinates.
#' @param x the x-component of the translation vector.
#' @param y the y-component of the translation vector.
#' @param z the z-component of the translation vector.
#' @param a the a-component of the translation vector.
#' @param b the b-component of the translation vector.
#' @param c the c-component of the translation vector.
#' @param mask a logical vector indicating the set of coordinates to which to
#'   apply the translation.
#' @param thickness a numeric value indicating the fraction of the thicknees of
#'   the selected atom to be added to the translation vector (Usually 0, 0.5 or
#'   1. See details).
#' @param crystal an object of class \sQuote{crystal} use to convert Cartesian
#'   into fraction coordinates (or Vis Versa) when need.
#' @param \dots further arguments passed to or from other methods.
#' 
#' @seealso Helper functions for translation along given Cartesian or lattice vector:\cr
#' \code{\link{Tx}}, \code{\link{Ty}}, \code{\link{Tz}}, \code{\link{Ta}}, \code{\link{Tb}}, \code{\link{Tc}}\cr
#' Passing from Cartesian to fractional coordinates (or Vis Versa):\cr
#' \code{\link{xyz2abc}}, \code{\link{abc2xyz}}
#' 
#' @examples 
#' # First lets read a pdb file
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' visualize(x, mode = NULL)
#' visualize(Txyz(x, y=10), mode = NULL)
#' visualize(Txyz(x, y=10,  mask = x$atoms$resid==1), mode = NULL)
#' visualize(Tabc(x, b= 1), mode = NULL)
#' visualize(Tabc(x, b= 1,  mask = x$atoms$resid==1), mode = NULL)
#' 
#' # Lets build a C70/Pentacene dimer with an inter-molecular distance equal to 3.5
#' C70 <- read.pdb(system.file("examples/C70.pdb", package="Rpdb"))
#' Pen <- read.pdb(system.file("examples/Pentacene.pdb", package="Rpdb"))
#' x <- merge(C70, Pen)
#' visualize(x, mode = NULL)
#' viewXY()
#' visualize(Txyz(x, x=0, y=0, z=3.5, mask = x$atoms$resname == "C70", thickness=0.5), mode = NULL)
#' viewXY()
#' 
#' @keywords manip
#' 
#' @name translation
#' @export
Txyz <- function(...)
  UseMethod("Txyz")

#' @rdname translation
#' @export
Txyz.coords <- function(obj, x = 0, y = 0, z = 0, mask = TRUE,
		thickness = NULL, crystal = NULL, ...) {
  if(!is.coords(obj)) stop("'object' must be an obj of class 'coords'")
  
  if(length(mask) != natom(obj)){
    if(length(mask) != 1)
      warning("'mask' has been recycled")
    mask <- rep(mask, length = natom(obj))
  }
  
  v <- coords(x,y,z, basis = "xyz")
  T <- coords(0,0,0, basis = "xyz")
  if(basis(obj) != "xyz"){
    if(is.null(crystal))
      stop("Please specify a 'crystal' obj to convert the fractional coordinates into Cartesian")
    v <- xyz2abc(v, crystal = crystal)
    T <- xyz2abc(T, crystal = crystal)
  }

  vn <- coords(0,0,0, basis = "xyz")
  if(sqrt(sum(v^2)) != 0) vn <- v/sqrt(sum(v^2))
  
  if(!is.null(thickness)) {
    if(length(thickness) != 1) stop("'thickness must be a single element numeric vector'")
    T <- as.matrix(obj[mask,])%*%t(vn)
    T <- diff(range(T))*vn*thickness
  }
  
  obj$x1[mask] <- obj$x1[mask] + v$x1 + T$x1
  obj$x2[mask] <- obj$x2[mask] + v$x2 + T$x2
  obj$x3[mask] <- obj$x3[mask] + v$x3 + T$x3

  return(obj)
}

#' @rdname translation
#' @export
Txyz.pdb <- function(obj, x = 0, y = 0, z = 0, mask = TRUE,
		thickness = NULL, crystal = obj$crystal, ...) {
	if(! is.pdb(obj)) stop("'obj' must be an object of class 'pdb'!");
	
	coords(obj) <- Txyz(coords(obj), x = x, y = y, z = z,
		mask = mask, thickness = thickness, crystal = crystal, ...);
	
	return(obj);
}

#' @rdname translation
#' @export
Tabc <- function(...)
  UseMethod("Tabc")

#' @rdname translation
#' @export
Tabc.coords <- function(obj, a = 0, b = 0, c = 0, mask = TRUE,
		thickness = NULL, crystal = NULL, ...){  
  if(!is.coords(obj)) stop("'object' must be an obj of class 'coords'")

  if(length(mask) != natom(obj)){
    if(length(mask) != 1)
      warning("'mask' has been recycled")
    mask <- rep(mask, length = natom(obj))
  }
  
  v <- coords(a,b,c, basis = "abc")
  T <- coords(0,0,0, basis = "abc")
	if(basis(obj) != "abc"){
		if(is.null(crystal))
			stop("Please specify a 'crystal' obj to convert the Cartesian into fractional coordinates");
		v = abc2xyz(v, crystal = crystal);
		T = abc2xyz(T, crystal = crystal);
	}

  vn <- coords(0,0,0, basis = "abc")
  if(sqrt(sum(v^2)) != 0) vn <- v/sqrt(sum(v^2))

  if(!is.null(thickness)) {
    if(length(thickness) != 1) stop("'thickness must be a single element numeric vector'")
    T <- as.matrix(obj[mask,])%*%t(vn)
    T <- diff(range(T))*vn*thickness
  }
  
  obj$x1[mask] <- obj$x1[mask] + v$x1 + T$x1
  obj$x2[mask] <- obj$x2[mask] + v$x2 + T$x2
  obj$x3[mask] <- obj$x3[mask] + v$x3 + T$x3
  
  return(obj)
}

#' @rdname translation
#' @export
Tabc.pdb <- function(obj, a = 0, b = 0, c = 0, mask = TRUE,
		thickness = NULL, crystal = obj$crystal, ...) {
	if(! is.pdb(obj)) stop("'object' must be an obj of class 'pdb'");
	
	coords(obj) = Tabc(coords(obj), a = a, b = b, c = c,
		mask = mask, thickness = thickness, crystal = crystal, ...);
	
	return(obj)
}

#' Rotation of Atomic Coordinates
#' 
#' Rotation of atomic coordinates around a given vector.
#' 
#' \code{R} is a generic function. Method for objects of class \sQuote{coords}
#' first convert the coordinates into Cartesian coordinates using \code{crystal}
#' if needed. Once rotated, the coordinates are reconverted back to the original
#' basis set using again \code{crystal}. Method for objects of class \sQuote{pdb}
#' first extracts coordinates from the object using the function \code{coords},
#' performs the rotation, and updates the coordinates of the \sQuote{pdb} object
#' using the function \code{coords<-}.
#' 
#' @return An object of the same class as \code{x} with rotated coordinates.
#' 
#' @param obj an R object containing atomic coordinates.
#' @param angle the angle of the rotation in degrees.
#' @param x the x-component of the rotation vector.
#' @param y the y-component of the rotation vector.
#' @param z the z-component of the rotation vector.
#' @param mask a logical vector indicating the set of coordinates to which the rotation has to be applyed.
#' @param crystal an object of class \sQuote{crystal} used to convert fractional into Cartesian coordinates (when needed).
#' @param \dots further arguments passed to or from other methods.
#' 
#' @seealso 
#' Helper functions for rotation around a given Cartesian vector:\cr
#' \code{\link{Rx}}, \code{\link{Ry}}, \code{\link{Rz}}\cr
#' Passing from Cartesian to fractional coordinates (or Vis Versa):\cr
#' \code{\link{xyz2abc}}, \code{\link{abc2xyz}}
#' 
#' @examples 
#' # First lets read a pdb file
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' cell <- cell.coords(x)
#' visualize(x, mode = NULL)
#'
#' # Rotation of the structure around the c-axis
#' visualize(R(x, 90, x=cell["x","c"], y=cell["y","c"], z=cell["z","c"]),
#'           mode = NULL)
#' # Rotation of the residue 1 around the c-axis
#' visualize(R(x, 90, x=cell["x","c"], y=cell["y","c"], z=cell["z","c"], mask = x$atoms$resid == 1),
#'           mode = NULL)
#'           
#' @keywords manip
#' 
#' @name rotation
#' @export
R <- function(...)
  UseMethod("R")

#' @rdname rotation
#' @export
R.coords <- function(obj, angle = 0, x = 0, y = 0, z = 1,
		mask = TRUE, crystal = NULL, ...) {
	if(! is.coords(obj)) stop("'obj' must be an object of class 'coords'");

  if(length(mask) != natom(obj)){
    if(length(mask) != 1)
      warning("'mask' has been recycled")
    mask <- rep(mask, length = natom(obj))
  }
  
  basis.ori <- basis(obj)
  if(basis.ori != "xyz"){
    if(is.null(crystal))
      stop("Please specify a 'crystal' obj to convert the fractional coordinates into Cartesian")
    obj <- abc2xyz(obj, crystal = crystal);
  }
  M <- rgl::rotationMatrix(angle=angle*pi/180, x = x, y = y, z = z)[1:3,1:3]
  obj[mask,] <- coords(as.matrix(obj[mask,])%*%M, basis = "xyz")
	if(basis.ori != "xyz")
		obj = xyz2abc(obj, crystal = crystal);
	return(obj)
}

#' @rdname rotation
#' @export
R.pdb <- function(obj, angle = 0, x = 0, y = 0, z = 1, mask = TRUE,
		crystal = obj$crystal, ...) {
	if(! is.pdb(obj)) stop("'obj' must be an object of class 'pdb'");
	
	coords(obj) = R(coords(obj), angle = angle, x = x, y = y, z = z,
		mask = mask, crystal = crystal, ...);
	
	return(obj)
}

#' Replicate Atomic Coordinates
#' 
#' Replicate atomic coordinates using periodic boundary conditions.
#' 
#' The \code{replicate} function replicate a unit cell along the lattice vectors a, b and c
#' as as many times as indicated by the \code{a.ind}, \code{b.ind} and \code{c.ind} arguments.
#' Discontinuous integer vectors can be used for \code{a.ind}, \code{b.ind} and \code{c.ind}
#' to create layered supercells (See examples).
#' 
#' @return Return an object of class \sQuote{pdb} with replicated atomic coordinates.
#' 
#' @param x an R object containing atomic coordinates to be replicated.
#' @param crystal an object of class \sQuote{crystal} containing periodical boundary conditions used for replicating.
#' @param a.ind a vector of integers indicating the positions of the replicated cells along the a-axis.
#' @param b.ind a vector of integers indicating the positions of the replicated cells along the b-axis.
#' @param c.ind a vector of integers indicating the positions of the replicated cells along the c-axis.
#' @param \dots further arguments passed to or from other methods.
#' 
#' @seealso \code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}}, \code{\link{crystal}}
#' 
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' 
#' # Create a 3x3 supercell
#' y <- replicate(x, a.ind = 0:2, b.ind = 0:2, c.ind = 0:2)
#' 
#' # Create a 3x3 supercell which might need to be wrapped (if some molecules are outside the cell)
#' y <- replicate(x, a.ind = -1:1, b.ind = -1:1, c.ind = -1:1)
#' 
#' # Create a layered supercell with a vacuum layer in the bc-plan
#' y <- replicate(x, a.ind = c(0,2), b.ind = 0:2, c.ind = 0:2)
#'
#' @keywords manip
#' 
#' @name replicate
#' @export
replicate <- function(x, ...)
  UseMethod("replicate")

#' @rdname replicate
#' @export
replicate.coords <- function(x, crystal = NULL, a.ind = 0, b.ind = 0, c.ind = 0, ...)
{
  if(!is.coords(x)) stop("'x' must be an object of class 'coords'")
  
  a.ind <- unique(a.ind)
  b.ind <- unique(b.ind)
  c.ind <- unique(c.ind)
  
	b = basis(x);
	if(b == "xyz") {
		check.crystal(crystal);
		x = xyz2abc(x, crystal);
	}
  
	abc.ind <- expand.grid(a.ind, b.ind, c.ind);
	x = add.abc(x, abc = abc.ind);
	if(b == "xyz") x = abc2xyz(x, crystal);
	
	return(x);
}

#' @rdname replicate
#' @export
replicate.atoms <- function(x, crystal = NULL, a.ind = 0, b.ind = 0, c.ind = 0, ...)
{
  if(!is.atoms(x)) stop("'x' must be an object of class 'atoms'")
  
  a.ind <- unique(a.ind)
  b.ind <- unique(b.ind)
  c.ind <- unique(c.ind)
  
	basis = basis(x);
	if(basis == "xyz") {
		check.crystal(crystal);
		x = xyz2abc(x, crystal);
	}
	
	abc.ind = expand.grid(a.ind, b.ind, c.ind);
	
	nID = max(x$eleid);
	# 1:nrow(abc.ind)-1
	idABC = rep(seq(0, nrow(abc.ind)-1), each=natom(x));
	eleid = rep(x$eleid, nrow(abc.ind)) + idABC * nID;
	resid = rep(x$resid, nrow(abc.ind)) + idABC * max(x$resid);
  
	x = add.abc(x, abc = abc.ind);
	x$resid = resid;
	x$eleid = eleid;
	
	if(basis == "xyz") x = abc2xyz(x, crystal);
	
	return(x);
}

#' @rdname replicate
#' @export
replicate.pdb <- function(x, a.ind = 0, b.ind = 0, c.ind = 0, crystal = NULL, ...)
{
	if(! is.pdb(x)) stop("'x' must be an object of class 'pdb'");
		
	if(is.null(crystal))
		crystal <- x$crystal;
  
  a.ind <- unique(a.ind)
  b.ind <- unique(b.ind)
  c.ind <- unique(c.ind)
  
  na <- length(a.ind)
  nb <- length(b.ind)
  nc <- length(c.ind)
  
  ncell <- na*nb*nc
  
	eleid.1 = x$connect$eleid.1;
	eleid.2 = x$connect$eleid.2;
	nID = max(x$atoms$eleid);
	eleid.1 = rep(eleid.1, ncell) + rep(1:ncell-1, each=length(eleid.1))*nID;
	eleid.2 = rep(eleid.2, ncell) + rep(1:ncell-1, each=length(eleid.2))*nID;
	connect = connect.default(eleid.1, eleid.2);
	
	atoms = replicate.atoms(x$atoms, crystal, a.ind, b.ind, c.ind);
	
	# TODO: 1 + c();
	idABC = c(diff(range(a.ind))+1, diff(range(b.ind))+1, diff(range(c.ind))+1);
	cryst = x$crystal;
	abc   = cryst$abc * idABC;
	cryst = crystal(abc = abc, abg = cryst$abg, sgroup = cryst$sgroup);
	
	x = pdb(atoms, cryst, connect);
  
  return(x)
}

check.crystal = function(x) {
	if(is.null(x))      stop("Please specify a 'crystal' object");
	if(! is.crystal(x)) stop("'crystal' must be an object of class 'crystal'");
}

add.abc = function(x, abc) {
	L = apply(abc, 1, function(abc, x) {
			x$x1 = x$x1 + abc[1];
			x$x2 = x$x2 + abc[2];
			x$x3 = x$x3 + abc[3];
			return(x);
		}, x);
	x = do.call(rbind, L);
	return(x);
}

#' Graphics Tools
#' 
#' Tools to modify or set \code{rgl} objects, scenes and the observer.


#' \code{translate} performs a translation of the observer to a new position
#'   specified using relative shifts.

#' \code{trans} is an abbreviated alias to function \code{translate}.

#' @return \code{translate} returns invisibly the old position of the observer.

#' @param x,y,z amount to shift the observer in the x, y and z directions.
#    Negative shifts are to the left, bottom and out of the scene,
#    while positive shifts are to the right, top and into the scene.
#' @param auto If TRUE, the location will be set automatically by RGL
#'   to make the whole bounding box visible.
#'   Unfortunately, it may fail, as it seems there are some bugs in rgl.


### Translate

#' @name translate
#' @export
trans = function(x, y = 0, z = 0, auto = FALSE) {
	old = translate(x, y, z, auto=auto);
}

#' @name translate
#' @export
translate = function(x, y = 0, z = 0, auto = FALSE) {
	old = observer3d();
	observer3d(old + c(x, y, z), auto=auto);
	invisible(old);
}

#' Create an Object of Class \sQuote{pdb}
#' 
#' Creates an object of class 'pdb'.
#' 
#' This function is a generic function used to create objects of class \sQuote{pdb}.
#'   The purpose of this class is to store the data of molecular systems contained in PDB files.
#' The default method of the \code{pdb} function creates an object of class \sQuote{pdb}
#'   from its different components, i.e.:
#'   \code{title}, \code{remark}, \code{crystal}, \code{atoms} and \code{connect}. 
#'   At least an object of class \sQuote{atoms} has to be specified.\cr\cr
#'
#' \code{is.pdb} tests if x is an object of class \sQuote{pdb},
#'   i.e. if x has a \dQuote{class} attribute equal to \code{pdb}.
#' 
#' @return 
#' \code{pdb} returns a list of class \sQuote{pdb} with the following components:
#' \item{title}{a character vector containing the TITLE records found in a PDB file.}
#' \item{remark}{a character vector containing the REMARK records found in a PDB file.}
#' \item{crystal}{a list of class \sQuote{crystal} containing the first CRYSTAL record
#'    found in a PDB file. All others are ignored.}
#' \item{atoms}{a data.frame of class \sQuote{atoms} containing the ATOM and HETATM records
#'    found in a PDB file.}
#' \item{connect}{a data.frame of class \sQuote{connect} containing the CONECT records found in a PDB file.}
#' \cr
#' \code{is.pdb} returns TRUE if x is an object of class \sQuote{pdb} and FALSE otherwise.
#' 
#' @param atoms a data.frame of class \code{atoms} containing ATOM and HETATM records use to create the \code{pdb} object.
#' @param crystal a list of class \code{crystal} containing the periodical boundary conditions and space group used to create the \code{pdb} object.
#' @param connect a data.frame of class \code{connect} containing CONECT records use to create the \code{pdb} object.
#' @param remark a character vector containing some REMARK records to be added to the \code{pdb} object.
#' @param title a character vector containing some TITLE records to be added to the \code{pdb} object.
#' @param resolution numeric value specifying the resolution; the unit should be specified as an attribute.
#' @param x an R object to be tested.
#' @param \dots further arguments passed to or from other methods.
#' @param cryst1 will be deprecated and replaced by argument crystal.
#' 
#' @seealso 
#' \code{\link{atoms}}, \code{\link{coords}}, \code{\link{crystal}}, \code{\link{connect}} and \code{\link{read.pdb}}
#' 
#' @examples 
#' title  <- "This is just an example"
#' remark <- NULL
#' crystal <- crystal(c(10,10,10))
#' atoms <- atoms(recname = c("ATOM","ATOM"), eleid = 1:2, elename = c("H","H"), alt = "",
#'                resname = c("H2","H2"), chainid = "", resid = c(1,1), insert = "",
#'                x1 = c(0,0), x2 = c(0,0), x3 = c(0,1), occ = c(0.0,0.0), temp = c(1.0,1.0),
#'                segid = c("H2","H2"))
#' connect <- connect(eleid.1 = c(1), eleid.2 = c(2))
#' x <- pdb(atoms = atoms, crystal = crystal, connect = connect, remark = remark, title = title)
#' is.pdb(x)
#' 
#' @keywords classes
#' 
#' @name pdb
#' @export
pdb <- function(...)
  UseMethod("pdb")

#' @rdname pdb
#' @export
pdb.default <- function(atoms, crystal = NULL, connect = NULL, remark = NULL, title = NULL,
		resolution = NULL, ..., cryst1 = NULL)
{
	if(missing(atoms)) stop("Please specify at least an 'atoms' object")
	if( ! is.atoms(atoms)) stop("'atoms' must be an object of class 'atoms'")
	
	# Crystal cell:
	crystal = checkArgCrystal(crystal, cryst1);
	if( ! is.null(crystal) & ! is.crystal(crystal))
		stop("'crystal' must be an object of class 'crystal'");
	if( ! is.null(connect) & ! is.connect(connect))
		stop("'connect' must be an object of class 'connect'");
  
  if(is.list(title ) | ! is.null(dim(title ))) stop("'title' must be a vector of character strings")
  if(is.list(remark) | ! is.null(dim(remark))) stop("'remark' must be a vector of character strings")
  
  if(! is.character(title ) & ! is.null(title )) title  <- as.character(title )
  if(! is.character(remark) & ! is.null(remark)) remark <- as.character(remark)
  
	to.return = list(title = title, remark = remark,
		crystal = crystal, atoms = atoms, connect = connect);
	if( ! is.null(resolution)) to.return$Resolution = resolution;
	
	class(to.return) = c("pdb", "list");
	return(to.return);
}

#' @rdname pdb
#' @export
is.pdb <- function(x)
{
  to.return <- any(class(x) == "pdb")
  return(to.return)
}

#' Format "Title" Field of PDB File
#' 
#' Formats the title field of a Protein Data Bank (PDB) file.
#' 
#' This function is mostly used internally, e.g. by \code{\link{write.pdb}},
#' but may be useful in other instances as well.
#'
#' 
#' @param x a vector of strings;
#' @param \dots currently not used;
#' 
#' @return properly formatted text string.
#'
#' @seealso \code{\link{write.pdb}}
#'
#' @examples
#' format.pdb.title(c("Molecule 1", "is just an example"))

#' @name format.pdb.title
#' @method format.pdb title
#' @export format.pdb.title
format.pdb.title = function(x, ...) {
	UseMethod("format.pdb.title");
}

#' @rdname format.pdb.title
#' @method format.pdb.title character
#' @exportS3Method format.pdb.title character
format.pdb.title.character = function(x, ...) {
	if(is.null(x)) {
		# Field is mandatory!
		x = format80("TITLE    NULL");
		return(x);
	}
	noHeader = substr(x, 1, 6) != "TITLE ";
	idHeader = which(noHeader);
	if(length(idHeader) > 0) {
		if(idHeader[1] == 1) {
			x[1] = format80(x[1], name = "TITLE    ");
			idHeader = idHeader[-1];
		}
		len = length(idHeader);
		if(len > 0) {
			id = as.character(idHeader);
			sp = sapply(nchar(id), function(len) {
				paste0(rep(" ", 4 - len), collapse = "");
			});
			x[idHeader] = paste0("TITLE", sp, id, " ", x[idHeader]);
			x[idHeader] = format80(x[idHeader]);
		}
	}
	return(x);
}

format80 = function(x, name = "") {
	len = 80 - nchar(x) - nchar(name);
	# TODO: Warn if len > 80?
	sp  = sapply(len, function(len) {
			if(len <= 0) "" else paste0(rep(" ", len), collapse = "");
		});
	paste0(name, x, sp);
}

#' Inter-Atomic Distances
#' 
#' Computes inter-atomic distance vectors.
#' 
#' The purpose of the \sQuote{distances} class is to store the inter-atomic 
#' distance vectors and facilitate their manipulation when passing from the 
#' Cartesian to fractional references and vice versa.\cr
#' The default method of the \code{distances} function is actually a builder
#'   allowing to create a \sQuote{distances} object from its different components, i.e.:
#'   \code{dx1}, \code{dx2}, \code{dx3}, and \code{basis}.
#'   All the arguments have to be specified except 'basis' which
#'   by default is set to "xyz" (Cartesian reference).
#' 
#' For objects of class \sQuote{coords}, \sQuote{atoms}, \sQuote{pdb},
#' two sets of atomic coordinates, defined by \code{sel1} and \code{sel2},
#' are extracted and inter-atomic distance vectors are computed 
#' between these two sets.
#' For objects of class \sQuote{connect}, distances are computed for the atoms
#'   specified by the \sQuote{connect} object.
#' 
#' The method for the \sQuote{dist.point} function computes the inter-atomic distances
#' between the atoms and a specified point.
#' 
#' The method of the \code{norm} function for objects of class \sQuote{distances}
#'   computes the norm of the distances vectors.
#' \code{type} specify how to project the distance vectors before 
#'   computing the norms. By default no projection is perform.
#'   The three dx, dy, and dz components of the distance vectors
#'   are used to compute the norm. 
#' \code{type} can take the following values:
#'    \itemize{
#'    \item   x: The distance vectors are projected over x.
#'    \item   y: The distance vectors are projected over y.
#'    \item   z: The distance vectors are projected over z.
#'    \item  xy: The distance vectors are projected in the xy-plan.
#'    \item  yz: The distance vectors are projected in the yz-plan.
#'    \item  zx: The distance vectors are projected in the zx-plan.
#'    \item xyz: The distance vectors are not projected 
#'      (The three components of the distance vectors are used to compute the norm).}
#' \code{is.distances} tests if x is an object of class \sQuote{distances};
#' the test is limited to the class attribute.
#' 
#' @return The \code{distances} and \code{dist.point} functions return an object
#'   of class \sQuote{distances} containing inter-atomic distance vectors.
#' The \code{norm} function return an array, with the same dimensions as
#'   the \code{dx1}, \code{dx2}, \code{dx3} components of the \sQuote{distances} object
#'   for which the norms have been computed, containing the norm of the distance vectors. 
#' \cr\cr
#' \code{is.distances} returns TRUE if x is an object of class \sQuote{distances}
#' and FALSE otherwise.
#' 
#' @param dx1,dx2,dx3 numeric arrays containing the first, second and third components of the distance vectors.
#' @param basis a single element character vector indicating the type of basis vector used to express the coordinates.
#' @param x,y,z an R object containing atomic coordinates, or a a connect object (for the corresponding method);
#' @param sel1,sel2 integer or logical vectors defining two atomic selections between which the distance vectors are computed.
#' @param type a single element character vector indicating how to project the distances vectors before computing the norms. See details.
#' @param data an object of type pdb, atoms or coords, containing atomic coordinates.
#' @param subset enables sub-setting the coords object;
#' @param \dots further arguments passed to or from other methods.
#' 
#' @seealso 
#' \code{\link{coords}}, \code{\link{basis}}, \code{\link{xyz2abc}}, \code{\link{abc2xyz}}
#' 
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' is.DCB7 <- x$atoms$resname == "DCB" & x$atoms$resid == 7
#' is.DCB8 <- x$atoms$resname == "DCB" & x$atoms$resid == 8
#' d <- distances(x, is.DCB7, is.DCB8)
#' norm(d, type = "xyz")
#' norm(d, type = "xy")
#' norm(d, type = "x")
#
#' d <- dist.point(x, c(0,0,0), subset = is.DCB7)
#' norm(d, type = "xyz")
#' 
#' @keywords classes manip
#' 
#' @name distances
#' @export
distances <- function(...)
  UseMethod("distances")

#' @rdname distances
#' @export
dist.point <- function(...)
  UseMethod("dist.point")

#' @rdname distances
#' @export
distances.default <- function(dx1 = numeric(0), dx2 = numeric(0), dx3 = numeric(0), basis = "xyz", ...) {
  if( ! is.numeric(dx1) || ! is.numeric(dx2) || ! is.numeric(dx3))
    stop("'dx1', 'dx2' and 'dx3' must be numeric");
  if(is.null(dim(dx1))) {
    if( ! is.null(dim(dx2)) && ! is.null(dim(dx3)))
      stop("'dx1', 'dx2' and 'dx3' must have the same length/dimensions");
    if(length(dx1) != length(dx2) || length(dx1) != length(dx3))
      stop("'dx1', 'dx2' and 'dx3' must have the same length");
  }
  else {
    if(any(dim(dx1) != dim(dx2)) || any(dim(dx1) != dim(dx3)))
      stop("'dx1', 'dx2' and 'dx3' must have the same dimensions");
  }
  check.basis(basis);
  
  to.return <- list(dx1 = dx1, dx2 = dx2, dx3 = dx3);
  class(to.return) <- c("distances", "list");
  attr(to.return, "basis") <- basis;
  return(to.return)
}

#' @rdname distances
#' @export
distances.coords <- function(x, sel1, sel2, ...){
  if(missing(sel1) | missing(sel2))
    stop("Please specify 'sel1' and 'sel2'")
  if(is.numeric(sel1)) {
    check.idOrLogical(sel1, natom(x), "'sel1'");
  }
  if(is.numeric(sel2)) {
    check.idOrLogical(sel2, natom(x), "'sel2'");
  }
  if(is.logical(sel1) & length(sel1) != natom(x))
    stop("'sel1' length must be equal to natom(x)")
  if(is.logical(sel2) & length(sel2) != natom(x))
    stop("'sel2' length must be equal to natom(x)")  

  xyz1 <- x[sel1,]
  xyz2 <- x[sel2,]
  
  dx1 <- t(outer(xyz2$x1, xyz1$x1, "-"))
  dx2 <- t(outer(xyz2$x2, xyz1$x2, "-"))
  dx3 <- t(outer(xyz2$x3, xyz1$x3, "-"))
  
  dimnames(dx1) <- list(sel1 = NULL, sel2 = NULL)
  dimnames(dx2) <- list(sel1 = NULL, sel2 = NULL)
  dimnames(dx3) <- list(sel1 = NULL, sel2 = NULL)
  
  to.return <- distances.default(dx1, dx2, dx3, basis = basis(x))
  return(to.return)
}

#' @rdname distances
#' @export
distances.atoms <- function(x, sel1, sel2, ...)
  distances.coords(coords(x), sel1 = sel1, sel2 = sel2, ...)

#' @rdname distances
#' @export
distances.connect = function(x, data, ...) {
	sel1 = x$eleid.1;
	sel2 = x$eleid.2;
	coords = coords(data);
	d = coords[sel1,] - coords[sel2,];
	d = data.frame(dx1 = d[,1], dx2 = d[,2], dx3 = d[,3]);
	attr(d, "basis")   = basis(data);
	class(d) = c("distances", "data.frame");
	return(d);
}

#' @rdname distances
#' @export
distances.pdb <- function(x, sel1, sel2, ...)
  distances.atoms(x$atoms, sel1 = sel1, sel2 = sel2, ...)

### Distance to a Point
#' @rdname distances
#' @export
dist.point.default <- function(data, x, y = NULL, z = NULL, subset = NULL, ...) {
	if(is.null(y)) {
		if(length(x) != 3) stop("Missing coordinates for given point!");
		y = x[2]; z = x[3]; x = x[1];
	}
	xyz = coords(data);
	if(is.null(xyz)) {
		stop("Data must be an object that contains coordinates, e.g.",
			"pdb, atoms or coords!");
	}
	basis = basis(xyz);
	if( ! is.null(subset)) {
		xyz = subset(xyz, subset);
	}
	
	dx1 = xyz$x1 - x;
	dx2 = xyz$x2 - y;
	dx3 = xyz$x3 - z;
	
	to.return <- distances.default(dx1, dx2, dx3, basis=basis);
	return(to.return)
}

#' @rdname distances
#' @export
is.distances <- function(x)
	inherits(x, "distances");

#' @rdname distances
#' @export
norm <- function(...)
  UseMethod("norm")

#' @rdname distances
#' @export
norm.distances <- function(x, type = "xyz", ...){
  if(basis(x) == "abc")
    stop("Please provide Cartesian coordinates. See 'abc2xyz'.")
  
  to.return <- switch(type,
    x   = x$dx1,
    y   = x$dx2,
    z   = x$dx3,
    xy  = sqrt(x$dx1^2 + x$dx2^2),
    yz  = sqrt(x$dx2^2 + x$dx3^2),
    zx  = sqrt(x$dx3^2 + x$dx1^2),
    xyz = sqrt(x$dx1^2 + x$dx2^2 + x$dx3^2))

  return(to.return)
}

#' @rdname distances
#' @export
norm.connect = function(x, data, type = "xyz", ...) {
	d = distances(x, data = data);
	norm.distances(d, type = type, ...);
}

#' The Basis of an Object
#' 
#' Functions to get or set the basis of an object containing atomic coordinates.
#' 
#' \code{basis} and \code{basis<-} are respectively generic accessor and 
#' replacement functions. The default methods get and set the \code{basis} 
#' attribute of an object containing atomic coordinates. This attribute indicate
#' the type basis vector used to express atomic coordinates.\cr \code{value} 
#' must be equal to \code{"xyz"}, for Cartesian, or \code{"abc"}, for fractional
#' coordinates.\cr The methods for objects of class \sQuote{pdb} get and set the
#' \code{basis} attribute of its \code{atoms} component.
#' 
#' @return \describe{ \item{For \code{basis}:}{NULL or a single element
#' character vector. (NULL is given if the object has no \code{basis}
#' attribute.)} \item{For \code{basis<-}:}{the updated object. (Note that the
#' value of \code{basis(x) <- value} is that of the assignment, value, not the
#' return value from the left-hand side.)} }
#' 
#' @param x an R object containing atomic coordinates.
#' @param value a single element character vector use to set the basis of 
#'   \code{x}.
#'
#' @seealso \code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}}
#' 
#' @examples
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' basis(x)
#' x <- xyz2abc(x)
#' basis(x)
#' 
#' @keywords attribute
#'
#' @name basis
#' @export
basis <- function(x)
  UseMethod("basis")

#' @rdname basis
#' @export
basis.default <- function(x)
  attr(x, which = "basis")

#' @rdname basis
#' @export
'basis<-' <- function(x, value)
  UseMethod("basis<-", x)

#' @rdname basis
#' @export
'basis<-.default' <- function(x, value)
{
  if(!value %in% c("xyz","abc")) stop("Unrecognized basis")
  attr(x, "basis") <- value
  return(x)
}

#' @rdname basis
#' @export
basis.pdb <- function(x)
{
  if(!is.pdb(x)) stop("'x' must be an object of class 'pdb'")
  
  to.return <- basis.default(x$atoms)
  return(to.return)
}

#' @rdname basis
#' @export
'basis<-.pdb' <- function(x, value)
{
  if(!is.pdb(x)) stop("'x' must be an object of class 'pdb'")
  
  if(!value %in% c("xyz","abc")) stop("Unrecognized basis set")
  attr(x$atoms, "basis") <- value
  return(x)
}

#' From Cartesian to Fractional Coordinates and Vice Versa
#' 
#' Converts Cartesian coordinates into fractional coordinates and vice versa.
#' 
#' For \code{\link{atoms}} and \code{\link{pdb}} objects, the atomic coordinates
#' are first extracted from \code{x} using the \code{\link{coords}} function. 
#' Then, using the periodic boundary conditions stored into \code{crystal}, the 
#' coordinates are converted from Cartesian to fractional (for the 
#' \code{xyz2abc} functions) or from fractional to Cartesian (for the 
#' \code{abc2xyz} functions) coordinates. Finally, for \code{\link{atoms}} and 
#' \code{\link{pdb}} objects, the new atomic coordinates are reassigned to the 
#' original \code{x} object using the \code{\link{coords<-}} function and 
#' \code{x} is returned.
#' 
#' @return Return an object of the same class as \code{x}, with atomic 
#'   coordinates expressed in a different basis set.
#'   
#' @param \dots arguments passed to methods.
#' @param x an R object containing atomic coordinates.
#' @param crystal an object of class \code{\link{crystal}}.
#' @param cryst1 will be deprecated; use \code{crystal} instead.
#'   
#' @seealso \code{\link{basis}}, \code{\link{coords}}, \code{\link{atoms}},
#' \code{\link{pdb}}, \code{\link{crystal}}
#' 
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' basis(x)
#' x <- xyz2abc(x)
#' basis(x)
#' x <- abc2xyz(x)
#' basis(x)
#' 
#' \donttest{
#'   
#'   # This example returns an error because the coordinates stored
#'   # into the PDB file are already Cartesian coordinates.
#'   x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#'   try(x <- abc2xyz(x))
#' }
#' 
#' @keywords manip
#' 
#' @name xyz2abc
#' @export
xyz2abc <- function(...)
  UseMethod("xyz2abc")

#' @rdname xyz2abc
#' @export
xyz2abc.coords <- function(x, crystal, ..., cryst1 = NULL)
{
	crystal = checkArgCrystal(crystal, cryst1);
	if(missing(crystal)) stop("Please specify a 'crystal' object");
	if(! is.coords(x)) stop("'x' must be an object of class 'coords'");
	if(! is.crystal(crystal))
		stop("'crystal' must be an object of class 'crystal'");
	
	if(basis(x) != "xyz") stop("Coordinates are not Cartesian coordinates")
	
	cell = cell.coords(crystal);
	to.return = solve(cell) %*% t(x);
	to.return = coords.default(
		to.return["a",], to.return["b",], to.return["c",], "abc");
	
	return(to.return);
}

#' @rdname xyz2abc
#' @export
xyz2abc.atoms <- function(x, crystal, ...)
{
	if(! is.atoms(x)) stop("'x' must be an object of class 'atoms'");
	
	value = xyz2abc.coords(coords.atoms(x), crystal);
	coords(x) = value;
	return(x);
}

#' @rdname xyz2abc
#' @export
xyz2abc.pdb <- function(x, crystal = x$crystal, ..., cryst1 = NULL)
{
	crystal = checkArgCrystal(crystal, cryst1);
	if(! is.pdb(x)) stop("'x' must be an object of class 'pdb'");
	value = xyz2abc.coords(coords.pdb(x), crystal);
	coords(x) = value;
	return(x);
}

#' @rdname xyz2abc
#' @export
xyz2abc.distances <- function(x, crystal, ...){
  if(!is.distances(x)) stop("'x' must be an object of class 'distances'")
  if(basis(x) != "xyz") stop("Coordinates are not Cartesian coordinates")
  
  dims <- dim(x$dx1)
  x <- coords(c(x$dx1), c(x$dx2), c(x$dx3), basis = basis(x))
  x <- xyz2abc.coords(x, crystal = crystal)
  dx1 <- array(x$x1, dim = dims)
  dx2 <- array(x$x2, dim = dims)
  dx3 <- array(x$x3, dim = dims)
  x <- distances.default(dx1, dx2, dx3, basis = "abc")
  return(x)
}

#' @rdname xyz2abc
#' @export
abc2xyz <- function(...)
  UseMethod("abc2xyz")

#' @rdname xyz2abc
#' @export
abc2xyz.coords <- function(x, crystal, ..., cryst1 = NULL)
{
	crystal = checkArgCrystal(crystal, cryst1);
	if(missing(crystal)) stop("Please specify a 'crystal' object");
	if(! is.coords(x)) stop("'x' must be an object of class 'coords'");
	if(! is.crystal(crystal)) stop("'crystal' must be an object of class 'crystal'");
	
	if(basis.default(x) != "abc")
		stop("Coordinates are not fractional coordinates");
	
	cell <- cell.coords(crystal);
	to.return <- cell %*% t(x);
	to.return <- coords.default(
		to.return["x",], to.return["y",], to.return["z",], "xyz");
  
	return(to.return)
}

#' @rdname xyz2abc
#' @export
abc2xyz.atoms <- function(x, crystal, ...)
{
	if(! is.atoms(x)) stop("'x' must be an object of class 'atoms'")
	
	value = abc2xyz.coords(coords.atoms(x), crystal);
	coords(x) = value;
	return(x)
}

#' @rdname xyz2abc
#' @export
abc2xyz.pdb <- function(x, crystal = x$crystal, ..., cryst1 = NULL)
{
	crystal = checkArgCrystal(crystal, cryst1);
	if(! is.pdb(x)) stop("'x' must be an object of class 'pdb'");
	
	value = abc2xyz.coords(coords.pdb(x), crystal);
	coords(x) = value;
	return(x)
}

#' @rdname xyz2abc
#' @export
abc2xyz.distances <- function(x, crystal, ...){
	if(! is.distances(x)) stop("'x' must be an object of class 'distances'")
	if(basis(x) != "abc") stop("Coordinates are not fractional coordinates")
	
	dims = dim(x$dx1);
	x   = coords(c(x$dx1), c(x$dx2), c(x$dx3), basis = basis(x));
	x   = abc2xyz.coords(x, crystal = crystal);
	dx1 = array(x$x1, dim = dims);
	dx2 = array(x$x2, dim = dims);
	dx3 = array(x$x3, dim = dims);
	x   = distances.default(dx1, dx2, dx3, basis = "xyz");
	return(x)
}

#' PDB File Writer
#' 
#' Writes a Protein Data Bank (PDB) coordinate file from an object of class \sQuote{pdb}.
#' 
#' All data stored in the \sQuote{pdb} object are written to a PDB file. A list of objects of class \sQuote{pdb} can be provided to write multiple MODEL data into a single file. In this case, each \sQuote{pdb} object of the list must have the same \code{crystal} and \code{connect} components.
#' \cr
#' To write only a subset of a \sQuote{pdb} object see function \code{\link{subset.pdb}}.
#' 
#' @param x an object, or a list of objects, of class \sQuote{pdb}.
#' @param file a single element character vector containing the name of the PDB file to be created.
#' 
#' @return No return value, called for side effects.
#'
#' @references
#' PDB format is described at:
#' http://www.wwpdb.org/documentation/format33/v3.3.html
#' 
#' @seealso \code{\link{read.pdb}}, \code{\link{pdb}}, \code{\link{crystal}}, \code{\link{atoms}}, \code{\link{connect}}, \code{\link{subset.pdb}}
#' 
#' @examples 
#' # Read a PDB file included with the package
#' pdb <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' 
#' # Write the pdb object to file "Rpdb.pdb" in the current directory
#' write.pdb(pdb, file = "Rpdb.pdb")
#'
#' # Cleanup
#' unlink("Rpdb.pdb")
#' 
#' @keywords IO
#' 
#' @name write.pdb
#' @export
write.pdb <- function(x, file="Rpdb.pdb")
{
  if(is.pdb(x)) x <- list(x)
  if( ! all(sapply(x, is.pdb)))
      stop("'x' must be an object of class 'pdb'");

  lines <- NULL;
  
  ### Title
  title = unique(unlist(lapply(x, function(y) return(y$title))));
  title = format.pdb.title.character(title);
  lines = c(lines, title);

  ### Remarks
  remark <- unique(unlist(lapply(x, function(y) return(y$remark))))
  if( ! is.null(remark))
  {
    noHeader = (substr(remark, 1, 6) != "REMARK");
    remark[noHeader] <- paste0("REMARK", remark[noHeader]);
    lines <- c(lines, remark)
  }

	### Crystal
	if(! is.null(x[[1]]$crystal))
	{
		cryst = x[[1]]$crystal;
		abc   = paste(format(cryst$abc, width=9, nsmall=3, justify="right"), collapse="")
		abg   = paste(format(cryst$abg, width=7, nsmall=2, justify="right"), collapse="")
		lines = c(lines, paste("CRYST1", abc, abg, sep=""));
	}

  # TODO: as.pdbModel()
  for(model in seq_along(x)) {
    if(length(x) > 1) lines <- c(lines, paste0("MODEL ",format(model, width = 4)))

    if(basis(x[[model]]) != "xyz") x[[model]] <- abc2xyz.pdb(x[[model]])
  
    X <- round(x[[model]]$atoms$x1, digits = 3)
    Y <- round(x[[model]]$atoms$x2, digits = 3)
    Z <- round(x[[model]]$atoms$x3, digits = 3)
    occ  <- round(x[[model]]$atoms$occ , digits = 2)
    temp <- round(x[[model]]$atoms$temp, digits = 2)
  
    recname <- format(x[[model]]$atoms$recname, justify = "left" , width = 6)
    eleid   <- format(x[[model]]$atoms$eleid  , justify = "right", width = 5)
    elename <- format(x[[model]]$atoms$elename, justify = "left" , width = 4)
    alt     <- format(x[[model]]$atoms$alt    , justify = "left" , width = 1)
    resname <- format(x[[model]]$atoms$resname, justify = "left" , width = 3)
    chainid <- format(x[[model]]$atoms$chainid, justify = "left" , width = 1)
    resid   <- format(x[[model]]$atoms$resid  , justify = "right", width = 4)
    insert  <- format(x[[model]]$atoms$insert , justify = "left" , width = 1)
    X       <- format(X                       , justify = "right", width = 8)
    Y       <- format(Y                       , justify = "right", width = 8)
    Z       <- format(Z                       , justify = "right", width = 8)
    occ     <- format(occ                     , justify = "right", width = 6)
    temp    <- format(temp                    , justify = "right", width = 6)
    segid   <- format(x[[model]]$atoms$segid  , justify = "left" , width = 3)
  
    lines = c(lines,
		paste(recname, eleid, " ",
			elename, alt, resname, " ",
			chainid, resid, insert, "   ",
			X,Y,Z, occ, temp, "      ", segid, sep = ""));
    if(length(x) > 1) lines <- c(lines,"ENDMDL")
  }
  
  if(! is.null(x[[1]]$connect))
  {
    eleid.1 = format(x[[1]]$connect$eleid.1, width=5, justify="right");
    eleid.2 = format(x[[1]]$connect$eleid.2, width=5, justify="right");
    connect = paste("CONECT", eleid.1, eleid.2, sep = "");
    lines   = c(lines, connect);
  }
  #
  writeLines(lines, file);
}

#' Reflection of Atomic Coordinates
#' 
#' Perform a reflection (or mirror) operation on atomic coordinates with respect 
#' to a given reflection plane.
#' 
#' \code{mirror} is a generic function.
#' Method for objects of class \sQuote{coords} first convert the coordinates
#' into Cartesian coordinates using \code{crystal} if needed.
#' Once reflected, the coordinates are reconverted back to the original basis set
#' using again \code{crystal}.
#'
#' Method for objects of class \sQuote{pdb} first extract coordinates from the object 
#' using the function \code{coords}, perform the reflection, and update the 
#' coordinates of the \sQuote{pdb} object using the function \code{coords<-}.
#' 
#' @return An object of the same class as \code{x} with reflected coordinates.
#' 
#' @param x an R object containing atomic coordinates.
#' @param p1 a numeric vector of length 3 containing the coordinates of the
#'   first point defining the reflexion plan. Can also be a 3x3 matrix or
#'   data.frame containing by row \code{p1}, \code{p2} and \code{p3}.
#' @param p2 a numeric vector of length 3 containing the coordinates of the
#'   second point defining the reflexion plane.
#' @param p3 a numeric vector of length 3 containing the coordinates of the
#'   third point defining the reflexion plane.
#' @param mask a logical vector indicating the set of coordinates to which to
#'   apply the reflexion.
#' @param crystal an object of class \sQuote{crystal} used to convert fractional
#'   into Cartesian coordinates (when needed).
#' @param \dots further arguments passed to or from other methods.
#' 
#' @seealso Helper functions for reflection with respect to a given Cartesian
#' plane or a plane defined by two lattice vectors:\cr \code{\link{Mxy}},
#' \code{\link{Myz}}, \code{\link{Mzx}}, \code{\link{Mab}}, \code{\link{Mbc}},
#' \code{\link{Mca}}\cr Passing from Cartesian to fractional coordinates (or Vis
#' Versa):\cr \code{\link{xyz2abc}}, \code{\link{abc2xyz}}
#' 
#' @examples 
#' # First lets read a pdb file
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' cell <- cell.coords(x)
#' visualize(x, mode = NULL)
#'
#' # Mirror operation with respect to the ab-plane
#' visualize(mirror(x, rep(0,3), p1=cell[, "a"], p2=cell[, "b"]), mode = NULL)
#' # Mirror operation with respect to the ab-plane for residue 1
#' visualize(mirror(x, rep(0,3), p1=cell[, "a"], p2=cell[, "b"],
#'    mask = x$atoms$resid == 1), mode = NULL)
#' 
#' @keywords manip
#'  
#' @name mirror
#' @export
mirror <- function(...)
  UseMethod("mirror")

#' @rdname mirror
#' @export
mirror.coords <- function(x, p1, p2 = NULL, p3 = NULL,
		mask = TRUE, crystal = NULL, ...) {
	# Checks: p1, p2, p3
	if(missing(p1))
		stop("Please specify at least 'p1'");
	if(is.null(p2) && is.null(p3)) {
		if(ncol(p1) != 3 || nrow(p1) != 3)
			stop("When 'p2' and 'p3' are not specified,",
				" then 'p1' must be a 3x3 matrix or data.frame");
		p3 = p1[3,];
		p2 = p1[2,];
		p1 = p1[1,];
	} else {
		if(length(p3) != 3 || length(p2) != 3 || length(p1) != 3)
			stop("'p1', 'p2' and 'p3' must be vectors of length 3");
	}
	if(all(p1 == p2) || all(p1 == p3) || all(p2 == p3))
		stop("'p1', 'p2' and 'p3' must be different to define the mirror");
	# Checks: Mask
  if(length(mask) != natom(x)){
    if(length(mask) != 1)
      warning("'mask' has been recycled")
    mask <- rep(mask, length = natom(x))
  }
	# Crystal
	basis.ori = basis(x);
	if(basis.ori != "xyz") {
		if(is.null(crystal))
			stop("Please specify a 'crystal' obj to convert your fractional coordinates into Cartesian");
		x = abc2xyz(x, crystal = crystal);
	}
	v12 = p2 - p1;
	v23 = p3 - p2;

  vn <- vectProd(v12, v23)
  vn <- vn/vectNorm(vn)
  x <- Txyz(x,  p1[1],  p1[2],  p1[3], mask=mask)
  rotM <- diag(3) - as.matrix(
    rbind(
      c(2*vn[1]*vn[1], 2*vn[1]*vn[2], 2*vn[1]*vn[3]),
      c(2*vn[2]*vn[1], 2*vn[2]*vn[2], 2*vn[2]*vn[3]),
      c(2*vn[3]*vn[1], 2*vn[3]*vn[2], 2*vn[3]*vn[3])
      )
    )
  x[mask,] <- coords(as.matrix(x[mask,])%*%rotM, basis = "xyz")
  x <- Txyz(x, -p1[1], -p1[2], -p1[3], mask=mask)
  return(x)
}

#' @rdname mirror
#' @export
mirror.pdb <- function(x, p1, p2 = NULL, p3 = NULL,
		mask = TRUE, crystal = x$crystal, ...) {
	coords(x) = mirror(coords(x), p1=p1, p2=p2, p3=p3,
		mask=mask, crystal=crystal, ...);
	return(x);
}

#' Select atoms
#' 
#' Select groups of atoms from a PDB object.
#' 
#' \code{select} is a generic function, which enables
#'   the selection of element IDs from molecules.

#' @return 
#' \code{select} returns a vector with the element IDs.


#' @param x an R object containing atoms;
#' @param id an integer vector specifying the id of residues.
#' @param residue a character vector specifying the residues to select.
#' @param chain character vector specifying which chains to process;
#' @param \dots arguments passed to methods.


#' @name select
#' @export
select = function(x, id, ...)
	UseMethod("select");

#' @rdname select
#' @export
select.pdb = function(x, id, residue, chain, ...) {
	x   = x$atoms;
	ids = select.atoms(x, id=id, residue=residue, chain=chain, ...);
	return(ids);
}

# Note:
# - Selects ALL atoms (not only 1 atom per residue)!
#' @rdname select
#' @export
select.atoms = function(x, id, residue, chain, ...) {
	isE = rep(TRUE, natom(x));
	if(! missing(chain)) {
		isE = x$chain %in% chain;
	}
	if(! missing(id)) {
		isE = isE & x$resid %in% id;
	}
	if(! missing(residue)) {
		residue = toupper(residue);
		isE = isE & x$resname %in% residue;
	}
	ids = x$eleid[isE];
	return(ids);
}


# Water
which.water = function(x) {
	if(inherits(x, "pdb")) x = x$atoms;
	isHOH = x$Hetero & x$resname == "HOH";
	ids = x$eleid[isHOH];
	return(ids)
}

### Molecular Structure


is.structure = function(x) {
	if(inherits(x, "structure")) return(TRUE);
	return(FALSE);
}

as.pdb.structure = function(helix = NULL, sheet = NULL, ...) {
	x = list(Helix = helix, Sheet = sheet);
	class(x) = c("structure", "list");
	return(x);
}

# Note:
# - Positions are coded using 4 digits;
# - Duplicate positions are disambiguated using the Insrtion Code;
# - Insertion codes use letters: potential for 26*2 more positions;
# - Sense of Strand:
#   0 = First strand;
#   1 = Parallel to previous, -1 = Anti-parallel to previous;
read.pdb.structure = function(x) {
	### Helix:
	helix = read.pdb.helix(x);
	### Sheet:
	sheet = read.pdb.sheet(x);
	#
	structMol = as.pdb.structure(helix, sheet);
	return(structMol);
}

###
# Helix Types:
# 1 = Right-handed Alpha;  2 = Right-handed Omega;
# ...
read.pdb.helix = function(x) {
	# https://www.wwpdb.org/documentation/file-format-content/format33/sect5.html#HELIX
	isHelix = grepl("^HELIX ", x);
	idHelix = which(isHelix);
	isHelix = length(idHelix) > 0;
	if(! isHelix) return(NULL);
	# Helix:
	sH = x[idHelix];
	hID   = trim(substr(sH,  8, 10));
	hName = trim(substr(sH, 12, 14));
	# Start:
	hAAS  = trim(substr(sH, 16, 18)); # Residue name
	hChS  = trim(substr(sH, 20, 20)); # Chain ID
	hAASn = trim(substr(sH, 22, 25)); # Seq No.
	hAASI = trim(substr(sH, 26, 26)); # Insert code
	# End:
	hAAE  = trim(substr(sH, 28, 30)); # Residue name
	hChE  = trim(substr(sH, 32, 32)); # Chain ID
	hAAEn = trim(substr(sH, 34, 37)); # Seq No.
	hAAEI = trim(substr(sH, 38, 38)); # Insert code
	hType = trim(substr(sH, 39, 40)); # Helix Class
	hComm = trim(substr(sH, 41, 70)); # Comments
	hLen  = trim(substr(sH, 72, 76)); # Length Helix
	helix = data.frame(ID = as.integer(hID), Name  = hName,
		Type = as.integer(hType), # Type of Helix (including handedness)
		Len = as.integer(hLen),
		chS = hChS, chE = hChE,
		AAS = hAAS, idAAS = as.integer(hAASn), insertS = hAASI,
		AAE = hAAE, idAAE = as.integer(hAAEn), insertE = hAAEI,
		Comment = hComm
		);
	return(helix);
}
read.pdb.sheet = function(x) {
	# https://www.wwpdb.org/documentation/file-format-content/format33/sect5.html#SHEET
	isSheet = grepl("^SHEET ", x);
	idSheet = which(isSheet);
	isSheet = length(idSheet) > 0;
	if(! isSheet) return(NULL);
	# Sheets:
	sSh = x[idSheet];
	sID   = trim(substr(sSh,  8, 10)); # Strand ID
	sName = trim(substr(sSh, 12, 14)); # Sheet ID
	sNStr = trim(substr(sSh, 15, 16)); # No. Strands
	# Start:
	sAAS  = trim(substr(sSh, 18, 20)); # AA Code
	sChS  = trim(substr(sSh, 22, 22)); # Chain ID
	sAASn = trim(substr(sSh, 23, 26)); # Seq No.
	sAASI = trim(substr(sSh, 27, 27)); # Insertion Code
	# End:
	sAAE  = trim(substr(sSh, 29, 31)); # AA Code
	sChE  = trim(substr(sSh, 32, 33)); # Chain ID
	sAAEn = trim(substr(sSh, 34, 37)); # Seq No
	sAAEI = trim(substr(sSh, 38, 38)); # Insertion Code
	sType = trim(substr(sSh, 39, 40)); # Sense of Strand
	### Registration
	# Current Strand:
	atomRc   = trim(substr(sSh, 42, 45)); # Atom
	insertRc = trim(substr(sSh, 55, 55)); # Insertion
	# Previous Strand:
	atomRp   = trim(substr(sSh, 42, 45)); # Atom
	insertRp = trim(substr(sSh, 70, 70)); # Insertion
	# TODO: remaining fields;
	sheet = data.frame(ID = as.integer(sID), Name  = sName,
		Type = as.integer(sType), # Sense of Strand
		chS = sChS, chE = sChE,
		AAS = sAAS, idAAS = as.integer(sAASn), insertS = sAASI,
		AAE = sAAE, idAAE = as.integer(sAAEn), insertE = sAAEI,
		# Registration
		atomRc   = atomRc,
		insertRc = insertRc,
		atomRp   = atomRp,
		insertRp = insertRp
		);
	return(sheet);
}

#' Reinitialize Object Indexing
#' 
#' Reinitialize the indexing of an object.
#' 
#' \code{reindex} is a generic function to reinitialize the indexing of an object or its components.
#' The methods for objects of class \sQuote{atoms} reinitialize the residue and element IDs starting
#' from 1 and avoiding gaps in the indexes. For objects of class \sQuote{pdb} their \code{atoms} and
#' \code{connect} components are reindexed consistently.
#' 
#' @return Return an object of the same class as \code{x} with updated indexes.
#' 
#' @param x an R object.
#' @param eleid a single element logical vector indicating if elements IDs have to reindexed.
#' @param resid a single element logical vector indicating if residues IDs have to reindexed.
#' @param \dots further arguments passed to or from other methods.
#' 
#' @seealso \code{\link{pdb}}, \code{\link{atoms}}, \code{\link{subset.atoms}}, \code{\link{subset.pdb}}
#' 
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' x <- subset(x, x$atoms$eleid %in% sample(x$atoms$eleid, 10))
#' print(x)
#' x <- reindex(x)
#' print(x)
#' 
#' @keywords manip
#' 
#' @name reindex
#' @export
reindex <- function(...)
  UseMethod("reindex")

#' @rdname reindex
#' @export
reindex.atoms <- function(x, eleid = TRUE, resid = TRUE, ...)
{
  if(eleid)
  {
    eleid <- as.factor(x$eleid)
    levels(eleid) <- 1:nlevels(eleid)
    x$eleid <- as.integer(as.character(eleid))
  }
  if(resid)
  {
    resid <- as.factor(x$resid)
    levels(resid) <- 1:nlevels(resid)
    x$resid <- as.integer(as.character(resid))
  }
  rownames(x) <- 1:nrow(x)
  return(x)
}

#' @rdname reindex
#' @export
reindex.pdb <- function(x, eleid = TRUE, resid = TRUE, ...)
{
  if(eleid)
  {
    if(anyDuplicated(x$atoms$eleid)){
      x$atoms$eleid <- 1:natom(x)
      if(! is.null(x$connect)) {
        cat("Recalculating connectivity\n");
        x$connect = connect(x);
      }
    } else {
      if(!is.null(x$connect)) {
        eleid.1 <- factor(x$connect$eleid.1, levels = x$atoms$eleid)
        eleid.2 <- factor(x$connect$eleid.2, levels = x$atoms$eleid)
        levels(eleid.1) <- 1:nlevels(eleid.1)
        levels(eleid.2) <- 1:nlevels(eleid.2)
		# TODO: simplify
        eleid.1 = as.integer(as.character(eleid.1));
        eleid.2 = as.integer(as.character(eleid.2));
        x$connect = connect.default(eleid.1, eleid.2);
      }
      eleid <- factor(x$atoms$eleid, levels = x$atoms$eleid)
      neleid <- nlevels(eleid)
      levels(eleid) <- 1:neleid
	  # TODO: simplify
      x$atoms$eleid <- as.integer(as.character(eleid))
    }
  }
  if(resid)
  {
    resid <- as.factor(x$atoms$resid)
    levels(resid) <- 1:nlevels(resid)
    x$atoms$resid <- as.integer(as.character(resid))
  }
  rownames(x$atoms) <- 1:nrow(x$atoms)
  return(x)
}

#' Helper Functions for reflection of Atomic Coordinates
#' 
#' Reflection of atomic coordinates with respect to a specific Cartesian plane or
#' a plane defined by two lattice vectors.
#' 
#' These functions are helper functions to perform a reflection with respect to 
#' a specific Cartesian plan or a plan defined by two lattice vectors. All of 
#' them call the \code{mirror} function.
#' 
#' @return An object of the same class as \code{x} with reflected coordinates.
#'   
#' @param x an R object containing atomic coordinates.
#' @param mask a logical vector indicating the set of coordinates to which to
#'   apply the reflection.
#' @param crystal an object of class \sQuote{crystal} used to convert fractional
#'   into Cartesian coordinates (when needed).
#' @param \dots further arguments passed to or from other methods.
#'   
#' @seealso \code{\link{mirror}} and \code{\link{xyz2abc}},
#' \code{\link{abc2xyz}} for passing from Cartesian to fractional coordinates
#' (or Vice Versa).
#' 
#' @examples
#' # First lets read a pdb file
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' visualize(x, mode = NULL)
#'
#' # Mirror operation with respect to the ab-plane
#' visualize(Mab(x), mode = NULL)
#' # Mirror operation with respect to the ab-plane for residue 1
#' visualize(Mab(x, mask = x$atoms$resid == 1), mode = NULL)
#' 
#' @keywords manip
#' 
#' @name mirrorHelpers
#' @export
Mxy <- function(...)
  UseMethod("Mxy")

#' @rdname mirrorHelpers
#' @export
Mxy.coords = function(x, mask = TRUE, crystal = NULL, ...)
	mirror(x, c(0,0,0), c(1,0,0), c(0,1,0), mask=mask, crystal=crystal);

#' @rdname mirrorHelpers
#' @export
Mxy.pdb = function(x, mask = TRUE, crystal = x$crystal, ...)
	mirror(x, c(0,0,0), c(1,0,0), c(0,1,0), mask=mask, crystal=crystal);


#' @rdname mirrorHelpers
#' @export
Myz <- function(...)
  UseMethod("Myz")

#' @rdname mirrorHelpers
#' @export
Myz.coords = function(x, mask = TRUE, crystal = NULL, ...)
	mirror(x, c(0,0,0), c(0,1,0), c(0,0,1), mask=mask, crystal=crystal);

#' @rdname mirrorHelpers
#' @export
Myz.pdb = function(x, mask = TRUE, crystal = x$crystal, ...)
	mirror(x, c(0,0,0), c(0,1,0), c(0,0,1), mask=mask, crystal=crystal);


#' @rdname mirrorHelpers
#' @export
Mzx <- function(...)
  UseMethod("Mzx")

#' @rdname mirrorHelpers
#' @export
Mzx.coords = function(x, mask = TRUE, crystal = NULL, ...)
	mirror(x, c(0,0,0), c(0,0,1), c(1,0,0), mask=mask, crystal=crystal);

#' @rdname mirrorHelpers
#' @export
Mzx.pdb = function(x, mask = TRUE, crystal = x$crystal, ...)
	mirror(x, c(0,0,0), c(0,0,1), c(1,0,0), mask=mask, crystal=crystal);


#' @rdname mirrorHelpers
#' @export
Mab <- function(...)
  UseMethod("Mab")

#' @rdname mirrorHelpers
#' @export
Mab.coords = function(x, crystal, mask = TRUE, ...) {
	cell = get.cellMirrorMiss(x, crystal=crystal);
	mirror(x, c(0,0,0), cell[,"a"], cell[,"b"], mask=mask, crystal=crystal);
}
  
#' @rdname mirrorHelpers
#' @export
Mab.pdb = function(x, crystal = x$crystal, mask = TRUE, ...) {
	cell = get.cellMirror(x, crystal=crystal);
	mirror(x, c(0,0,0), cell[,"a"], cell[,"b"], mask=mask, crystal=crystal);
}

#' @rdname mirrorHelpers
#' @export
Mbc <- function(...)
  UseMethod("Mbc")

#' @rdname mirrorHelpers
#' @export
Mbc.coords = function(x, crystal, mask = TRUE, ...) {
	cell = get.cellMirrorMiss(x, crystal=crystal);
	mirror(x, c(0,0,0), cell[,"b"], cell[,"c"], mask=mask, crystal=crystal);
}

#' @rdname mirrorHelpers
#' @export
Mbc.pdb <- function(x, crystal = x$crystal, mask = TRUE, ...) {
	cell = get.cellMirror(x, crystal=crystal);
	mirror(x, c(0,0,0), cell[,"b"], cell[,"c"], mask=mask, crystal=crystal);
}

#' @rdname mirrorHelpers
#' @export
Mca <- function(...)
  UseMethod("Mca")

#' @rdname mirrorHelpers
#' @export
Mca.coords = function(x, crystal, mask = TRUE, ...) {
	cell = get.cellMirrorMiss(x, crystal=crystal);
	mirror(x, c(0,0,0), cell[,"c"], cell[,"a"], mask=mask, crystal=crystal);
}

#' @rdname mirrorHelpers
#' @export
Mca.pdb = function(x, crystal = x$crystal, mask = TRUE, ...) {
	cell = get.cellMirror(x, crystal=crystal);
	mirror(x, c(0,0,0), cell[,"c"], cell[,"a"], mask=mask, crystal=crystal);
}

### Helper

# Cell:
get.cellMirror = function(x, crystal) {
	check.crystalMirror(crystal);
	cell.coords(x, crystal=crystal);
}
get.cellMirrorMiss = function(x, crystal) {
	check.crystalMirrorMiss(crystal);
	cell.coords(x, crystal=crystal);
}

# Check:
check.crystalMirror = function(x) {
	if(is.null(x))
		stop("'crystal' is required to define the mirror plane!");
}
check.crystalMirrorMiss = function(x) {
	if(missing(x) || is.null(x))
		stop("'crystal' is required to define the mirror plane!");
}

#' Atomic Symbols Converter
#' 
#' Converts character strings or atomic numbers into atomic symbols.
#' 
#' Each elements of \code{x} are converted into atomic symbols.\cr
#' When \code{x} is an integer (or numeric) vector, atomic number are search into the \code{elements} data set to find associated atomic symbols.\cr
#' When \code{x} is a character vector, \code{toSymbols} first removes all leading and trailing white spaces and numbers.
#' Then translates the first character of the character strings to uppercase and all the others to lowercase.
#' Finally, the character strings are tested for matching with element symbols provided by the \code{elements} data set.
#' NA are produced for no matching.
#' 
#' @return a character vector containing atomic symbols
#' 
#' @param x a vector to be converted into atomic symbols.
#' @param nletters an integer used to truncate the character strings before conversion.
#' @param na the default value to use for invalid or missing element symbols.
#' @param \dots further arguments passed to or from other methods.
#' 
#' @seealso \code{\link{elements}}
#' 
#' @examples 
#' x <- c(1:10)
#' toSymbols(x)
#' 
#' x <- c("C  "," o","h1","1h","UU","SI0","cR")
#' toSymbols(x)
#' 
#' # 'nletters' can be used to truncate the character strings before
#' # conversion, if needed:
#' toSymbols("SIL", nletters=3) # return NA
#' toSymbols("SIL", nletters=2) # return "Si"
#' toSymbols("SIL", nletters=1) # return "S"
#' toSymbols("SIL", nletters=3, na="X") # return "X"
#' 
#' @keywords manip
#' 
#' @name toSymbols
#' @export
toSymbols <- function(x, ...)
  UseMethod("toSymbols")

#' @rdname toSymbols
#' @export
toSymbols.integer <- function(x, ...){
  if(!is.integer(x))
    stop("'x' must be an integer")
  
  symb <- Rpdb::elements[match(x, Rpdb::elements[, "num"]), "symb"]
  symb <- as.character(symb)
  
  if(any(is.na(x)))
    warning("NAs introduced by coercion")
  
  return(symb)
}

#' @rdname toSymbols
#' @export
toSymbols.numeric <- function(x, ...){
  if(!is.numeric(x))
    stop("'x' must be numeric")
  
  if(any(round(x) != x))
    stop("'x' must be a whole number")
  
  symb <- toSymbols(as.integer(x))
  
  return(symb)
}

#' @rdname toSymbols
#' @export
toSymbols.character <- function(x, nletters = 3, na = NA, ...)
{
	x = match.element.character(x, nletters = nletters, na = NA);
	x = as.character(Rpdb::elements[x, "symb"]);
	
	if(! is.na(na) && any(isNA <- is.na(x))) {
		x[isNA] = na;
	}
	
	return(x)
}

# Note: not yet exported;
match.element.character = function(x, nletters = 3, na = NA) {
	isSymbol = attr(x, "isSymbol");
	isSymbol = if(is.null(isSymbol)) FALSE else as.logical(isSymbol);
	if(isSymbol) {
		x = as.symbolCase0(x);
	} else {
		# Ugly:
		x = sub(' +$', '', sub('^ +', '', x));
		x = gsub("[0-9]", "", x);
		x = as.symbolCase(x, n = nletters);
	}
	
	x = match(x, Rpdb::elements[,"symb"]);
	
	isNA = is.na(x);
	if(any(isNA)) {
		warning("NAs introduced by coercion: ", sum(isNA));
		if( ! is.na(na)) x[isNA] = na;
	}
	
	return(x)
}
as.symbolCase0 = function(x) {
	L1 = substr(x, 1, 1);
	Ln = substr(x, 2, nchar(x));
	x  = paste0(toupper(L1), tolower(Ln));
	return(x);
}
as.symbolCase = function(x, n) {
	L1 = substr(x, 1, 1);
	Ln = substr(x, 2, n);
	x  = paste0(toupper(L1), tolower(Ln));
	return(x);
}

# Note: not yet exported;
split.symbol.character = function(x, f, drop, rm.digits = TRUE, ...) {
	# Perl is partly broken in R 4.3, but this works:
	regex = "(?<=[A-Z])(?![a-z]|$)|(?<=.)(?=[A-Z])|(?<=[a-z])(?=[^a-z])";
	# stringi::stri_split(x, regex = regex);
	s = strsplit(x, regex, perl = TRUE);
	if(rm.digits) {
		s = lapply(s, function(s) {
			# TODO
			isNotD = is.na(suppressWarnings(as.numeric(s)));
			s = s[isNotD];
		});
	}
	return(s);
}

#' PDB File Reader
#' 
#' Reads a Protein Data Bank (PDB) coordinates file.
#' 
#' The \code{read.pdb} function reads the TITLE, REMARK, ATOM, HETATM, CRYSTAL and CONECT records
#'   from a PDB file. Three different reading modes can be used depending on the value of \code{MODEL}: 
#' \itemize{
#'   \item When \code{MODEL} is a vector of integers, MODEL sections whose serial numbers
#'      match these integers are read.
#'   \item When \code{MODEL == NULL}, all MODEL sections are read.
#'   \item When \code{MODEL == NA}, MODEL records are ignored and all ATOM and/or HETATM records
#'     are merged together to return a single object.
#' }
#' When multiple models are specified, each of the models is actually stored
#'   as a separate pdb molecule in a list of pdb molecules.
#' If the \code{resolution} parameter is set, the function attempts to extract the resolution
#'   from the REMARKS field. Note: The resolution is only meaningful for X-ray crystallography.
#' 
#' @return 
#' When a single MODEL section is read, this function returns an object of class  \sQuote{pdb} (a list with a \code{class} attribute equal to \code{pdb}) with the following components:
#' \item{title}{a character vector containing the TITLE records found in the PDB file.}
#' \item{remark}{a character vector containing the REMARK records found in the PDB file.}
#' \item{crystal}{a list of class \sQuote{crystal} containing the first CRYSTAL record found in the PDB file. All others are ignored.}
#' \item{atoms}{a data.frame of class \sQuote{atoms} containing the ATOM and HETATM records found in the PDB file.}
#' \item{connect}{a data.frame of class \sQuote{connect} containing the CONNECT records found in the PDB file.}
#' When multiple MODEL sections are read, a list of object of class \sQuote{pdb} is returned.
#' 
#' @param file a single element character vector containing the name of the PDB file to be read.
#' @param ATOM a logical value indicating whether to read the ATOM records.
#' @param HETATM a logical value indicating whether to read the HETATM records.
#' @param CRYSTAL a logical value indicating whether to read the crystal cell parameters (from the CRYST1 pdb record).
#' @param CONNECT a logical value indicating whether to read the CONNECT records.
#' @param TITLE a logical value indicating whether to read the TITLE records.
#' @param REMARK a logical value indicating whether to read the REMARK records.
#' @param MODEL an integer vector containing the serial number of the MODEL sections to be read. Can also be equal to NULL to read all the MODEL sections or to NA to ignore MODEL records (see details).
#' @param CRYST1 will be replaced by the CRYSTAL argument; existing code should be migrated to use the CRYSTAL argument.
#' @param resolution logical value indicating whether to extract the resolution (see details).
#' @param verbose logical value indicating whether to print additional information, e.g. number of models.
#' 
#' @references 
#' PDB format has been taken from:
#' http://www.wwpdb.org/documentation/format33/v3.3.html
#' 
#' @seealso
#' \code{\link{write.pdb}}, \code{\link{pdb}}, \code{\link{crystal}}, \code{\link{atoms}}, \code{\link{connect}}
#' 
#' @examples 
#' # Read a PDB file included with the package
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' 
#' # Visualize the PDB file
#' visualize(x, mode = NULL)
#' 
#' # Write the 'pdb' object 'x' in file "Rpdb.pdb" into the current directory
#' write.pdb(x, file = "Rpdb.pdb")
#'
#' # Cleanup
#' unlink("Rpdb.pdb")
#' 
#' @keywords IO
#' 
#' @name read.pdb
#' @export
read.pdb <- function(file, ATOM = TRUE, HETATM = TRUE, CRYSTAL = TRUE,
                     CONNECT = TRUE, TITLE = TRUE, REMARK = TRUE, MODEL = 1,
					 CRYST1 = NULL, resolution = TRUE, verbose = TRUE)
{
	if( ! is.null(CRYST1)) {
		if(CRYSTAL) {
			# Temporary:
			CRYSTAL = CRYST1;
		} else {
			stop("Please provide only a logical value for CRYSTAL!",
			"Argument CRYST1 will be deprecated!");
		}
	}
	if(!file.exists(file))
		stop("File '", file, "' is missing!");
	
	lines = readLines(file);
	recname = substr(lines, 1, 6);
	
	trim = function(str)
		sub(' +$', '', sub('^ +', '', str));
	
	if(verbose) {
		### Obsolete:
		isObsolete = (recname == "OBSLTE");
		idObsolete = which(isObsolete);
		if(length(idObsolete) > 0) {
			warning("Obsolete pdb!");
			txtObsolete = lines[idObsolete];
			txt = trim(substr(txtObsolete, 22, 80));
			txt[1] = paste0("PDB ",
				substr(txt[1],  1,  4), " replaced by: ",
				substr(txt[1], 11, 59));
			cat(txt, sep = "\n");
			rm(txtObsolete);
		}
		### Caveat:
		isCaveat = (recname == "CAVEAT");
		idCaveat = which(isCaveat);
		if(length(idCaveat) > 0) {
			warning("The pdb contains a Caveat record!");
			txtCaveat = lines[idCaveat];
			txt = trim(substr(txtCaveat, 20, 80));
			# TODO
		}
	}
	
	### Title:
	title = NULL;
	isTitle = (recname == "TITLE ");
	if(TITLE) {
		if(any(isTitle)) {
			title = subset(lines, isTitle);
			isTitle = TRUE;
		} else {
			# Only HEADER
			isTitle = (recname == "HEADER");
			title   = subset(lines, isTitle);
			isTitle = FALSE; # actually NOT a Title;
		}
		nposE = if(isTitle) 80 else 59;
		title = substr(title, 11, nposE);
		title = trim(title);
	}
	
	### Remarks:
	# TODO: improved extraction;
	remark = NULL;
	isRemark  = (recname == "REMARK");
	hasRemark = any(isRemark);
	if(REMARK && hasRemark)
		remark = subset(lines, isRemark);
	### Resolution:
	dfResolution = NULL;
	if(resolution && hasRemark) {
		tmp = if(is.null(remark)) subset(lines, isRemark) else remark;
		# can start both at +11 & + 12;
		dfResolution = extract.pdb.Resolution(tmp);
	}
  
	### Atoms:
	is.hetatm = rep(FALSE, length(recname));
	is.atom   = rep(FALSE, length(recname));
	if(ATOM  ) is.atom   = recname == "ATOM  ";
	if(HETATM) is.hetatm = recname == "HETATM";
	isAtomField = is.atom | is.hetatm;
	atoms = subset(lines, isAtomField);
	if(length(atoms) == 0) stop("No atoms have been selected");
	### Atoms:
	atoms = as.atoms.character(atoms, isHetero = is.hetatm[isAtomField]);
  
	### Models:
	# Field NUMMDL: is optional;
	isNModels = (recname == "NUMMDL");
	if(any(isNModels)) {
		txtModels = subset(lines, isNModels);
		if(length(txtModels) > 1) {
			warning("Malformed pdb file: incorrect number of models!");
			txtModels = txtModels[1];
		}
		nModels = as.integer(substr(txtModels, 7, 80)); # TODO: 7, 11?
	} else nModels = 0;
	if(verbose) cat("Number of models: ", nModels, "\n");
  #
  model.factor <- rep(0, length(recname))
  model.ids   <- "MODEL.1"
  model.start <- which(recname == "MODEL "); # grep("^MODEL ", recname)
  model.end   <- which(recname == "ENDMDL"); # grep("^ENDMDL", recname)
  if(length(model.start) != length(model.end))
    stop("'Unterminated MODEL section'");
  #
  hasModel = ! (length(model.start) == 0);
  if(hasModel) {
		if(any(model.start >= model.end))
			stop("'Unterminated MODEL section'");
		# All models:
		if(is.null(MODEL)) {
			model.ids = as.integer(substr(lines[model.start], 11, 14));
			MODEL = model.ids;
		}
		# Only requested models:
		MODEL = MODEL[! is.na(MODEL)];
		if(length(MODEL) == 0) {
			# TODO
		} else {
			model.ids = as.integer(substr(lines[model.start], 11, 14));
			if(verbose) {
				if(length(model.ids) > length(MODEL))
					cat("Note: Loading only models: ",
						paste0(MODEL, collapse = ", "), ";\n", sep = "");
			}
			idModels = match(model.ids, MODEL);
			naModels = is.na(idModels);
			if(any(naModels)) {
				if(verbose) {
					cat("Note: Some models not found: ",
						paste(idModels[naModels], collapse = ", "), ";\n");
				}
				idModels = idModels[! naModels];
			}
      model.start <- model.start[idModels]
      model.end   <- model.end  [idModels]
      model.ids   <- model.ids  [idModels]
      model.ids   <- paste("MODEL", model.ids, sep=".");
		model.factor [model.start + 1] = model.start;
		model.factor [model.end      ] = - model.start;
		model.factor = cumsum(model.factor);
		model.factor[model.factor == 0] <- NA
    }
  } else if(verbose && nModels > 0) {
    cat("Number of actual models: 0\n");
  }
  model.factor <- as.factor(model.factor)
  levels(model.factor) <- model.ids
  model.factor <- model.factor[is.atom | is.hetatm]
  
	### Crystal Cell:
	crystal = NULL;
	# Note: could also use recname;
	isCrystal = (recname == "CRYST1");
	if(CRYSTAL && any(isCrystal)) {
		crystal = subset(lines, isCrystal);
		crystal = as.crystal.character(crystal);
	} else if(CRYSTAL) {
		warning("No 'CRYSTAL' record!");
		crystal = NULL; # TODO
	}
  
	### Connections:
	connect = NULL
	isConnect = grepl("^CONECT", lines);
	if(CONNECT && any(isConnect)) {
		connect = subset(lines, isConnect);
		connect = connect.character(connect, atoms);
		# Re-index: because of TER-record;
		idA = atoms$eleid;
		atoms$eleid = seq_along(idA);
		connect$eleid.1 = match(connect$eleid.1, idA);
		connect$eleid.2 = match(connect$eleid.2, idA);
	} else {
		# match(atoms$eleid, atoms$eleid);
		atoms$eleid = seq_along(atoms$eleid);
	}
	
	### PDB Object:
	pdbObj = pdb(atoms, crystal, connect, remark, title,
		resolution = dfResolution);
	pdbObj = split(pdbObj, model.factor);
	if(length(pdbObj) == 1) pdbObj = pdbObj[[1]];
	
	return(pdbObj)
}

### Helper

trim = function(x)
	sub(' +$', '', sub('^ +', '', x));

extract.regex = function(x, pattern, gr=0, perl=TRUE, simplify=TRUE, verbose=FALSE) {
	if(inherits(x, "data.frame")) stop("x should be an array!");
	r = regexec(pattern, x, perl=perl);
	if(verbose) cat("Finished Regex.\nStarting extraction.\n");
	gr  = gr + 1;
	len = length(gr);
	if(len > 1) {
		s = lapply(seq(length(x)), function(id) {
			tmp = r[[id]];
			if(tmp[1] == -1) return(rep("", len));
			LEN = attr(tmp, "match.length");
			sapply(seq(len), function(idGr) {
				gr = gr[idGr];
				nStart = tmp[gr];
				substr(x[id], nStart, nStart - 1 + LEN[gr]);
			})
		});
		if(simplify) s = do.call("rbind", s);
	} else {
		s = lapply(seq(length(x)), function(id) {
			tmp = r[[id]];
			if(tmp[1] == -1) return("");
			nStart = tmp[gr];
			substr(x[id], nStart, nStart - 1 + attr(tmp, "match.length")[gr]);
		});
		if(simplify) s = unlist(s);
	}
	return(s)
}

extract.pdb.Resolution = function(x) {
	# can start both at +11 & + 12;
	strRes = trim(substr(x, 11, 22));
	isRes  = (strRes == "RESOLUTION.");
	if(any(isRes)) {
			x = x[isRes];
			x = trim(substr(x, 23, 80));
			res  = extract.regex(x, "([0-9.]+) ([A-Z]+)", gr = c(1,2));
			unit = res[,2];
			res  = as.numeric(res[,1]);
			isNA = is.na(res);
			unit[isNA] = x[isNA];
			attr(res, "Unit") = unit;
	} else res = NULL;
	return(res);
}

#' Basic Vectorial Operations
#' 
#' Basic vectorial operations such as scalar product and vectorial product
#' 
#' 
#' @return
#' \itemize{
#' \item{\code{dotProd}  return a single element numeric vector.}
#' \item{\code{vectNorm} return a single element numeric vector.}
#' \item{\code{rotVect}  return a numeric vector of length 3.}
#' \item{\code{vectProd} return a numeric vector of length 3.}
#' }
#' 
#' @param U a numeric vector of length 3.
#' @param V a numeric vector of length 3.
#' @param n an integer.
#' 
#' @seealso \code{\link[base]{matmult}}
#' 
#' @examples 
#' Vx <- c(3,0,0)
#' vectNorm(Vx)
#' Vx <- Vx / vectNorm(Vx)
#' Vy <- c(0,1,0)
#' Vz <- vectProd(Vx, Vy)
#' print(Vz)
#' 
#' @keywords manip
#' 
#' @name vectorialOperations
#' @export
dotProd <- function(U,V){
  if(!(is.vector(U) & length(U)==3 & is.numeric(U)))
    stop("'U' must be a numeric vector of length 3")
  if(!(is.vector(V) & length(V)==3 & is.numeric(V)))
    stop("'V' must be a numeric vector of length 3")
  sum(U*V)
}

#' @rdname vectorialOperations
#' @export
vectNorm <- function(U){
  if(!(is.vector(U) & length(U)==3 & is.numeric(U)))
    stop("'U' must be a numeric vector of length 3")
  sqrt(dotProd(U,U))
}

#' @rdname vectorialOperations
#' @export
rotVect <- function(U, n = 1){
  if(!(is.vector(U) & length(U)==3 & is.numeric(U)))
    stop("'U' must be a numeric vector of length 3")
  if(!(length(n)==1 & round(n)==n))
    stop("'n' must be an integer")
  if(n>0)
    U <- U[c((length(U)-n+1):length(U),1:(length(U)-n))]
  if(n<0)
    U <- U[c((abs(n)+1):length(U),1:abs(n))]
  return(U)
}

#' @rdname vectorialOperations
#' @export
vectProd <- function(U,V){
  if(!(is.vector(U) & length(U)==3 & is.numeric(U)))
    stop("'U' must be a numeric vector of length 3")
  if(!(is.vector(V) & length(V)==3 & is.numeric(V)))
    stop("'V' must be a numeric vector of length 3")
  rotVect(U,-1) * rotVect(V, 1) - rotVect(U, 1) * rotVect(V,-1)
}
  

#' Create \sQuote{connect} Object
#' 
#' Creates an object of class \sQuote{connect} containing the IDs of bonded atoms
#' defining the connectivity of a molecular system.
#' 
#' \code{connect} is a generic function to create objects of class \sQuote{connect}.
#'   The purpose of this class is to store CONNECT records from PDB files,
#'   indicating the connectivity of a molecular system.\cr
#' The default method creates a \code{connect} object from its different components, i.e.: 
#'   \code{eleid.1} and \code{eleid.2}. Both arguments have to be specified.\cr 
#' The S3 method for an object of class \sQuote{coords} determines the connectivity 
#'   from atomic coordinates. A distance matrix is computed, then, for each pair 
#'   of atoms the distance is compared to a bounding distance computed from atomic 
#'   radii. If this distance is lower than the bounding distance then the atoms 
#'   are assumed to be connected.\cr
#' The S3 method for object of class \sQuote{pdb} first uses the element names
#'   to search for atomic radii in the \code{elements} data set.
#'   Then atomic coordinates and radii are passed to \code{connect.coords}.\cr
#'   If \code{by.block == TRUE}, a grid is defined to determined the connectivity by block.
#'   The method is slow but allows to deal with very large systems.\cr
#' The S3 method for object of class \sQuote{character} converts the raw string
#'   from a pdb file into a \code{connect} object.
#'
#' \code{is.connect} tests if an object is of class \sQuote{connect},
#'   i.e. if it has a \dQuote{class} attribute equal to \code{connect}.
#' 
#' @return \code{connect} returns a two-column data.frame of class \sQuote{connect}
#'   whose rows contain the IDs of bonded atoms. The columns of this data.frame
#'   are described below:
#' \item{eleid.1}{an integer vector containing the elements IDs
#'    defining the connectivity of the system.} 
#' \item{eleid.2}{an integer vector containing the elements IDs
#'    defining the connectivity of the system.}\cr\cr
#'
#' \code{is.connect} returns TRUE if \code{x} is an object of class \sQuote{coords}
#'   and FALSE otherwise.
#' 
#' @param \dots arguments passed to methods.
#' @param eleid.1 a integer vector containing the IDs of bonded atoms.
#' @param eleid.2 a integer vector containing the IDs of bonded atoms.
#' @param x,atoms an R object containing atomic coordinates.
#' @param radii a numeric vector containing atomic radii used to find neigbours.
#' @param safety a numeric value used to extend the atomic radii.
#' @param by.block a logical value indicating whether the connectivity has to be
#'   determine by block (see details).
#' @param maxLimit integer value specifying the maximum number of atoms permitted
#'   to try connecting by distance.
#' @param pdbRec the raw text lines from the pdb file.
#'   
#' @seealso \code{\link{pdb}}
#' 
#' @examples 
#' # If atom 1 is connected to atom 2, 3, 4 and 5
#' # then we can prepare the following 'connect' object:
#' x <- connect(rep(1,4), 2:5)
#' print(x)
#' is.connect(x)
#' 
#' # Compute connectivity from coordinates
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"), CONNECT = FALSE)
#' x$connect
#' x$connect <- connect(x)
#' x$connect
#' 
#' @keywords classes
#' 

#' @name connect
# TODO: remove
conect.default = function(eleid.1, eleid.2, ...) {
	if(missing(eleid.2)) {
		if(is.connect(eleid.1)) {
			return(eleid.1);
		} else if(inherits(eleid.1, "data.frame")) {
			class(eleid.1) = c("connect", "data.frame");
			return(eleid.1);
		} else
			stop("Missing eleid2!");
	}
	connect.default(eleid.1, eleid.2, ...);
}

#' @name connect
#' @export
connect <- function(...)
  UseMethod("connect")

#' @rdname connect
#' @export
connect.default <- function(eleid.1, eleid.2, ...)
{
	if(missing(eleid.2)) {
		if(is.connect(eleid.1)) {
			return(eleid.1);
		} else if(inherits(eleid.1, "data.frame")) {
			class(eleid.1) = c("connect", "data.frame");
			return(eleid.1);
		} else
			stop("Missing eleid2!");
	}
	if(is.null(eleid.1) & is.null(eleid.2))
		return(NULL);
	#
	eleid.1 = as.integer(eleid.1);
	eleid.2 = as.integer(eleid.2);
	con = data.frame(eleid.1, eleid.2);
	if(nrow(con) == 0) return(NULL);
	### Order:
	# Swap ?
	# con[con$eleid.1 > con$eleid.2,] <- rev(con[con$eleid.1 > con$eleid.2,])
	id  = do.call(order, con);
	con = con[id,];
	rownames(con) = NULL;
	class(con) = c("connect", "data.frame");
	return(con)
}

### Determine Connectivity by Distance
# Note:
# - Brute force connectivity: slow & NOT very robust;
#' @rdname connect
#' @export
connect.coords <- function(x, radii = 0.75, safety = 1.2,
		by.block = FALSE, maxLimit = 3200, ...) {
	if(! is.coords(x))
		stop("'x' must be an object of class 'coords'");
	if(nrow(x) > maxLimit) {
		warning("Molecule is bigger than maxLimit!",
			"Increase the value for maxLimit or subset only certain atoms.");
		return(NULL);
	}

	radii = radii * safety;
	data  = cbind(x, radii);

  findCon <- function(data) {
    nat <- nrow(data)
    if(nat==0) return(NULL)
    r <- sqrt(
        outer(data$x1, data$x1, "-")^2 +
        outer(data$x2, data$x2, "-")^2 +
        outer(data$x3, data$x3, "-")^2
    )
    bond.dist <- outer(data$radii, data$radii, "+")
    M <- lower.tri(r) & (r < bond.dist)
    if(all(!M)) return(NULL)
    eleid <- matrix(rownames(data), nrow = nat, ncol = nat)   
    eleid.1 <- as.integer(t(eleid)[M])
    eleid.2 <- as.integer(  eleid [M]) 
    return(connect.default(eleid.1, eleid.2))
  }

  if(!by.block) {
    con <- findCon(data)
  }
  else {
    get.con <- function(shift = c(0,0,0), x, radii, step) {
      coords.range <- range(x)
      coords.range <- t(t(coords.range) - shift)

      x.cuts <- seq(coords.range["min","x"], coords.range["max","x"] + step, step)
      y.cuts <- seq(coords.range["min","y"], coords.range["max","y"] + step, step)
      z.cuts <- seq(coords.range["min","z"], coords.range["max","z"] + step, step)
    
      x.index <- cut(x$x1, x.cuts, include.lowest = TRUE)
      y.index <- cut(x$x2, y.cuts, include.lowest = TRUE)
      z.index <- cut(x$x3, z.cuts, include.lowest = TRUE)
      
      f <- interaction(x.index, y.index, z.index)
      
      data <- cbind(x, radii)
      
      con <- split(data, f)
      con <- lapply(con, findCon)
      con <- do.call(rbind, con)

      return(con)
    }

    width <- 10
    shift <- expand.grid(0:1,0:1,0:1) * width/2;
	
	# TODO: check of correct
    con = apply(shift, 1, get.con, x, radii, width)
    con = unique(do.call(rbind, con))
    con = connect(con$eleid.1, con$eleid.2);
    rownames(con) = NULL;
  }
  
  return(con)
}

#' @rdname connect
#' @export
connect.atoms = function(x, safety = 1.2, by.block = FALSE, ...) {
	symbE = x$symbol;
	if(is.null(symbE)) {
		symbE = toSymbols(x$elename);
	}
	symbE[is.na(symbE)] = "Xx";
	idAt = match(symbE, Rpdb::elements[,"symb"]);
	rcov = Rpdb::elements[idAt, "rcov"];
	xyz  = coords(x);
	# Connect within Chains
	ch = chains(x);
	con = lapply(ch, function(ch) {
		isCh = x$chainid == ch;
		xyzj = xyz[isCh, , drop = FALSE];
		if(nrow(xyzj) == 0) return(NULL);
		rj  = rcov[isCh];
		con = connect(xyzj, rj, safety, by.block, ...);
		con = connect(
			x$eleid[con$eleid.1],
			x$eleid[con$eleid.2]);
	})
	con = do.call(rbind, con);
	return(con);
}

#' @rdname connect
#' @export
connect.pdb = function(x, safety = 1.2, by.block = FALSE, ...) {
	con = connect.atoms(x$atoms,
		safety = safety, by.block = by.block, ...);
	return(con);
}


#' @rdname connect
#' @export
connect.character = function(pdbRec, atoms, ...) {
	C0 = as.integer(substr(pdbRec,  7, 11));
	C1 = as.integer(substr(pdbRec, 12, 16));
	C2 = as.integer(substr(pdbRec, 17, 21));
	C3 = as.integer(substr(pdbRec, 22, 26));
	C4 = as.integer(substr(pdbRec, 27, 31));
	C0 = rep(C0, 4);
	C1 = c(C1,C2,C3,C4);
	CN =  ! is.na(C1);
	C0 = subset(C0, CN);
	C1 = subset(C1, CN);
	connect = connect.default(eleid.1 = C0, eleid.2 = C1);
	tokeep  = connect$eleid.1 %in% atoms$eleid & connect$eleid.2 %in% atoms$eleid;
	connect = subset(connect, tokeep);
	return(connect);
}

#' @rdname connect
#' @export
is.connect <- function(x)
{
  to.return = inherits(x, c("connect", "conect"));
  return(to.return)
}

#' Range of Atomic Coordinates
#' 
#' Determines the range of atomic coordinates.
#' 
#' \code{range} computes the range of each of the coordinates.
#' \code{lattice.range} computes the range of the relative coordinates when expressed
#'   in units of lattice vectors.
#' 
#' @return Return a \code{\link{data.frame}} whose columns contain the range of
#'   the first, second and third coordinates of \code{x}.
#'   
#' @param x an R object containing atomic coordinates.
#' @param xyz set of lattice vectors, by default extracted from the PDB object \code{x}.
#' @param na.rm logical, indicating if \code{\link{NA}}'s should be omitted.
#' @param finite logical, indicating if all non-finite elements should be omitted.
#' @param \dots further arguments passed to or from other methods. 
#'
#' @seealso \code{range}, \code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}}
#' 
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' range(x)
#' range(range(x))
#' 
#' @keywords manip


### Functions:

#' @name range.coords
#' @export
range.coords <- function(x, na.rm = FALSE, finite = FALSE, ...)
{
	if(! is.coords(x)) stop("'x' must be an object of class 'coords'");
	
	to.return = lapply(x[, c("x1","x2","x3")], range, na.rm, finite);
	to.return = as.data.frame(to.return, row.names = c("min","max"));

  c.names <- unlist(strsplit(basis(x), ""))
  colnames(to.return) <- c.names
  
  return(to.return)
}

#' @rdname range.coords
#' @export
range.atoms <- function(x, na.rm = FALSE, finite = FALSE, ...)
{
  if(!is.atoms(x)) stop("'x' must be an object of class 'atoms'")
  
  to.return <- range.coords(coords(x), na.rm, finite)
  return(to.return)
}

#' @rdname range.coords
#' @export
range.pdb = function(x, na.rm = FALSE, finite = FALSE, ...)
{
	if(! is.pdb(x)) stop("'x' must be an object of class 'pdb'");
	
	res = range.coords(coords(x), na.rm=na.rm, finite=finite);
	return(res);
}


### Range of Coordinates as Lattice Vectors

# Hack to enable methods(lattice);
lattice = function() {}

#' @name range.coords
#' @export lattice.range
lattice.range = function(x, xyz = NULL, ..., na.rm = TRUE)
	UseMethod("lattice.range");

#' @rdname range.coords
#' @exportS3Method  lattice.range pdb
lattice.range.pdb = function(x, xyz = NULL, ..., na.rm = TRUE) {
	range.lattice.pdb(x, xyz=xyz, ..., na.rm=na.rm);
}


#' @rdname range.coords
#' @method range lattice
#' @export range.lattice
range.lattice = function(x, ...) {
	UseMethod("range.lattice");
}

#' @rdname range.coords
#' @method range.lattice pdb
#' @exportS3Method range.lattice pdb
range.lattice.pdb = function(x, xyz = NULL, ..., na.rm = TRUE) {
	if(is.null(xyz)) {
		cell = get.PDBCrystal(x);
		xyzV = cell.coords(cell)
	} else xyzV = xyz;
	xyzE = coords(x$atoms);
	tt = solve(xyzV, t(xyzE));
	tt = apply(tt, 1, range, na.rm=na.rm);
	# xyz = xyzV %*% t(tt);
	return(tt);
}

#' Projections of Objects
#' 
#' Computes 3D Projections
#' 
#' The purpose of the \sQuote{proj} helper functions is to compute
#' the spatial projections of various objects on other geometrical structures.
#' Currently, only projection of points on a line segment are implemented.
#' 
#' The methods for the \sQuote{proj.line3d} function compute the projections
#' of 1 point or a collection of points on a line segment defined by the 2
#' delimiting points.
#' 
#' The \sQuote{numeric} method projects a point, while the \sQuote{matrix} method
#' projects a collection of points on another geometric object.
#' 
#' @return The \code{proj.line3d} function returns an object
#' of class \sQuote{proj} containing the 3-dimensional coordinates of
#' the projected point, as well as a scalar value representing the fraction
#' of the path on the line segment.
#' 
#' @param p numeric array or matrix with the x,y,z coordinates of the point or collection of points.
#' @param x,y,z an R object containing the coordinates of the end-points of the line segment; can be specified either as individual values or as a matrix.
#' @param \dots currently not used.
#' 
#' @examples
#' p = c(1,2,3)
#' line = matrix(c(0,5,2,3,1,4), nrow = 2)
#' proj.line3d(p, line)
#' 
#' @keywords classes manip
#' 
#' @name proj.line3d
#' @export proj.line3d
proj.line3d <- function(p, x, y, z, ...)
  UseMethod("proj.line3d")

#' @rdname proj.line3d
#' @exportS3Method  proj.line3d numeric
proj.line3d.numeric = function(p, x, y = NULL, z = NULL, ...) {
	if(is.null(y)) {
		y = x[,2]; z = x[,3]; x = x[,1];
	}
	dx = x[2] - x[1]; dx0 = p[1] - x[1];
	dy = y[2] - y[1]; dy0 = p[2] - y[1];
	dz = z[2] - z[1]; dz0 = p[3] - z[1];
	tt = (dx*dx0 + dy*dy0 + dz*dz0);
	tt = tt / (dx^2 + dy^2 + dz^2);
	t1 = 1 - tt;
	px = t1*x[1] + tt*x[2];
	py = t1*y[1] + tt*y[2];
	pz = t1*z[1] + tt*z[2];
	lst = list(x = px, y = py, z = pz, t = tt);
	class(lst) = c("proj", "list");
	return(lst);
}

#' @rdname proj.line3d
#' @exportS3Method  proj.line3d matrix
proj.line3d.matrix = function(p, x, y = NULL, z = NULL, ...) {
	if(is.null(y)) {
		y = x[,2]; z = x[,3]; x = x[,1];
	}
	dx = x[2] - x[1]; dx0 = p[, 1] - x[1];
	dy = y[2] - y[1]; dy0 = p[, 2] - y[1];
	dz = z[2] - z[1]; dz0 = p[, 3] - z[1];
	tt = (dx*dx0 + dy*dy0 + dz*dz0);
	tt = tt / (dx^2 + dy^2 + dz^2);
	t1 = 1 - tt;
	px = t1*x[1] + tt*x[2];
	py = t1*y[1] + tt*y[2];
	pz = t1*z[1] + tt*z[2];
	lst = list(x = px, y = py, z = pz, t = tt);
	class(lst) = c("proj", "list");
	return(lst);
}

#' The Atomic Coordinates of an Object
#' 
#' Get or set the atomic coordinates (either Cartesian or fractional 
#' coordinates) of an object.
#' 
#' The purpose of the \sQuote{coords} class is to store the coordinates of a 
#' molecular system and facilitate their manipulation when passing from the 
#' Cartesian to fractional coordinates and vice versa.\cr
#' \code{coords} and \code{coords<-} are generic accessor and replacement
#' functions. \code{as.coords} is an alias to \code{coords}.\cr
#' The default method of the \code{coords} function is actually a builder allowing 
#' to create a \sQuote{coords} object from its different components, i.e.: 
#' \code{x1}, \code{x2}, \code{x3}, and \code{basis}. All the arguments have to 
#' be specified except 'basis' which by default is set to "xyz" (Cartesian 
#' coordinates). \cr\cr
#' For an object of class \sQuote{atoms}, the accessor function
#' extracts its \code{x1}, \code{x2} and \code{x3} components as well 
#' as its \code{basis} attribute to create a \sQuote{coords} object.
#'
#' The replacement function sets \code{x1}, \code{x2} and \code{x3} components,
#' as well as the \code{basis} attribute. \cr\cr
#' For an object of class \sQuote{coords}, the accessor function
#' returns the \sQuote{coords} object as is.
#'
#' For an object of class \sQuote{pdb}, the accessor function extracts the \code{x1},
#' \code{x2} and \code{x3} components, as well as the \code{basis} attribute of its 
#' \code{atoms} component to create a \sQuote{coords} object. The replacement 
#' function sets the \code{x1}, \code{x2} and \code{x3} components as well as the
#' \code{basis} attribute of its \code{atoms} component. \cr\cr
#'
#' For \sQuote{matrix} and \sQuote{data.frame} objects, when \code{basis == NULL}
#' this function searches x, y, z or a, b, c columns in \code{x}.\cr
#' If x, y, z columns are found, they are used to set the first, second and
#' third coordinates of the returned \sQuote{coords} object.
#' In that case the basis set of \code{x} is set to \code{"xyz"}.\cr
#' If a, b, c columns are found they are used to a 
#' set the first, second and third coordinates of the returned \sQuote{coords} 
#' object. In that case the basis set of \code{x} is set to \code{"abc"}.\cr If 
#' the function doesn't found neither the x, y, z nor the a, b, c columns an 
#' error is returned.\cr When \code{basis!=NULL} it has to be equal to 
#' \code{"xyz"} or \code{"abc"} and \code{x} must have exactly 3 columns. \cr\cr
#' \code{is.coords} tests if x is an object of class \sQuote{coords}, i.e. if x 
#' has a \dQuote{class} attribute equal to \code{coords}.
#' 
#' @return The accessor function returns a data.frame of class \sQuote{coords}
#' whose columns contain the three coordinates of the atoms of a molecular
#' system. The coordinates can either be Cartesian (\code{basis} attribute equal
#' to \code{"xyz"}) or fractional coordinates (\code{basis} attribute equal to
#' \code{"abc"}). \cr\cr The replacement function returns an object of the same
#' class as \code{x} with updated coordinates. \cr\cr \code{is.coords} returns
#' TRUE if x is an object of class \sQuote{coords} and FALSE otherwise
#' 
#' @param x1,x2,x3 numeric vectors containing the first, second and third coordinates.
#' @param basis a single element character vector indicating the type of basis vector
#'   used to express the atomic coordinates.
#' @param x an R object containing atomic coordinates.
#' @param value an object of class \sQuote{coords} used for replacement
#' @param \dots further arguments passed to or from other methods.
#' 
#' @seealso 
#' \code{\link{basis}}
#' 
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' is.coords(x)
#' is.coords(x$atoms)
#' 
#' ## Replace the coordinates of x by translated coordinates
#' coords(x) <- coords(Tz(x, 10))
#' coords(x)
#' 
#' @keywords classes manip
#' 
#' @name coords
#' @export
coords <- function(...)
  UseMethod("coords")

#' @rdname coords
#' @export
'coords<-' = function(x, value)
  UseMethod("coords<-", x)

#' @rdname coords
#' @export
'as.coords' = function(...) {
	UseMethod("coords");
}

#' @rdname coords
#' @export
coords.default <- function(x1, x2, x3, basis = "xyz", ...)
{
  if(!basis %in% c("xyz", "abc")) stop("Unrecognized basis")
  
  to.return <- data.frame(x1,x2,x3)
  attr(to.return, which = "basis") <- basis
  
  class(to.return) <- c("coords", "data.frame")
  
  return(to.return)
}

#' @rdname coords
#' @export
coords.data.frame <- function(x, basis = NULL, ...)
{
  if(!is.data.frame(x)) stop("'x' must be a 'data.frame'")
  
  if(is.null(basis)){
    if(all(c("x","y","z") %in% names(x))){
      x <- x[, c("x","y","z")];
      basis <- "xyz";
    }
    else if(all(c("a","b","c") %in% names(x))){
      x <- x[,c("a","b","c")]
      basis <- "abc"
    }
    else stop("Can not convert this 'data.frame' into 'coords': Coordinates not found")
  }
  else if( ! basis %in% c("xyz","abc")) stop("Unrecognized 'basis'");
  if(ncol(x) != 3L) stop("'x' must be a three-columns data.frame")

  to.return <- coords.default(x[,1], x[,2], x[,3], basis = basis)

  return(to.return)
}

#' @rdname coords
#' @export
coords.matrix <- function(x, basis = NULL, ...){
  if( ! is.matrix(x)) stop("'x' must be a 'matrix'");
  
  to.return <- coords.data.frame(as.data.frame(x), basis = basis, ...)
  return(to.return)
}

#' @rdname coords
#' @export
coords.coords <- function(x, ...) {
  if( ! is.coords(x)) stop("'x' must be a 'coords'-object");
  return(x);
}

#' @rdname coords
#' @export
coords.atoms <- function(x, ...)
{
  if(!is.atoms(x)) stop("'x' must be an object of class 'atoms'")
  
  to.return <- coords(x$x1,x$x2,x$x3,basis.default(x))
  return(to.return)
}

#' @rdname coords
#' @export
'coords<-.atoms' <- function(x, value)
{
  if(!is.atoms(x)) stop("'x' must be an object of class 'atoms'")
  
  if(!is.coords(value)) stop("'value' must be an object of class 'coords'")
  if(nrow(x) != nrow(value)) stop(paste("arguments imply different number of rows: ",nrow(x),", ",nrow(value),sep=""))
  x[c("x1","x2","x3")] <- value
  basis(x) <- basis.default(value)
  
  return(x)
}

#' @rdname coords
#' @export
coords.pdb <- function(x, ...)
{
  if(!is.pdb(x)) stop("'x' must be an object of class 'pdb'")
  to.return <- coords.atoms(x$atoms)
  return(to.return)
}

#' @rdname coords
#' @export
'coords<-.pdb' <- function(x, value)
{
  if( ! is.pdb(x)) stop("'x' must be an object of class 'pdb'");
  
  if( ! is.coords(value)) stop("'value' must be an object of class 'coords'");
  if(nrow(x$atoms) != nrow(value)) {
		stop(paste("arguments imply different number of rows: ",
			nrow(x$atoms), ", ", nrow(value), sep=""));
	}
  coords(x$atoms) <- value
  
  return(x)
}

#' @rdname coords
#' @export
is.coords <- function(x)
{
  to.return <- inherits(x, "coords");
  return(to.return)
}

#' Merging Molecular Systems
#' 
#' Merge two objects containing atomic coordinates
#' 
#' To merge \code{x} and \code{y} they must have the same \code{basis} 
#' attributes (see \code{\link{basis}}). \cr\cr For objects of class 
#' \sQuote{coords} and \sQuote{atoms} the atomic coordinates are directly merged
#' by row. \cr\cr For objects of class \sQuote{pdb}, the \code{atoms} and 
#' \code{connect} components of the two \code{pdb} objects are merged by row and 
#' the \code{crystal} component of \code{x} is used to build the returned 
#' object. \cr\cr For objects of class \sQuote{atoms} and \sQuote{pdb} the 
#' residue and element IDs of \code{y} are shifted to avoid any confusion with 
#' those of \code{x}. If \code{reindex == TRUE} the \code{\link{reindex}} function
#' is called to reinitialize the indexing of the returned object.
#' 
#' @return Return an object of the same class as \code{x} and \code{y} merging 
#'   \code{x} and \code{y}. If \code{x} and \code{y} have different \code{basis}
#'   attributes an error is returned.
#'   
#' @param x,y objects of class \code{coords} to be merged.
#' @param reindex a single element logical vector indicating if residue and
#'   element IDs have to be reindexed after merging.
#' @param \dots further arguments passed to or from other methods.
#'   
#' @seealso \code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}},
#'   \code{\link{basis}}, \code{merge}, \code{merge.data.frame}
#'   
#' @examples 
#' c1 <- coords( 1:3 ,  4:6 ,  7:9 , basis = "xyz")
#' c2 <- coords(10:12, 13:15, 16:18, basis = "xyz")
#' merge(c1,c2)
#' 
#' \donttest{
#' ## Merging objects with different basis sets returns an error.
#' c2 <- coords(9:11, 12:14, 15:17, basis = "abc")
#' try(merge(c1,c2))
#' }
#' 
#' ## Prepare a Pentacene/C70 dimer
#' C70 <- read.pdb(system.file("examples/C70.pdb", package="Rpdb"))
#' Pen <- read.pdb(system.file("examples/Pentacene.pdb", package="Rpdb"))
#' x <- merge(Tz(C70, 3.5, thickness=0.5), Pen)
#'   
#' @keywords manip
#' 
#' @name merge.coords
#' @export
merge.coords <- function(x, y, ...)
{
  if(!is.coords(x)) stop("'x' must be an object of class 'coords'")
  if(!is.coords(y)) stop("'y' must be an object of class 'coords'")
  
  if(basis(x) != basis(y)) stop("'x' and 'y' basis differ")
  
  to.return <- rbind(x,y)
  return(to.return)
}

#' @rdname merge.coords
#' @export
merge.atoms <- function(x, y, reindex = TRUE, ...)
{
  if(! is.atoms(x)) stop("'x' must be an object of class 'atoms'");
  if(! is.atoms(y)) stop("'y' must be an object of class 'atoms'");
  
  if(basis(x) != basis(y)) stop("'x' and 'y' basis differ")
  
  y.resid <- as.integer(y$resid + max(x$resid))
  y.eleid <- as.integer(y$eleid + max(x$eleid))
  
  y$resid <- y.resid
  y$eleid <- y.eleid
  
  to.return <- rbind(x,y)
  if(reindex) to.return <- reindex.atoms(to.return)
  
  return(to.return)
}

#' @rdname merge.coords
#' @export
merge.pdb <- function(x, y, reindex = TRUE, ...)
{
	if(! is.pdb(x)) stop("'x' must be an object of class 'pdb'");
	if(! is.pdb(y)) stop("'y' must be an object of class 'pdb'");
	
	if(basis(x) != basis(y)) stop("'x' and 'y' basis differ")
	
	xcryst = x$crystal; ycryst = y$crystal;
	if( any(xcryst$abc != ycryst$abc) ||
		any(xcryst$abg != ycryst$abg) ||
		any(xcryst$sgroup != ycryst$sgroup))
		warning("Different periodical boundary conditions are defined for 'x' and 'y'.",
			"'x$crystal' has been kept.");
  
	title  = unique(c(x$title , y$title ));
	remark = unique(c(x$remark, y$remark));
	crystal = x$crystal;
	
	y$connect$eleid.1 <- y$connect$eleid.1 + max(x$atoms$eleid)
	y$connect$eleid.2 <- y$connect$eleid.2 + max(x$atoms$eleid)
	
	eleid.1 = c(x$connect$eleid.1, y$connect$eleid.1);
	eleid.2 = c(x$connect$eleid.2, y$connect$eleid.2);
	connect = connect.default(eleid.1, eleid.2);

  atoms <- merge.atoms(x$atoms, y$atoms, reindex = FALSE)
  
  to.return = pdb(atoms, crystal, connect, remark, title);
  if(reindex) to.return = reindex.pdb(to.return);
  
  return(to.return)
}

#'
#' Set the View of the \sQuote{rgl} Scene
#' 
#' Set the view of the current \sQuote{rgl} scene aligning one vector 
#' perpendicularly to the screen and placing another one in the horizontal plane.
#' 
#' \code{viewAxis} set the view of the current rgl scene (by setting
#' \code{UserMatrix}; see \code{\link[rgl]{par3d}} for more details) so that
#' \code{V1} is perpendicular to the screen and \code{V2} is in the horizontal
#' plane. The other functions documented here are helper functions calling
#' \code{viewAxis} to set the view using particular Cartesian or lattice
#' vectors. For functions \code{viewAB}, \code{viewBC} and \code{viewCA} a
#' \sQuote{crystal} object has to be specified to define the lattice vectors
#' used to set the view. The function \code{viewInertia} computes the inertia
#' tensor from the atomic coordinates and masses (see \code{\link{inertia}})
#' and sets the view to its eigen vectors basis set.
#' 
#' @param V1 a length 3 numeric vector.
#' @param V2 a length 3 numeric vector.
#' @param crystal an object of class \sQuote{crystal}.
#' @param x an R object containing atomic coordinates.
#' @param m a numeric vector containing atomic masses.
#'
#' @return No return value, called for side effects.
#' 
#' @seealso \code{\link{visualize}}, \code{\link{cell.coords}}, \code{\link[rgl]{par3d}}, \code{\link[rgl]{rgl.open}}
#' 
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' visualize(x, mode = NULL)
#' viewAB(x$crystal)
#' 
#' C70 <- read.pdb(system.file("examples/C70.pdb", package="Rpdb"))
#' visualize(C70, mode = NULL)
#' viewXY()
#' viewInertia(C70)
#' 
#' @keywords dynamic
#' 
#' @name viewAxis
#' @export
viewAxis <- function(V1, V2){
	if(is.list(V1) || is.list(V2))
		stop("'V1' and 'V2' can not be lists!");
	if(length(V1) != 3 || length(V2) != 3)
		stop("'V1' and 'V2' must be of length 3!");
	if(vectNorm(V1) == 0 || vectNorm(V2) == 0)
		stop("'V1' and 'V2' can not be null vectors!");
	#
	V1 = matrix(c(V1, 0))
	V2 = matrix(c(V2, 0))
	A  = acos(V1[1] / sqrt(sum(V1[c(1,2)]^2)))
	A  = ifelse(V1[2] > 0, -A, A) # TODO: simple IF
	R1 = rgl::rotationMatrix(A, 0, 0, 1)
	V1 = R1 %*% V1;
	V2 = R1 %*% V2;
	A  = acos(V1[3] / sqrt(sum(V1[c(1,3)]^2)))
	A  = ifelse(V1[1] > 0, -A, A)
	R2 = rotationMatrix(A, 0, 1, 0)
	Vz = R2 %*% V1;
	V2 = R2 %*% V2;
	A  = acos(V2[1] / sqrt(sum(V2[c(1,2)]^2)))
	A  = ifelse(V2[2] > 0, -A, A)
	R3 = rgl::rotationMatrix(A, 0, 0, 1);
	#
	rgl::par3d(userMatrix = R3 %*% R2 %*% R1)
}

#' @rdname viewAxis
#' @export
viewXY <- function()
	viewAxis(c(1,0,0), c(0,1,0));

#' @rdname viewAxis
#' @export
viewYZ <- function()
	viewAxis(c(0,1,0), c(0,0,1));

#' @rdname viewAxis
#' @export
viewZX <- function()
	viewAxis(c(0,0,1), c(1,0,0));

#' @rdname viewAxis
#' @export
viewAB <- function(crystal) {
	if(missing(crystal))
		stop("Please specify 'crystal' to define the lattice vectors used to set the view");
	cell = cell.coords.crystal(crystal);
	viewAxis(cell[,"a"], cell[,"b"]);
}

#' @rdname viewAxis
#' @export
viewBC <- function(crystal) {
	if(missing(crystal))
		stop("Please specify 'crystal' to define the lattice vectors used to set the view");
	cell = cell.coords.crystal(crystal);
	viewAxis(cell[,"b"], cell[,"c"]);
}

#' @rdname viewAxis
#' @export
viewCA <- function(crystal) {
	if(missing(crystal))
		stop("Please specify 'crystal' to define the lattice vectors used to set the view");
	cell = cell.coords.crystal(crystal);
	viewAxis(cell[,"c"], cell[,"a"]);
}

#' @rdname viewAxis
#' @export
viewInertia <- function(x, m = NULL){
  M <- diag(4)
  M[1:3, 1:3] <- eigen(inertia(x, m))$vectors
  rgl::par3d(userMatrix = M)
}

#' Subsetting \sQuote{atoms} and \sQuote{pdb} Objects
#' 
#' Return subsets of \sQuote{atoms} or \sQuote{pdb} objects which meet
#' conditions.
#' 
#' For an \sQuote{atoms} object the method is similar to the data.frame method
#' (see \code{\link{subset}}) but allow to directly reindex the elements and
#' residues IDs. For a \sQuote{pdb} object subsetting is applied on the
#' \code{atoms} and \code{connect} components of the object in a consistent way.
#' First the \code{atoms} component is subsetted and then the \code{connect}
#' component is filtered to keep only the connectivity for the subset.
#' 
#' @return
#' Return a subsetted object of the same class as \code{x}.
#' 
#' @param x object to be subsetted.
#' @param subset logical expression indicating elements or rows to keep: missing values are taken as false.
#' @param drop passed on to [ indexing operator.
#' @param reindex.all a single element logical vector indicating if residues and elements IDs have to be reindexed after subsetting.
#' @param \dots further arguments to be passed to or from other methods.
#' 
#' @seealso \code{\link[base]{subset}}, \code{\link{pdb}}, \code{\link{atoms}}, \code{\link{reindex}}
#' 
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' y <- subset(x, x$atoms$eleid %in% sample(x$atoms$eleid, 10))
#' is(y)
#' y <- subset(x$atoms, x$atoms$eleid %in% sample(x$atoms$eleid, 10))
#' is(y)
#' x <- coords(x)
#' y <- subset(x, x < 0)
#' is(y)
#' 
#' @keywords manip
#' 
#' @name subset.atoms
#' @export
subset.atoms <- function(x, subset, drop = FALSE, reindex.all = TRUE, ...)
{
  if (missing(subset)) 
    r <- TRUE
  else
  {
    e <- substitute(subset)
    r <- eval(e, x, parent.frame())
    if (!is.logical(r)) 
      stop("'subset' must evaluate to logical")
    r <- r & !is.na(r)
  }
  x <- x[r, , drop = drop]
  
  if(nrow(x) == 0) x <- NULL
  
  if(reindex.all) x <- reindex.atoms(x)
  return(x)
}

#' @rdname subset.atoms
#' @export
subset.pdb <- function(x, subset, drop = FALSE, reindex.all = TRUE, ...)
{
  if (missing(subset)) 
    r <- TRUE
  else
  {
    e <- substitute(subset)
    r <- eval(e, x$atoms, parent.frame())
    if (!is.logical(r)) 
      stop("'subset' must evaluate to logical")
    r <- r & !is.na(r)
  }
  if(any(r)) x$atoms <- x$atoms[r, , drop = drop]
  else x["atoms"] <- list(NULL)
  r <-     x$connect$eleid.1 %in% x$atoms$eleid;
  r <- r & x$connect$eleid.2 %in% x$atoms$eleid;
  if(any(r)) {
	x$connect = x$connect[r,];
  } else x["connect"] = list(NULL);
  
  if(reindex.all) x <- reindex.pdb(x)
  
  return(x)
}

#' Create \sQuote{atoms} Object
#' 
#' Creates an object of class \sQuote{atoms} containing the data related to ATOM
#' and HETATM records of a PDB file.
#' 
#' \code{atoms} is a generic function to create objects of class \sQuote{atoms}.
#' The purpose of this class is to store ATOM and HETATM records from PDB files.
#'
#' The default method creates an \code{atoms} object from its different components, i.e.:
#'   \code{recname}, \code{eleid}, \code{elename}, \code{alt},
#'   \code{resname}, \code{chainid}, \code{resid}, \code{insert},
#'   \code{x1}, \code{x2}, \code{x3}, \code{occ}, \code{temp},
#'   \code{segid} and \code{basis}.
#' All the arguments have to be specified, except \code{basis}, which by default
#'   is set to "xyz" (Cartesian coordinates).\cr
#' \code{is.atoms} tests if an object is of class \sQuote{atoms}, i.e.
#'   it only checks that the object inherits the \dQuote{class} \code{atoms}.
#' 
#' @return 
#' \code{atoms} returns a data.frame of class \sQuote{atoms} with the following components:
#' \describe{
#' \item{recname}{a character vector containing the record name for each element.}
#' \item{eleid}{a integer vector containing the element ID for each element.}
#' \item{elename}{a character vector containing the element name for each element.}
#' \item{alt}{a character vector containing the alternate location indicator for each element.}
#' \item{resname}{a character vector containing the residue name for each element.}
#' \item{chainid}{a character vector containing the chain ID for each element.}
#' \item{resid}{a integer vector containing the residue ID for each element.}
#' \item{insert}{a character vector containing the codes for insertion of residue for each element.}
#' \item{x1, x2, x3}{a numeric vector containing the first, second and third coordinate for each element.}
#' \item{occ}{a numeric vector containing the occupencie for each element.}
#' \item{temp}{a numeric vector containing the temperature factor for each element.}
#' \item{segid}{a character vector containing the segment ID for each element.}
#' \item{basis}{a single element character vector indicating the type of basis vector used to express the atomic coordinates.}
#' }
#' \code{is.atoms} returns TRUE if \code{x} is an object of class \sQuote{atoms} and FALSE otherwise.
#' 
#' @param \dots arguments passed to methods.
#' @param recname a character vector containing the record name for each element.
#' @param eleid a integer vector containing the element ID for each element.
#' @param elename a character vector containing the element name for each element.
#' @param alt a character vector containing the alternate location indicator for each element.
#' @param resname a character vector containing the residue name for each element.
#' @param chainid a character vector containing the chain ID for each element.
#' @param resid a integer vector containing the residue ID for each element.
#' @param insert a character vector containing the codes for insertion of residue of each element.
#' @param x1,x2,x3 a numeric vector containing the first, second and third coordinate for each element.
#' @param occ a numeric vector containing the occupancie for each element.
#' @param temp a numeric vector containing the temperature factor for each element.
#' @param segid a character vector containing the segment ID for each element.
#' @param basis a single element character vector indicating the type of basis vector used to express the atomic coordinates.
#' @param symbol the chemical symbol of the atom.
#' @param isHetero logical indicating if the record corresponds to the non-protein ligands.
#' @param x an R object to be tested or from whom to extract the \code{atoms} component.
#' 
#' @seealso \code{\link{basis}}, \code{\link{coords}}, \code{\link{pdb}}
#' 
#' @examples 
#' x <- atoms(recname = c("ATOM","ATOM"), eleid = 1:2, elename = c("H","H"), alt = "",
#'   resname = c("H2","H2"), chainid = "", resid = c(1,1), insert = "",
#'   x1 = c(0,0), x2 = c(0,0), x3 = c(0,1), occ = c(0.0,0.0), temp = c(1.0,1.0),
#'   segid = c("H2","H2"))
#' print(x)
#' is.atoms(x)
#' 
#' @keywords classes
#' 
#' @import rgl
#' @importFrom grDevices rgb
#' 
#' @name atoms
#' @export
atoms <- function(...)
  UseMethod("atoms")

#' @rdname atoms
#' @export
atoms.pdb = function(x, ...) {
	return(x$atoms);
}

#' @rdname atoms
#' @export
atoms.atoms = function(x, ...) {
	if(inherits(x, "atoms")) {
		return(x);
	}
	if(inherits(x, "data.frame")) {
		if(check.atoms(x)) {
			class(x) = c("atoms", class(x));
			return(x);
		}
	}
	stop("x must be an object of class 'atoms'!");
}

#' @rdname atoms
#' @export
atoms.default <- function(recname, eleid, elename, alt,
					resname, chainid, resid, insert,
					x1, x2, x3, occ, temp, segid,
					basis = "xyz", symbol = NULL, isHetero = NULL, ...)
{
  recname <- as.character(recname)
  eleid   <- suppressWarnings(as.integer(eleid))
  elename <- as.character(elename)
  alt     <- as.character(alt)
  resname <- as.character(resname)
  chainid <- as.character(chainid)
  resid   <- suppressWarnings(as.integer(resid))
  insert  <- as.character(insert)
  x1      <- suppressWarnings(as.numeric(x1))
  x2      <- suppressWarnings(as.numeric(x2))
  x3      <- suppressWarnings(as.numeric(x3))
  occ     <- suppressWarnings(as.numeric(occ))
  temp    <- suppressWarnings(as.numeric(temp))
  segid   <- as.character(segid)
  
  if(any(is.na(eleid))) warning("In 'atoms': 'eleid' contains NA values")
  if(any(is.na(resid))) warning("In 'atoms': 'resid' contains NA values")
  if(any(is.na(   x1))) warning("In 'atoms':    'x1' contains NA values")
  if(any(is.na(   x2))) warning("In 'atoms':    'x2' contains NA values")
  if(any(is.na(   x3))) warning("In 'atoms':    'x3' contains NA values")
  if(any(is.na(  occ))) warning("In 'atoms':   'occ' contains NA values")
  if(any(is.na( temp))) warning("In 'atoms':  'temp' contains NA values")
  
  atoms <- data.frame(
    recname = recname,
    eleid   = eleid  ,
    elename = elename,
    alt     = alt    ,
    resname = resname,
    chainid = chainid,
    resid   = resid  ,
    insert  = insert ,
    x1      = x1     ,
    x2      = x2     ,
    x3      = x3     ,
    occ     = occ    ,
    temp    = temp   ,
    segid   = segid  ,
    stringsAsFactors = FALSE
  )
	### Protein vs Non-Protein:
	# Note: atoms$recname is this field!
	if(is.null(isHetero)) {
		isHetero = atoms$recname == "HETATM";
	}
	atoms$Hetero = isHetero;
	# Chemical Symbols:
	if(is.null(symbol)) {
		# C, N, O, S, (Se)?
		symbol = substr(atoms$elename, 1, 1);
		symbol[isHetero] = sub("[0-9].*$", "",
			atoms$elename[isHetero]);
	}
	if(! is.null(symbol)) atoms$symbol = symbol;
	# Atoms Object:
	attr(atoms, "basis") = basis;
	class(atoms) = c("atoms", "coords", "data.frame");
	return(atoms);
}

# Read atoms from raw pdb text lines;
as.atoms.character = function(atoms, isHetero = NULL) {
	### Atoms:
	recAtom <- trim(substr(atoms,  1,  6))
	eleid   <- trim(substr(atoms,  7, 11))
	elename <- trim(substr(atoms, 13, 16))
	alt     <- trim(substr(atoms, 17, 17))
	resname <- trim(substr(atoms, 18, 21))
	chainid <- trim(substr(atoms, 22, 22))
	resid   <- trim(substr(atoms, 23, 26))
	insert  <- trim(substr(atoms, 27, 27))
	x1      <-      substr(atoms, 31, 38)
	x2      <-      substr(atoms, 39, 46)
	x3      <-      substr(atoms, 47, 54)
	occ     <-      substr(atoms, 55, 60)
	temp    <-      substr(atoms, 61, 66)
	segid   <- trim(substr(atoms, 73, 75))
	# Atomic Symbol:
	symbol  = trim(substr(atoms, 77, 78));
	symbol  = sub("\\d+", "", symbol); # Old PDB format;
	# TODO: ugly extraction;
	if(all(nchar(symbol) == 0)) symbol = NULL;
	
	atoms = atoms.default(recAtom, eleid, elename, alt,
				resname, chainid, resid, insert,
				x1, x2, x3, occ, temp, segid,
				basis = "xyz", symbol = symbol, isHetero = isHetero);
	return(atoms);
}

#' @rdname atoms
#' @export
is.atoms <- function(x)
{
  to.return <- any(attr(x,which="class") == "atoms")
  return(to.return)
}

### Helper:

# Minimal Data:
check.atoms = function(x) {
	isAtoms = c("eleid", "elename", "resid", "resname",
		"x1", "x2", "x3") %in% names(x);
	isAtoms = all(isAtoms);
	return(isAtoms);
}

#' Connects atoms in Nucleic acids
#' 
#' Builds the skeleton for a nucleic acid, by connecting the individual atoms
#' for each nucleotide.
#'
#' \code{connect.nucleic} is a generic function with methods for
#'   \code{pdb} and \code{atoms} objects.
#' It creates object of class \sQuote{connect} with the skeleton of each nucleotide.

#' @return
#' \code{connect.nucleic} returns an object of class \code{connect} containing
#'   the explicit connections between all nucleotide atoms.
#' Note: the detailed format of the returned value may change in the future.

#' @param x a \code{pdb} of \code{atoms} object containing atomic identifiers.
#' @param \dots arguments passed to methods.

#' 
#' @seealso \code{\link{pdb}}, \code{\link{connect}}
#' 
#' @examples
#' # Download some pdb files:
#  #   9J4O = tRNA Leu;
#' #   7EQJ = tRNA Val;
#' # x = read.pdb("9J4O.pdb")
#' # visualize(atoms(x), connect = connect.nucleic(x), type="l", pbc = FALSE);


#' @name connect.nucleic
#' @method connect nucleic
#' @exportS3Method connect nucleic
#' @export connect.nucleic
connect.nucleic = function(x, ...)
  UseMethod("connect.nucleic");

#' @rdname connect.nucleic
#' @method connect.nucleic pdb
#' @exportS3Method connect.nucleic pdb
connect.nucleic.pdb = function(x, ...) {
	connect.nucleic.atoms(atoms(x), ...);
}

#' @rdname connect.nucleic
#' @method connect.nucleic atoms
#' @exportS3Method connect.nucleic atoms
connect.nucleic.atoms = function(x, ...) {
	atoms = x;
	# Older variants:
	atoms$elename = sub("\\*", "'", atoms$elename);
	# Duplicate resid:
	lvl = unique(atoms$insert);
	lvl = sort(lvl[nchar(lvl) > 0]);
	if(length(lvl) == 0) {
		hasDuplicates = FALSE;
	} else {
		hasDuplicates = TRUE;
		lvl = c("", lvl);
		atoms$insert = ordered(atoms$insert, levels = lvl);
	}
	isNc = atoms$resname %in% c("A","C","G", "T", "U");
	# Process each chain:
	ch = chains(atoms);
	idBB = lapply(ch, function(ch) {
		isCh = atoms$chainid == ch & isNc;
		tmp  = atoms[isCh, c("resid", "insert", "eleid", "elename", "resname")];
		if(nrow(tmp) == 0) return(NULL);
		#
		idGroup = if(hasDuplicates) list(tmp$insert, tmp$resid) else tmp$resid;
		idAll   = tapply(tmp, idGroup, connectNc, simplify = FALSE);
		if(is.null(idAll)) return(NULL);
		# TODO:
		idNc = lapply(idAll, function(x) rbind(x$BB, x$N, x$Jn));
		idNc = do.call(rbind, idNc);
		# BB-Joins:
		idJb = lapply(idAll, function(x) x$Jb);
		idJb = unlist(idJb);
		if(! is.null(idJb) && length(idJb) > 3) {
			idJb = data.frame(
				E1 = idJb[seq(2, length(idJb) - 1, by = 2)],
				E2 = idJb[seq(3, length(idJb), by = 2)]);
			idNc = rbind(idNc, idJb);
		}
		names(idNc) = c("eleid.1", "eleid.2");
		return(idNc);
	});
	idBB = do.call(rbind, idBB);
}

# Helper:
connectNc = function(x) {
		if(nrow(x) == 0) return(NULL);
		### BackBone:
		id = match(
			c("C1'", "C2'", "C3'", "C4'", "C5'",
			"O5'", "O4'", "O3'",
			"P", "O1P", "O2P", "O3P"), x$elename);
		if(is.na(id[10])) {
			idOP = match(c("OP1", "OP2", "OP3"), x$elename);
			id[10:12] = idOP;
		}
		id = x$eleid[id];
		idJ = id[1]; Jb = c(id[9], id[8]);
		# RNA: O2
		idO2 = match("O2'", x$elename);
		if(! is.na(idO2)) {
			id = c(id, x$eleid[idO2]);
			idO2_1 = 2; idO2_2 = length(id);
		} else {
			idO2_1 = NULL; idO2_2 = NULL;
		}
		# TODO: OP3 on Terminal PO4;
		idB = data.frame(
			E1 = id[c(9, 9, 9, 1,2,3,4,4,7, 5, 3, idO2_1)],
			E2 = id[c(6,10,11, 2,3,4,5,7,1, 6, 8, idO2_2)]);
		### N-Base:
		nc  = x$resname[1];
		nmN = switch(nc,
			"A" = c("N1", "C2", "N3", "C4", "C5", "C6", # Cycle 1
				"N7", "C8", "N9",    "C6", "N6"), # Cycle 2, Substitutions
			"G" = c("N1", "C2", "N3", "C4", "C5", "C6",
				"N7", "C8", "N9",    "C2", "N2",  "C6", "O6"),
			"T" = c("N1", "C2", "N3", "C4", "C5", "C6",
				"C2", "O2",  "C4", "O4",  "C5", "C7"), # Ring-Substitutions
			"U" = c("N1", "C2", "N3", "C4", "C5", "C6",
				"C2", "O2",  "C4", "O4"),
			"C" = c("N1", "C2", "N3", "C4", "C5", "C6",
				"C2", "O2",  "C4", "N4")
		)
		idN = match(nmN, x$elename);
		idN = x$eleid[idN];
		# Cycle: always C6 - N1;
		cycNc = data.frame(E1 = idN[1:6], E2 = idN[c(2:6, 1)]);
		isAG  = nc == "A" || nc == "G";
		if(isAG) {
			id1 = c(5,7,8,9,  10);
			id2 = c(7,8,9,4,  11);
			if(nc == "G") {
				id1 = c(id1, 12); id2 = c(id2, 13);
			}
		} else {
			id1 = c(7, 9);
			id2 = c(8,10);
			if(nc == "T") {
				id1 = c(id1, 11); id2 = c(id2, 12);
			}
		}
		cyc2  = data.frame(E1 = idN[id1], E2 = idN[id2]);
		cycNc = rbind(cycNc, cyc2);
		idJnc = if(isAG) 9 else 1;
		# Nucleotide Skeleton:
		lst = list(BB = idB, N = cycNc,
			Jb = Jb,
			Jn = data.frame(E1 = idJ, E2 = idN[idJnc]));
		return(lst);
}

#' @rdname bond-angle-dihedral
#' @export
measure <- function(...)
  UseMethod("measure")

#' @rdname bond-angle-dihedral
#' @export
measure.default <- function(id = rgl::rgl.ids(), verbose = TRUE, ...){
  cat("Press ESC when you have finish your selections.\n")
  ids <- id[id$type!="text",]$id

  sph.ids <- NULL
  txt.ids <- NULL
  clean <- function(){
    if(!is.null(sph.ids))
      rgl::rgl.pop(id = sph.ids)
    if(!is.null(txt.ids))
      rgl::rgl.pop(id = txt.ids)
  }
  on.exit(clean())
  
  dist <- 0.0015
  sel <- NULL
  while(TRUE){
    f <- rgl::rgl.select3d(button = "right", ...)
    if(is.null(f)) 
      break
    e <- environment(f)
    rgl.info <- lapply(ids,
      function(id){
        verts <- rgl::rgl.attrib(id,"vertices")
        radii <- rgl::rgl.attrib(id,"radii")
        if(nrow(radii)==0)
          radii <- rep(0, nrow(verts))
        radii <- radii + 0.2
        info <- cbind(verts, radii)
        return(info)
      })
    info <- do.call(rbind, rgl.info)
    verts <- info[,1:3]
    radii <- info[,4]
    hits <- f(verts)
    if(!any(hits)){
      wincoords <- rgl::rgl.user2window(verts, projection = e$proj)
      hits <- (0 <= wincoords[,3]) && (wincoords[,3] <= 1)
      if(any(hits)){
        dists <- (wincoords[, 1] - e$llx)^2 +
                 (wincoords[, 2] - e$lly)^2
        hits <- (dists < dist) & (dists == min(dists))
      }
    }
    if(any(hits)){
      verts <- unique(verts[hits, , drop = FALSE])
      radii <- unique(radii[hits,   drop = FALSE])
      sel <- rbind(sel, verts)
      if(nrow(sel) > 4){
        clean()
        sph.ids <- NULL
        txt.ids <- NULL
        sel <- NULL
      }
      else{
        if(verbose)
          print(verts)
        
        sph.ids <- c(sph.ids,
                     rgl::spheres3d(verts, color = "red", alpha = 0.3, radius = radii))
        if(nrow(sel)==2){
          sel.coords <- coords(sel, basis = "xyz")
          B <- bond(sel.coords, 1, 2)
          if(verbose) print(B)
          B <- round(B, digits = 4)
          txt.ids <- rgl::rgl.texts(centres(sel.coords), text = B, col = "red", cex = 1.5, adj = 0)
        }
        if(nrow(sel)==3){
          sel.coords <- coords(sel, basis = "xyz")
          A <- angle(coords(sel.coords, basis = "xyz"), 1, 2, 3)
          if(verbose) print(A)
          A <- round(A, digits = 2)
          rgl::rgl.pop(id = txt.ids)
          txt.ids <- rgl::rgl.texts(centres(sel.coords), text = A, col = "red", cex = 1.5, adj = 0)
        }
        if(nrow(sel)==4){
          sel.coords <- coords(sel, basis = "xyz")
          D <- dihedral(coords(sel.coords, basis = "xyz"), 1, 2, 3, 4)
          if(verbose) print(D)
          D <- round(D, digits = 2)
          rgl::rgl.pop(id = txt.ids)
          txt.ids <- rgl::rgl.texts(centres(sel.coords), text = D, col = "red", cex = 1.5, adj = 0)
        }
      }
    }
    else{
      clean()
      sph.ids <- NULL
      txt.ids <- NULL
      sel <- NULL
    }
  }
}

#' @rdname bond-angle-dihedral
#' @export
measure.coords <- function(x, id = rgl::rgl.ids(), verbose = TRUE, ...){
  cat("Presse ESC when you have finish your selections.\n")
  ids <- id[id$type!="text",]$id
  if(basis(x) == "abc")
    stop("'x' must contain Cartesian coordinates")
  xyz <- coords(x, basis = "xyz")
  
  sph.ids <- NULL
  txt.ids <- NULL
  clean <- function(){
    if(!is.null(sph.ids))
      rgl::rgl.pop(id = sph.ids)
    if(!is.null(txt.ids))
      rgl::rgl.pop(id = txt.ids)
  }
  on.exit(clean())
  
  dist <- 0.0015
  sel <- NULL
  while(TRUE){
    f <- rgl::rgl.select3d(button = "right", ...)
    if(is.null(f)) 
      break
    e <- environment(f)
    rgl.info <- lapply(ids,
                       function(id){
                         verts <- rgl::rgl.attrib(id,"vertices")
                         radii <- rgl::rgl.attrib(id,"radii")
                         if(nrow(radii)==0)
                           radii <- rep(0, nrow(verts))
                         radii <- radii + 0.2
                         info <- cbind(verts, radii)
                         return(info)
                       })
    info <- do.call(rbind, rgl.info)
    verts <- info[,1:3]
    radii <- info[,4]
    hits <- f(verts)
    if(!any(hits)){
      wincoords <- rgl::rgl.user2window(verts, projection = e$proj)
      hits <- (0 <= wincoords[,3]) && (wincoords[,3] <= 1)
      if(any(hits)){
        dists <- (wincoords[, 1] - e$llx)^2 +
          (wincoords[, 2] - e$lly)^2
        hits <- (dists < dist) & (dists == min(dists))
      }
    }
    if(any(hits)){
      verts <- unique(verts[hits, , drop = FALSE])
      radii <- unique(radii[hits,   drop = FALSE])
      sel <- rbind(sel, verts)
      if(nrow(sel) > 4){
        clean()
        sph.ids <- NULL
        txt.ids <- NULL
        sel <- NULL
      }
      else{
        if(verbose){
          M <-     xyz$x1%in%round(verts[,1],digits=3)
          M <- M & xyz$x2%in%round(verts[,2],digits=3)
          M <- M & xyz$x3%in%round(verts[,3],digits=3)
          if(length(which(M))!=0)
            print(subset(x, M))
        }
        
        sph.ids <- c(sph.ids,
                     rgl::spheres3d(verts, color = "red", alpha = 0.3, radius = radii))
        if(nrow(sel)==2){
          sel.coords <- coords(sel, basis = "xyz")
          B <- bond(sel.coords, 1, 2)
          if(verbose) print(B)
          B <- round(B, digits = 4)
          txt.ids <- rgl::rgl.texts(centres(sel.coords), text = B, col = "red", cex = 1.5, adj = 0)
        }
        if(nrow(sel)==3){
          sel.coords <- coords(sel, basis = "xyz")
          A <- angle(coords(sel.coords, basis = "xyz"), 1, 2, 3)
          if(verbose) print(A)
          A <- round(A, digits = 2)
          rgl::rgl.pop(id = txt.ids)
          txt.ids <- rgl::rgl.texts(centres(sel.coords), text = A, col = "red", cex = 1.5, adj = 0)
        }
        if(nrow(sel)==4){
          sel.coords <- coords(sel, basis = "xyz")
          D <- dihedral(coords(sel.coords, basis = "xyz"), 1, 2, 3, 4)
          if(verbose) print(D)
          D <- round(D, digits = 2)
          rgl::rgl.pop(id = txt.ids)
          txt.ids <- rgl::rgl.texts(centres(sel.coords), text = D, col = "red", cex = 1.5, adj = 0)
        }
      }
    }
    else{
      clean()
      sph.ids <- NULL
      txt.ids <- NULL
      sel <- NULL
    }
  }
}

#' @rdname bond-angle-dihedral
#' @export
measure.pdb <- function(x, id = rgl::rgl.ids(), verbose = TRUE, ...)
  measure.coords(x$atoms, id, verbose, ...)

#' Periodic Table of the Elements
#' 
#' This data set gives various information on chemical elements
#' 
#' @format A data frame containing for each chemical element the following information.
#' \describe{
#' \item{\code{num}}{atomic number} 
#' \item{\code{symb}}{elemental symbol}
#' \item{\code{areneg}}{Allred and Rochow electronegativity  (0.0 if unknown)}
#' \item{\code{rcov}}{covalent radii (in Angstrom) (1.6 if unknown)}
#' \item{\code{rbo}}{"bond order" radii} 
#' \item{\code{rvdw}}{van der Waals radii (in Angstrom) (2.0 if unknown)} 
#' \item{\code{maxbnd}}{maximum bond valence (6 if unknown)} 
#' \item{\code{mass}}{IUPAC recommended atomic masses (in amu)} 
#' \item{\code{elneg}}{Pauling electronegativity (0.0 if unknown)} 
#' \item{\code{ionization}}{ionization potential (in eV) (0.0 if unknown)} 
#' \item{\code{elaffinity}}{electron affinity (in eV) (0.0 if unknown)} 
#' \item{\code{red}}{red value for visualization}
#' \item{\code{green}}{green value for visualization}
#' \item{\code{blue}}{blue value for visualization} 
#' \item{\code{name}}{element name} }
#' 
#' @source 
#' Open Babel (2.3.1) file: element.txt
#' \cr\cr
#' Created from the Blue Obelisk Cheminformatics Data Repository\cr
#' Direct Source: http://www.blueobelisk.org/\cr
#' http://www.blueobelisk.org/repos/blueobelisk/elements.xml includes furhter bibliographic citation information\cr
#' \cr
#' - Allred and Rochow Electronegativity from http://www.hull.ac.uk/chemistry/electroneg.php?type=Allred-Rochow\cr
#' - Covalent radii from http://dx.doi.org/10.1039/b801115j\cr
#' - Van der Waals radii from http://dx.doi.org/10.1021/jp8111556\cr
#' 
#' @examples 
#' data(elements)
#' elements
#' 
#' # Get the mass of some elements
#' symb <- c("C","O","H")
#' elements[match(symb, elements[,"symb"]), "mass"]
#' 
#' # Get the van der Waals radii of some elements
#' symb <- c("C","O","H")
#' elements[match(symb, elements[,"symb"]), "rvdw"]
#'
#' @keywords datasets
#' 
#' @name elements
NULL

# TODO: better strategy;
isProtein = function(x) {
	isPr = "CA" %in% x$atoms$elename;
	return(isPr);
}
isProteinA = function(x) {
	isPr = "CA" %in% x$elename;
	return(isPr);
}
# TODO
isNucleicAcid.atoms = function(x) {
	nc = unique(x$resname);
	isNc = any(nc %in% c("A","C","G", "T", "U"));
	return(isNc);
}
isNucleicAcid.character = function(x) {
	nc = unique(x);
	isNc = any(nc %in% c("A","C","G", "T", "U"));
	return(isNc);
}
# Special AA:
isAASpecial = function(x) {
	x %in% c("HZP", "PTR", "TYS");
}

### Backbone:
asBackbone = function(x) {
	# TODO: check continuity
	FUN = function(tmp) {
		id = match(c("N", "CA", "C", "O"), tmp$elename);
		id = tmp$eleid[id];
		id = c(id, NA, id[3]);
	}
	FUN.Nc = backbone.nucleic();
	if(inherits(x, "atoms")) {
		atoms = x;
	} else if(inherits(x, "pdb")) atoms = x$atoms;
	# Process each chain:
	ch = chains(x);
	idBB = lapply(ch, function(ch) {
		# Modified AA: may require better checking if true AA;
		isAA = atoms$Hetero == FALSE;
		isAS = isAASpecial(atoms$resname[atoms$Hetero]);
		isAA[atoms$Hetero] = isAS;
		isCh = atoms$chainid == ch & isAA;
		tmp  = atoms[isCh, c("resid", "eleid", "elename")];
		if(nrow(tmp) == 0) return(NULL);
		#
		if(isNucleicAcid.character(atoms$resname[isCh])) {
			FUN = FUN.Nc;
		}
		idBB = tapply(tmp, tmp$resid, FUN);
		idBB = unlist(idBB);
		idBB = data.frame(idBB[- length(idBB)], idBB[-1]);
		isNA = is.na(idBB[,1]) | is.na(idBB[,2]);
		idBB = idBB[! isNA, , drop = FALSE];
		names(idBB) = c("eleid.1", "eleid.2");
		return(idBB);
	});
	idBB = do.call(rbind, idBB);
	connect.default(idBB);
}

residues = function(x, chain = NULL, hetero = NULL) {
	if(inherits(x, "pdb")) x = x$atoms;
	ch = chains(x);
	if(! is.null(chains)) {
		idCh = match(chain, ch);
		isNA = is.na(idCh);
		if(any(isNA)) {
			warning("The chains: ",
				paste(chain[isNA], collapse = ", "), " do NOT exist!");
			idCh = idCh[! isNA];
		}
		ch = ch[idCh];
		if(length(ch) == 0) return();
	}
	doHetero = ! is.null(hetero);
	tmp = lapply(ch, function(ch) {
		isCh = x$chainid == ch;
		if(doHetero) isCh = isCh & (x$Hetero == hetero);
		tmp = x[isCh, c("resname", "resid"), drop = FALSE];
		tmp = unique(tmp);
		if(nrow(tmp) > 0) tmp$chainid = ch;
		return(tmp);
	})
	tmp = do.call(rbind, tmp);
	return(tmp);
}

# 5' O-CH2-C4-C3-O-PO2-O-> 3'
backbone.nucleic = function(x) {
	FUN = function(x) {
		atom = x$elename;
		atom = sub("\\*", "'", atom);
		id = match(
			c("O5'", "C5'", "C4'", "C3'", "O3'",
			"P", "O1P", "O2P",  "C2'", "C1'", "O4'"), atom);
		id = x$eleid[id];
		id = c(id[6:7], NA, id[6], id[8], NA, # PO4
			id[6], id[-(6:11)], NA,
			id[4], id[9:11], id[3], NA, # Pentose-Ring
			id[5]);
		return(id);
	}
}

#' Helper Functions for Translation of Atomic Coordinates
#' 
#' Translation of atomic coordinates along a specific Cartesian or lattice
#' vector.
#' 
#' These functions are helper functions to perform a translation along a
#' specific Cartesian or lattice vector. All of them call either the \code{Txyz}
#' or \code{Tabc} function.
#' 
#' @return 
#' An object of the same class as \code{x} with translated coordinates.
#' 
#' @param obj an R object containing atomic coordinates.
#' @param x the x-component of the translation vector.
#' @param y the y-component of the translation vector.
#' @param z the z-component of the translation vector.
#' @param a the a-component of the translation vector.
#' @param b the b-component of the translation vector.
#' @param c the c-component of the translation vector.
#' @param mask a logical vector indicating the set of coordinates to which to apply the translation.
#' @param thickness a numeric value indicating the fraction of the thicknees of the selected atom to be added to the translation vector (Usually 0, 0.5 or 1. See details).
#' @param crystal an object of class \sQuote{crystal} used to convert Cartesian into fractional coordinates (or Vice Versa) when need.
#' @param \dots further arguments passed to or from other methods.
#' 
#' @seealso 
#' \code{\link{Txyz}}, \code{\link{Tabc}}\cr
#' Passing from Cartesian to fractional coordinates (or Vis Versa):\cr
#' \code{\link{xyz2abc}}, \code{\link{abc2xyz}}
#' 
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' visualize(x, mode = NULL)
#' visualize(Ty(x, 10), mode = NULL)
#' visualize(Ty(x, 10, mask = x$atoms$resid==1), mode = NULL)
#' visualize(Tb(x, 1 ), mode = NULL)
#' visualize(Tb(x, 1 , mask = x$atoms$resid==1), mode = NULL)
#' 
#' # Lets build a C70/Pentacene dimer with an inter-molecular distance equal to 3.5
#' C70 <- read.pdb(system.file("examples/C70.pdb", package="Rpdb"))
#' Pen <- read.pdb(system.file("examples/Pentacene.pdb", package="Rpdb"))
#' x <- merge(C70, Pen)
#' visualize(x, mode = NULL)
#' viewXY()
#' visualize(Tz(x, z=3.5, mask = x$atoms$resname == "C70", thickness=0.5), mode = NULL)
#' viewXY()
#' 
#' @keywords manip
#' 
#' @name translationHelpers
#' @export
Tx <- function(...)
  UseMethod("Tx")
#' @rdname translationHelpers
#' @export
Tx.coords <- function(obj, x = 0, mask = TRUE, thickness = NULL, crystal = NULL, ...)
  Txyz(obj, x = x, mask = mask, thickness = thickness, crystal = crystal, ...)
#' @rdname translationHelpers
#' @export
Tx.pdb <- function(obj, x = 0, mask = TRUE, thickness = NULL, crystal = obj$crystal, ...)
  Txyz(obj, x = x, mask = mask, thickness = thickness, crystal = crystal, ...)

#' @rdname translationHelpers
#' @export
Ty <- function(...)
  UseMethod("Ty")
#' @rdname translationHelpers
#' @export
Ty.coords <- function(obj, y = 0, mask = TRUE, thickness = NULL, crystal = NULL, ...)
  Txyz(obj, y = y, mask = mask, thickness = thickness, crystal = crystal, ...)
#' @rdname translationHelpers
#' @export
Ty.pdb <- function(obj, y = 0, mask = TRUE, thickness = NULL, crystal = obj$crystal, ...)
  Txyz(obj, y = y, mask = mask, thickness = thickness, crystal = crystal, ...)

#' @rdname translationHelpers
#' @export
Tz <- function(...)
  UseMethod("Tz")
#' @rdname translationHelpers
#' @export
Tz.coords <- function(obj, z = 0, mask = TRUE, thickness = NULL, crystal = NULL, ...)
  Txyz.coords(obj, z = z, mask = mask, thickness = thickness, crystal = crystal, ...)
#' @rdname translationHelpers
#' @export
Tz.pdb <- function(obj, z = 0, mask = TRUE, thickness = NULL, crystal = obj$crystal, ...)
  Txyz.pdb(obj, z = z, mask = mask, thickness = thickness, crystal = crystal, ...)

#' @rdname translationHelpers
#' @export
Ta <- function(...)
  UseMethod("Ta")
#' @rdname translationHelpers
#' @export
Ta.coords <- function(obj, a = 0, mask = TRUE, crystal = NULL, ...)
  Tabc(obj, a = a, mask = mask, crystal = crystal, ...)
#' @rdname translationHelpers
#' @export
Ta.pdb <- function(obj, a = 0, mask = TRUE, crystal = obj$crystal, ...)
  Tabc(obj, a = a, mask = mask, crystal = crystal, ...)

#' @rdname translationHelpers
#' @export
Tb <- function(...)
  UseMethod("Tb")
#' @rdname translationHelpers
#' @export
Tb.coords <- function(obj, b = 0, mask = TRUE, crystal = NULL, ...)
  Tabc(obj, b = b, mask = mask, crystal = crystal, ...)
#' @rdname translationHelpers
#' @export
Tb.pdb <- function(obj, b = 0, mask = TRUE, crystal = obj$crystal, ...)
  Tabc(obj, b = b, mask = mask, crystal = crystal, ...)

#' @rdname translationHelpers
#' @export
Tc <- function(...)
  UseMethod("Tc")
#' @rdname translationHelpers
#' @export
Tc.coords <- function(obj, c = 0, mask = TRUE, crystal = NULL, ...)
  Tabc.coords(obj, c = c, mask = mask, crystal = crystal, ...)
#' @rdname translationHelpers
#' @export
Tc.pdb <- function(obj, c = 0, mask = TRUE, crystal = obj$crystal, ...)
  Tabc.pdb(obj, c = c, mask = mask, crystal = crystal, ...)

#' Add Axes or PBC Box to the \sQuote{rgl} Scene
#' 
#' Add lattice vectors, Cartesian axes or PBC box to the current \sQuote{rgl} scene.
#' 
#' \code{addABC}: Add the lattice vectors a, b and c to the current rgl device.\cr
#' \code{addXYZ}: Add the Cartesian axes x, y and z to the current rgl device.\cr
#' \code{addPBCBox}: Add a box representing the Periodic Boundary Conditions
#'   of a molecular system.\cr
#' \code{addBBox}: Add a bounding box around the PDB molecule
#'   constructed using the lattice vectors.\cr
#' \code{bbox.pdb}: Computes the bounding box around the PDB molecule.\cr
#' 
#' @return Returns invisibly a two-column data.frame containing the IDs 
#'   and type indicators of the objects added to the scene.
#'
#' \code{bbox.pdb}: Returns the coordinates of the bounding box.\cr
#'
#'
#' @param x an object of class \sQuote{crystal} containing the unit cell parameters.
#' @param lwd a numeric value indicating the line width used to draw the axes or
#'   the PBC box.
#' @param labels a logical value indicating whether to draw the labels of the axes.
#' @param scale a scalar value, or a numeric vector of length 2 or 3 or 6, or a 3x3 matrix
#'   used to scale the PBC box or the axes; a length 3 vector is converted to a diagonal matrix,
#'   while vectors of length 2 or 4 or 6 are converted to a diagonal matrix for scaling
#'   and a relative shift;
#' @param cex a numeric value indicating the magnification used to draw the 
#'   labels of the axes.
#' @param col colour used for the PBC box;
#' @param alpha a numeric value specifying the transparency of the lines;
#' @param dx a numeric value specifying the absolute length of the xyz axes-vectors;
#' @param adj.lab numeric value performing relative adjustment of label position;
#'   
#' @seealso \code{\link{visualize}}, \code{\link[rgl]{rgl.open}}, \code{\link[rgl]{par3d}},
#'   \code{\link{addLabels}}
#'   
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' visualize(x, type = "l", xyz = FALSE, abc = FALSE, pbc.box = FALSE, mode = NULL)
#' addXYZ()
#' addABC(x$crystal)
#' addPBCBox(x$crystal)
#' 
#' @keywords dynamic
#'     
#' @name addAxes
#' @export
addABC <- function(x, lwd = 2, scale = NULL, labels = TRUE,
		cex = 2, adj.lab = 1.2) {
  if(missing(x)) stop("Please specify a 'crystal' object")
  if(! is.crystal(x)) stop("'x' must be an object of class 'crystal")
  
	cell = cell.coords(x);
	# Scale Axes:
	mAxes = rbind(
		c(0,0,0), cell[,1],
		c(0,0,0), cell[,2],
		c(0,0,0), cell[,3]);
	if(! is.null(scale)) {
		mAxes = scaleBox(scale, mAxes, cell=cell);
	}
	seg.id = rgl::segments3d(
		mAxes,
		col = c("red","red","green","green","blue","blue"),
		lwd = lwd);
	seg.id = data.frame(id = seg.id, type = "abc.seg");
	#
	an = cell[,1] / sqrt(sum(cell[,1]^2));
	bn = cell[,2] / sqrt(sum(cell[,2]^2));
	cn = cell[,3] / sqrt(sum(cell[,3]^2));
	# Labels:
	lab.id = NULL;
	if(labels) {
		mLab = rbind(
			cell[,1] + adj.lab * an,
			cell[,2] + adj.lab * bn,
			cell[,3] + adj.lab * cn);
		if(! is.null(scale)) {
			mLab = scaleBox(scale, mLab, cell=cell);
		}
		lab.id = rgl::text3d(
			mLab,
			texts = c("a","b","c"),
			col   = c("red","green","blue"),
			cex   = cex);
		lab.id = data.frame(id = lab.id, type = "abc.lab");
	}
	# All:
	ids = rbind(seg.id, lab.id);
	invisible(ids);
}

#' @rdname addAxes
#' @export
addXYZ <- function(lwd = 2, scale = NULL, labels = TRUE, cex = 2, dx = 5) {
	if(length(dx) == 1) dx = c(dx,dx,dx);
	if(length(dx) != 3) stop("Invalid value for dx!");
	dy = dx[2]; dz = dx[3]; dx = dx[1];
	mAxes = rbind(
		c(0,0,0), c( dx,  0,  0),
		c(0,0,0), c(  0, dy,  0),
		c(0,0,0), c(  0,  0, dz),
		c(0,0,0), c(-dx,  0,  0),
		c(0,0,0), c(  0,-dy,  0),
		c(0,0,0), c(  0,  0,-dz)
    );
	# Scale Axes:
	if(! is.null(scale)) {
		# Note: What cell value to use: = 1 or dx?
		mAxes = scaleBox(scale, mAxes, cell = diag(1, 3));
	}
	seg.id = rgl::segments3d(
		mAxes, lwd = lwd, alpha=c(rep(1, 6), rep(0, 6))
	)
	seg.id = data.frame(id = seg.id, type = "xyz.seg");
	# Labels:
	dx = dx + 1.0; dy = dy + 1.0; dz = dz + 1.0;
	lab.id = NULL;
	if(labels){
		mLab = rbind(
			c(0,0,0) + c( dx, 0.0, 0.0),
			c(0,0,0) + c(0.0,  dy, 0.0),
			c(0,0,0) + c(0.0, 0.0,  dz));
		lab.id = rgl::text3d(
			mLab,
			texts = c("x","y","z"),
			cex   = cex
		)
		lab.id = data.frame(id = lab.id, type = "xyz.lab");
	}
	ids = rbind(seg.id, lab.id);
  
  invisible(ids)
}

#' @rdname addAxes
#' @export
addPBCBox = function(x, scale = NULL,
		lwd = 2, col = "black", alpha = 0.25) {
	if(missing(x)) stop("Please specify a 'crystal' object");
	x = get.PDBCrystal(x);
	# Bounding Box:
	cell = cell.coords(x);
	mBox = box.coords(cell);
	# Scale Box:
	if(! is.null(scale)) {
		mBox = scaleBox(scale, mBox, cell=cell);
	}
	# PBC Box:
	seg.id = rgl::segments3d(
		mBox,
		lwd = lwd,
		col = col,
		alpha = alpha
	);
	#
	seg.id = data.frame(id = seg.id, type = "pbc.box");
	invisible(seg.id)
}


#' @rdname addAxes
#' @export
addBBox = function(x, lwd = 2, col = "black", alpha = 0.25) {
	mBox = bbox.pdb(x);
	# BBox:
	seg.id = rgl::segments3d(
		mBox,
		lwd = lwd,
		col = col,
		alpha = alpha
	);
	#
	seg.id = data.frame(id = seg.id, type = "pbc.box");
	invisible(seg.id)
}
#' @rdname addAxes
#' @export
bbox.pdb = function(x) {
	xyzB = range.coords.pdb(x);
	xyzB = t(xyzB);
	dBox = xyzB[,2] - xyzB[,1];
	cell = cell.coords(x$crystal);
	origin = as.vector(cell %*% xyzB[, 1, drop = FALSE]);
	cell = cell %*% diag(dBox);
	bb = box.coords(cell, origin = origin);
	return(bb);
}

range.coords.pdb = function(x, xyz = NULL, ..., na.rm = TRUE) {
	cell = get.PDBCrystal(x);
	xyzV = if(is.null(xyz)) cell.coords(cell) else xyz;
	xyzE = coords(x$atoms);
	tt  = solve(xyzV, t(xyzE));
	tt  = apply(tt, 1, range);
	# xyz = xyzV %*% t(tt);
	return(tt);
}

### Helper:

get.PDBCrystal = function(x) {
	if(is.pdb(x)) {
		x = x$crystal;
		if(is.null(x))
			stop("The PDB molecule does not contain crystal information!");
	} else if(! is.crystal(x)) {
		stop("'x' must be an object of class 'crystal'!");
	}
	return(x);
}

scaleBox = function(scale, mBox, cell) {
	len = length(scale);
	if(len == 1) {
		mBox = scale * mBox;
	} else if(len == 2 || len == 4 || len == 6) {
		# Diag & Relative Shift:
		if(len == 6) {
			m1 = diag(scale[1:3]);
			m2 = scale[4:6];
		} else if(len == 2) {
			m1 = diag(scale[1], 3);
			m2 = rep(scale[2], 3);
		} else {
			m1 = diag(scale[1], 3);
			m2 = scale[2:4];
		}
		mBox  = mBox %*% m1;
		shift = cell %*% m2;
		shift = rep(shift, each = nrow(mBox));
		mBox = mBox + shift;
	} else {
		if(len == 3) {
			scale = diag(scale);
		} else if(len == 9) {
			# OK
		} else stop("Invalid scale argument!");
		mBox = mBox %*% scale;
	}
	return(mBox);
}

box.coords = function(x, origin = c(0,0,0)) {
	cell_12 = x[,1] + x[,2] + origin;
	cell_13 = x[,1] + x[,3] + origin;
	cell_23 = x[,2] + x[,3] + origin;
	cell_Sm = cell_12  + x[,3]; # Opposite Point
	# Shift Cell:
	x = x + rep(origin, 3);
	mBox = rbind(
		origin,   x[,1],
		origin,   x[,2],
		origin,   x[,3],
		cell_12,  x[,1],
		cell_12,  x[,2],
		cell_12,  cell_Sm,
		cell_13,  x[,1],
		cell_13,  cell_Sm,
		cell_13,  x[,3],
		cell_23,  cell_Sm,
		cell_23,  x[,2],
		cell_23,  x[,3]
	);
	rownames(mBox) = NULL;
	return(mBox);
}

#' Extract Chain-Codes from a PDB Molecule

#' Extract the codes of all chains from a PDB object.

#' \code{chains} is a generic function to extract the ids
#' of the chains from a PDB molecule.


#' @param x Object of class \code{pdb} or class \code{atoms};
#' @param sorted logical indicating if the returned chain-codes should be sorted;
#' @param \dots arguments passed to other methods.

#' @return \code{chains} returns the codes of the individual chains in the molecule.


#' @name chains
#' @export
chains = function(x, ...)
	UseMethod("chains")

#' @rdname chains
#' @export
chains.atoms = function(x, sorted = TRUE, ...) {
	ch = unique(x$chainid);
	if(sorted) ch = sort(ch);
	return(ch);
}

#' @rdname chains
#' @export
chains.pdb = function(x, ...) {
	chains(x$atoms);
}

#' Atomic Bond Lengths, Angles and Dihedrals
#' 
#' Compute bond lengths, angles and dihedral from atomic coordinates.
#' 
#' The number of selected atoms with \code{sel1}, \code{sel2}, \code{sel3} and 
#' \code{sel4} must be the same. \code{sel1}, \code{sel2}, \code{sel3} and 
#' \code{sel4} respectively select the first, second, third and fouth atoms 
#' defining bonds, angles or dihedrals.\cr\cr\code{measure} activate an 
#' interactive mode to compute bond lengths, angles and dihedrals by selecting 
#' atoms by \bold{right-clicing} on the current \sQuote{rgl} scene. To escape 
#' the active mode press the ESC key.
#' 
#' @return A numeric vector containing atomic bond lengths (in Angstrom), angles
#' or dihedrals (in degrees)
#' 
#' @param x an R object containing atomic coordinates.
#' @param sel1,sel2,sel3,sel4 an integer or logical vector used to select atoms 
#'   defining bonds, angles or dihedrals. See details.
#' @param id vector of ID numbers of \sQuote{rgl} items, as returned by 
#'   \code{rgl.ids}. The vertexes of these items are used to compute the bond 
#'   lengths, angles or dihedrals.
#' @param verbose a logical value specifying if the information have to be 
#'   printed to the terminal.
#' @param \dots further arguments passed to or from other methods.
#'   
#' @seealso \code{\link{coords}}, \code{\link{pdb}}, \code{\link{info3d}},
#'   \code{\link{visualize}}
#'   
#' @examples
#' Pen <- read.pdb(system.file("examples/Pentacene.pdb", package="Rpdb"))
#' visualize(Pen, mode = NULL)
#' text3d(coords(Pen), texts = Pen$atoms$eleid)
#' bond(Pen, 3:4, 1:2)
#' angle(Pen, 3:4, 1:2, 5:6)
#' dihedral(Pen, 3:4, 1:2, 5:6, 6:5)
#' 
#' @keywords manip dynamic
#'    
#' @name bond-angle-dihedral
#' @export
bond <- function(...)
  UseMethod("bond")

#' @rdname bond-angle-dihedral
#' @export
bond.coords <- function(x, sel1, sel2, ...){
  if(!is.coords(x))
    stop("'x' must be an object of class 'coords'")
  if(basis(x) != "xyz")
    stop("'x' coordinates must be Cartesian coordinates")
  if(missing(sel1) | missing(sel2))
    stop("Please specify 'sel1' and 'sel2'")
  if(length(sel1) != length(sel2))
    stop("'sel1' and 'sel3' must have the same length")
  
  if(is.logical(sel1))
    sel1 <- which(sel1)
  if(is.logical(sel2))
    sel2 <- which(sel2)
  
  ill.bonds <- any(sel1==sel2)
  if(ill.bonds)
    warning("Ill defined bonds")
  
  at1 <- x[sel1,]
  at2 <- x[sel2,]
  if(natom(at1) != natom(at2))
    stop("'sel1' and 'sel2' must select the same number of atoms")
  
  B <- cbind(
    at2$x1 - at1$x1,
    at2$x2 - at1$x2,
    at2$x3 - at1$x3)
  B <- apply(B, 1, vectNorm)
  
  return(B)
}

#' @rdname bond-angle-dihedral
#' @export
bond.pdb <- function(x, sel1, sel2, ...){
  if(!is.pdb(x))
    stop("'x' must be an object of class 'pdb'")
  if(missing(sel1) | missing(sel2))
    stop("Please specify 'sel1' and 'sel2'")
  if(length(sel1) != length(sel2))
    stop("'sel1' and 'sel2' must have the same length")
  if(basis(x) == "abc") {
    # TODO: function checkHasCrystal;
    if(is.null(x$crystal))
      stop("'x' contains fractional coordinates but no 'crystal' component for conversion to Cartesian coordinates")
    x <- abc2xyz(x)    
  }
  A <- bond.coords(x$atoms, sel1, sel2, ...)
  return(A)
}

#' @rdname bond-angle-dihedral
#' @export
angle <- function(...)
  UseMethod("angle")

#' @rdname bond-angle-dihedral
#' @export
angle.coords <- function(x, sel1, sel2, sel3, ...){
  if(!is.coords(x))
    stop("'x' must be an object of class 'coords'")
  if(basis(x) != "xyz")
    stop("'x' coordinates must be Cartesian coordinates")
  if(missing(sel1) | missing(sel2) | missing(sel3))
    stop("Please specify 'sel1','sel2' and 'sel3'")
  if(length(sel1) != length(sel2) | length(sel1) != length(sel3))
    stop("'sel1', 'sel2' and 'sel3' must have the same length")
  
  if(is.logical(sel1))
    sel1 <- which(sel1)
  if(is.logical(sel2))
    sel2 <- which(sel2)
  if(is.logical(sel3))
    sel3 <- which(sel3)
  
  ill.angles <- any(sel1==sel2) | any(sel2==sel3)
  if(ill.angles)
    warning("Ill defined angles")
  
  at1 <- x[sel1,]
  at2 <- x[sel2,]
  at3 <- x[sel3,]
  if(natom(at1) != natom(at2) | natom(at1) != natom(at3))
    stop("'sel1', 'sel2' and 'sel3' must select the same number of atoms")
  
  U <- cbind(
    at1$x1 - at2$x1,
    at1$x2 - at2$x2,
    at1$x3 - at2$x3)
  Un <- apply(U, 1, vectNorm)
  U <- U/Un
  
  V <- cbind(
    at3$x1 - at2$x1,
    at3$x2 - at2$x2,
    at3$x3 - at2$x3)
  Vn <- apply(V, 1, vectNorm)
  V <- V/Vn
  
  A <- (180/pi)*acos(rowSums(U*V))
  return(A)
}

#' @rdname bond-angle-dihedral
#' @export
angle.pdb <- function(x, sel1, sel2, sel3, ...){
  if(!is.pdb(x))
    stop("'x' must be an object of class 'pdb'")
  if(missing(sel1) | missing(sel2) | missing(sel3))
    stop("Please specify 'sel1','sel2' and 'sel3'")
  if(length(sel1) != length(sel2) | length(sel1) != length(sel3))
    stop("'sel1', 'sel2' and 'sel3' must have the same length")
  if(basis(x)=="abc"){
    if(is.null(x$crystal))
      stop("'x' contains fractional coordinates but no 'crystal' component for conversion to Cartesian coordinates");
    x <- abc2xyz(x)    
  }
  A <- angle.coords(x$atoms, sel1, sel2, sel3, ...)
  return(A)
}

#' @rdname bond-angle-dihedral
#' @export
dihedral <- function(...)
  UseMethod("dihedral")

#' @rdname bond-angle-dihedral
#' @export
dihedral.coords <- function(x, sel1, sel2, sel3, sel4, ...){
  if(!is.coords(x))
    stop("'x' must be an object of class 'coords'")
  if(basis(x) != "xyz")
    stop("'x' coordinates must be Cartesian coordinates")
  if(missing(sel1) | missing(sel2) | missing(sel3) | missing(sel4))
    stop("Please specify 'sel1','sel2', 'sel3' and 'sel4'")
  if(length(sel1) != length(sel2) |
       length(sel1) != length(sel3) |
       length(sel1) != length(sel4))
    stop("'sel1', 'sel2', 'sel3' and 'sel4' must have the same length")
  
  if(is.logical(sel1))
    sel1 <- which(sel1)
  if(is.logical(sel2))
    sel2 <- which(sel2)
  if(is.logical(sel3))
    sel3 <- which(sel3)
  if(is.logical(sel4))
    sel4 <- which(sel4)
  
  at1 <- x[sel1,]
  at2 <- x[sel2,]
  at3 <- x[sel3,]
  at4 <- x[sel4,]
  if(natom(at1) != natom(at2) |
       natom(at1) != natom(at3) |
       natom(at1) != natom(at4))
    stop("'sel1', 'sel2', 'sel3' and 'sel4' must select the same number of atoms")
  
  ill.dihedrals <- any(sel1==sel2) | any(sel2==sel3) |
    any(sel2==sel3) | any(sel3==sel4)
  if(ill.dihedrals)
    warning("Ill defined dihedrals")
  
  U <- cbind(
    at2$x1 - at1$x1,
    at2$x2 - at1$x2,
    at2$x3 - at1$x3)
  U <- U/apply(U, 1, vectNorm)
  
  V <- cbind(
    at3$x1 - at2$x1,
    at3$x2 - at2$x2,
    at3$x3 - at2$x3)
  V <- V/apply(V, 1, vectNorm)
  
  W <- cbind(
    at4$x1 - at3$x1,
    at4$x2 - at3$x2,
    at4$x3 - at3$x3)
  W <- W/apply(W, 1, vectNorm)
  
  N1 <- apply(cbind(U,V), 1,
              function(x){
                x <- vectProd(x[1:3], x[4:6])
                x <- x/vectNorm(x)
              })
  N2 <- apply(cbind(V,W), 1,
              function(x){
                x <- vectProd(x[1:3], x[4:6])
                x <- x/vectNorm(x)
              })
  
  D <- (180/pi)*acos(colSums(N1*N2))
  
  return(D)
}

#' @rdname bond-angle-dihedral
#' @export
dihedral.pdb <- function(x, sel1, sel2, sel3, sel4, ...){
  if(!is.pdb(x))
    stop("'x' must be an object of class 'pdb'")
  if(missing(sel1) | missing(sel2) | missing(sel3) | missing(sel4))
    stop("Please specify 'sel1', 'sel2', 'sel3' and 'sel4'")
  if(length(sel1) != length(sel2) |
       length(sel1) != length(sel3) |
       length(sel1) != length(sel4))
    stop("'sel1', 'sel2', 'sel3' and 'sel4' must have the same length")
  if(basis(x)=="abc") {
    if(is.null(x$crystal))
      stop("'x' contains fractional coordinates but no 'crystal' component for conversion to Cartesian coordinates")
    x <- abc2xyz(x)    
  }
  A <- dihedral.coords(x$atoms, sel1, sel2, sel3, sel4, ...)
  return(A)
}

#' Number of Atoms in an Object Containing Atomic Coordinates
#' 
#' Evaluates the number of atoms in an object containing atomic coordinates.
#' 
#' \code{natom} is a generic function to evalute the number of atom in an object
#' containing atomic coordinates. The atomic coordinates of the object are first
#' filtered to keep ATOM and/or HETATM records as indicated by the 'ATOM' and 
#' 'HETATM' arguments. Then, if \code{factor} is specify, the object is splitted
#' to evalute the number of atoms in each group defined by \code{factor}. If 
#' \code{factor} is not specify then the total number of atoms in the object is 
#' return.
#' 
#' @return Return an integer or a vector of integer of lenght equal to 
#'   \code{nlevels(factor)} (if \code{factor} is specify) indication the number
#'   of atoms in the object or in the groups defined by \code{factor}.
#'   
#' @param x an R object containing atomic coordinates.
#' @param factor a factor used to split the object and evaluate the number of atoms
#'   in each group.
#' @param ATOM a single element logical vector indicating if ATOM records have
#'   to be considered or not.
#' @param HETATM a single element logical vector indicating if HETATM records
#'   have to be considered or not.
#' @param \dots further arguments passed to or from other methods.
#'   
#' @seealso \code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}},
#'   \code{\link[base]{factor}}, \code{\link[base]{split}}
#'   
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#'
#' natom(x)
#' natom(x, x$atoms$resid)
#' natom(x, x$atoms$resname)
#' natom(x, HETATM = FALSE)
#'   
#' @keywords manip
#' 
#' @name natom
#' @export
natom <- function(x, ...)
  UseMethod("natom")

#' @rdname natom
#' @export
natom.coords <- function(x, factor = NULL, ...)
{
  if(length(factor) == 0)
    nrow(x)
  else
    unlist(lapply(split(x, factor), nrow))
}

#' @rdname natom
#' @export
natom.atoms <- function(x, factor = NULL, ATOM = TRUE, HETATM = TRUE, ...)
{
  M <- ATOM & x$recname == "ATOM" | HETATM & x$recname == "HETATM"
  x <- x[M,]
  if(length(factor) != 0)
    factor <- factor[M]
  natom.coords(x, factor)
}

#' @rdname natom
#' @export
natom.pdb <- function(x, factor = NULL, ATOM = TRUE, HETATM = TRUE, ...)
  natom.atoms(x$atoms, factor, ATOM, HETATM, ...)

#' Moment of Inertia of a Molecular System
#' 
#' Computes the inertia tensor of a molecular system from atomic coordinates and
#' masses.
#' 
#' \code{inertia} is a generic function to compute the inertia tensor of a
#' molecular system. For object of class \sQuote{coords} both atomic coordinates
#' and masses have to be speifyed. For object of class \sQuote{atoms} the masses
#' are determined from the \code{elename} component of the object (see
#' \code{\link{toSymbols}} and \code{\link{masses}}). For object of class
#' \sQuote{pdb} the \code{atoms} component is used.
#' 
#' @return Return the inertia tensor in a 3x3 matrix.
#' 
#' @param x an R object containing atomic coordinates.
#' @param m a numeric vector containing atomic masses.
#' @param \dots further arguments passed to or from other methods.
#'
#' @seealso 
#' \code{\link{toSymbols}}, \code{\link{masses}}, \code{\link{viewInertia}}
#'
#' @examples 
#' C70 <- read.pdb(system.file("examples/C70.pdb", package="Rpdb"))
#' inertia(C70)
#' visualize(C70, mode = NULL)
#' viewXY()
#' viewInertia(C70)
#' 
#' @keywords manip
#' 
#' @name inertia
#' @export
inertia <- function(...)
  UseMethod("inertia")

#' @rdname inertia
#' @export
inertia.coords <- function(x, m = NULL, ...){
  if(!is.coords(x))
    stop("'x' must be an object of class coords")
  if(is.null(m))
    stop("Please specify the masses")
  if(any(is.na(m))|any(is.na(x)))
    stop("NA values not permetted")
  Ixx<-sum(m*(x$x2^2+x$x3^2))
  Iyy<-sum(m*(x$x1^2+x$x3^2))
  Izz<-sum(m*(x$x1^2+x$x2^2))
  Ixy<-sum(m*(x$x1*x$x2))
  Ixz<-sum(m*(x$x1*x$x3))
  Iyz<-sum(m*(x$x2*x$x3))
  I<-matrix(c(Ixx,-Ixy,-Ixz,-Ixy,Iyy,-Iyz,-Ixz,-Iyz,Izz),ncol=3)
  return(I)
}

#' @rdname inertia
#' @export
inertia.atoms <- function(x, m = NULL, ...){
  if(is.null(m))
    m <- masses(toSymbols(x$elename))
  inertia(coords(x), m)
}

#' @rdname inertia
#' @export
inertia.pdb <- function(x, m = NULL, ...)
  inertia(x$atoms, m)

#' Properties of a Unit Cell
#' 
#' Compute the Cartesian coordinates of lattice vectors, the volume or the density of a unit cell.
#'
#' \code{cell.coords} is a generic function which computes a 3x3 matrix whose columns contrain the Cartesian coordinates of lattice vectors.
#' The 'a' and 'b' vectors are assumed to be respectively along the x-axis and in the xy-plane.
#' The default method takes directly the lattice parameters as arguments.
#' For objects of class \code{\link{crystal}} the lattice parameters are first extracted from the object and then the default method is called.
#' For objects of class \code{\link{pdb}} the lattice parameters are extracted from their \code{crystal} component and the default method is called.
#' \cr
#' \cr
#' \code{cell.volume} is a generic function to compute the volume of a unit cell.
#' For objects of class \sQuote{crystal}, the unit cell parameters are directly used to compute the volume.
#' For objects of class \sQuote{pdb}, their \code{crystal} component is used.
#' \cr
#' \cr
#' \code{cell.density} is a generic function to compute the density of a unit cell.
#' For objects of class \sQuote{pdb}:
#' First the volume of the unit cell is calculated by calling the \code{cell.volume} function on the \code{crystal} component of the \sQuote{pdb} object.
#' Then the element names are converted into element symbols using the \code{toSymbols} function and their masses are taken from the \code{elements} data set.
#' Finally the density is calculated using the sum of the atomic masses and the volume of the unit cell.
#'
#' @return 
#' \code{cell.coords} returns a 3x3 matrix containing the Cartesian coordinates of lattice vectors arranged by columns.\cr
#' \code{cell.volume} returns a single element numeric vector containing the volume of the unit cell in Angstrom cube.\cr
#' \code{cell.density} returns a single element numeric vector containing the density of the unit cell in g.cm-3.
#'
#' @param abc a length 3 numeric vector containing the length of the a, b and c lattice vectors.
#' @param abg a length 3 numeric vector containing the angles (degrees) between the a, b and c lattice vectors (alpha, beta, gamma).
#' @param digits an integer used to round the lattice vectors coordinates.
#' @param x an R object containing lattice parameters.
#' @param masses a numeric vector containing atomic masses.
#' @param volume a single element numeric vector containing the volume of the unit cell in Angstrom cube.
#' @param \dots further arguments passed to or from other methods.
#' 
#' @seealso \code{\link{crystal}}, \code{\link{pdb}}, \code{\link{xyz2abc}}
#' 
#' @examples 
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#' cell.volume(x)
#' cell.density(x)
#' cell.coords(x)
#'
#' @keywords manip
#'
#' @name cellProperties
#' @export
cell.coords <- function(...)
  UseMethod("cell.coords")

#' @rdname cellProperties
#' @export
cell.coords.default <- function(abc, abg = c(90,90,90), digits = 3, ...)
{
  if(missing(abc)) stop("Please provide at list a 'abc' vector containing the length of the lattice vectors")
  if(length(abc) != 3) stop("'abc' must be a vector of length 3")
  if(length(abg) != 3) stop("'abg' must be a vector of length 3")
  
  abg <- abg*pi/180
  
  M <- matrix(ncol=3,nrow=3)
  M[ ,1] <- c(abc[1],0,0)
  M[ ,2] <- c(abc[2]*cos(abg[3]),abc[2]*sin(abg[3]),0)
  M[1,3] <-   abc[3]*(cos(abg[2]))
  M[2,3] <-   abc[3]*(cos(abg[1])-cos(abg[2])*cos(abg[3]))/sin(abg[3])
  M[3,3] <-   abc[3]*sqrt(1+2*cos(abg[1])*cos(abg[2])*cos(abg[3])-(cos(abg[1]))^2-(cos(abg[2]))^2-(cos(abg[3]))^2)/sin(abg[3])
  M <- round(M, digits=3)
  dimnames(M) <- list(c("x","y","z"), c("a","b","c"))
  
  return(M)
}

#' @rdname cellProperties
#' @export
cell.coords.crystal <- function(x, digits = 3, ...)
{
  if(! is.crystal(x)) stop("'x' must be an object of class 'crystal'")
  
  M <- cell.coords.default(x$abc, x$abg, digits)
  return(M)
}

#' @rdname cellProperties
#' @export
cell.coords.pdb <- function(x, digits = 3, ...)
{
	if(! is.pdb(x)) stop("'x' must be an object of class 'atoms'");
	if(is.null(x$crystal)) stop("'x' must contained a 'crystal' object!");
	
	M = cell.coords.crystal(x$crystal, digits = 3);
	return(M);
}

#' @rdname cellProperties
#' @export
cell.volume <- function(...)
  UseMethod("cell.volume")

#' @rdname cellProperties
#' @export
cell.volume.crystal <- function(x, ...)
{
  if(! is.crystal(x)) stop("'x' must be an object of class 'crystal'")
  
  alpha = x$abg*pi/180;
  V = prod(x$abc) * sqrt(1 - sum(cos(alpha)^2) + 2*prod(cos(alpha)));
  #   attr(V, "unit") <- "AngtromCube"
  return(V)
}

#' @rdname cellProperties
#' @export
cell.volume.pdb <- function(x, ...)
{
	if(! is.pdb(x)) stop("'x' must be an object of class 'atoms'");
	if(is.null(x$crystal)) stop("'x' must contain a 'crystal' object");
	
	V = cell.volume.crystal(x$crystal);
	return(V);
}

#' @rdname cellProperties
#' @export
cell.density <- function(...)
  UseMethod("cell.density")

#' @rdname cellProperties
#' @export
cell.density.default <- function(masses, volume, ...) {
  Na <- Rpdb::universalConstants["Na","Value"]
  d <- sum(masses)/(volume*1E-24*Na)
  #   attr(d, "unit") <- "g.cm-3"
  return(d)
}

#' @rdname cellProperties
#' @export
cell.density.pdb <- function(x, ...) {
  M <- masses(x)
  V <- cell.volume(x)
  d <- cell.density(M, V)
  return(d)
}

#' Divide and Reassemble \sQuote{pdb} Objects
#' 
#' \code{split} divides a \sQuote{pdb} object by groups of atoms defined by 
#' \code{f}. \code{unsplit} reverses the effect of \code{split}.
#' 
#' \code{split} produce a list of \sQuote{pdb} objects with the same 
#' \code{crystal}, \code{title} and \code{remark} components as \code{x}. Only 
#' its \code{atoms} component is split while its \code{connect} component is 
#' cleaned to keep only the meaningful connectivity for each \sQuote{pdb} object
#' of the list returned by the function. \code{unlist} produce a \sQuote{pdb} 
#' object with the same \code{crystal}, \code{title} and \code{remark} components
#' as the first element of \code{value}. The \code{atoms} and \code{connect} 
#' components of all the elements of \code{value} are combined by row.
#' 
#' @return The value returned from \code{split} is a list of \sQuote{pdb}
#' objects containing the data for the groups of atoms. The components of the
#' list are named by the levels of \code{f} (after converting to a factor, or if
#' already a factor and \code{drop=TRUE}, dropping unused levels). \cr 
#' \code{unsplit} returns a \sQuote{pdb} object for which \code{split(x, f)}
#' equals \code{value}.
#' 
#' @param x an object of class \sQuote{pdb} to be divided into groups.
#' @param f a \sQuote{factor} in the sense that
#'   \code{\link{as.factor}(f)} defines the grouping, or a list of such factors
#'   in which case their interaction is used for the grouping.
#' @param drop logical indicating if levels that do not occur should be
#'   dropped (if \code{f} is a \code{factor} or a list).
#' @param value a list of 'pdb' objects compatible with a splitting of
#'   \code{x}. Recycling applies if the lengths do not match.
#' @param \dots further potential arguments passed to methods.
#' 
#' @seealso \code{\link[base]{split}}, \code{\link{unsplit}}, \code{\link{pdb}}
#' 
#' @examples 
#' \donttest{
#' ### Split a pdb file by residue IDs and write them into separate files
#' x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
#'
#' file.names <- paste0(x$atoms$resname, "_", x$atoms$resid, ".pdb")
#' file.names <- unique(file.names)
#' pdb.resid  <- split(x, x$atoms$resid)
#' length(pdb.resid)
#' useless <- mapply(write.pdb, pdb.resid, file.names)
#'
#' # Cleanup
#' unlink(file.names)
#' }
#' 
#' @keywords category
#' 
#' @name split.pdb
#' @export
split.pdb <- function(x, f, drop = FALSE, ...)
{
  if(!is.pdb(x)) stop("'x' must be an object of class 'pdb'")
  
  atoms <- split(x$atoms, f, drop)

  to.return <- lapply(atoms, pdb, x$crystal, x$connect, x$remark, x$title, x$Resolution);
  to.return <- lapply(to.return, function(x) {
		r =     x$connect$eleid.1 %in% x$atoms$eleid;
		r = r & x$connect$eleid.2 %in% x$atoms$eleid;
		if(any(r)) {
			x$connect = x$connect[r,];
		} else x["connect"] = list(NULL);
		return(x)
		}
  )
  
  return(to.return)
}

#' @rdname split.pdb
#' @export
unsplit.pdb <- function(value, f, drop = FALSE, ...)
{
	if(!all(unlist(lapply(value, is.pdb))))
		stop("'value' must be a list containing only 'pdb' objects");
  
	title   = value[[1]]$title;
	remark  = value[[1]]$remark;
	crystal = value[[1]]$crystal;
	resolution = value[[1]]$resolution;
  
  atoms <- lapply(value, function(x) return(x$atoms))
  atoms <- unsplit(atoms, f, drop)
	
	connect = lapply(value, function(x) return(x$connect));
	connect = do.call(rbind, connect);
	lenConn = nrow(connect);
	if(lenConn > 0) rownames(connect) = 1:lenConn;
	
	to.return <- pdb(atoms, crystal, connect, remark, title,
		resolution = resolution);
  
  return(to.return)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize.R
\name{visualize}
\alias{visualize}
\alias{windowRect0}
\alias{visualize.coords}
\alias{visualize.data.frame}
\alias{visualize.matrix}
\alias{visualize.atoms}
\alias{visualize.pdb}
\alias{visualize.character}
\title{Visualize a Molecular Structure}
\usage{
windowRect0()

visualize(...)

\method{visualize}{coords}(
  x,
  elename = NULL,
  crystal = NULL,
  connect = NULL,
  mode = NULL,
  type = "l",
  xyz = NULL,
  abc = NULL,
  pbc.box = NULL,
  lwd = 2,
  lwd.xyz = lwd,
  lwd.abc = lwd,
  lwd.pbc.box = lwd,
  cex.xyz = 2,
  cex.abc = 2,
  col = NULL,
  bg = "#FAFAD2",
  radii = "rvdw",
  scale.atoms = 1,
  add = FALSE,
  windowRect = c(0, 0, 800, 600),
  FOV = 0,
  userMatrix = diag(4),
  scale.box = 1,
  scale.axis = scale.box,
  alpha.box = 0.25,
  ...
)

\method{visualize}{data.frame}(
  x,
  elename = NULL,
  crystal = NULL,
  connect = NULL,
  mode = NULL,
  type = "l",
  xyz = NULL,
  abc = NULL,
  pbc.box = NULL,
  lwd = 2,
  lwd.xyz = lwd,
  lwd.abc = lwd,
  lwd.pbc.box = lwd,
  cex.xyz = 2,
  cex.abc = 2,
  col = NULL,
  bg = "#FAFAD2",
  radii = "rvdw",
  add = FALSE,
  windowRect = c(0, 0, 800, 600),
  FOV = 0,
  userMatrix = diag(4),
  ...
)

\method{visualize}{matrix}(
  x,
  elename = NULL,
  crystal = NULL,
  connect = NULL,
  mode = NULL,
  type = "l",
  xyz = NULL,
  abc = NULL,
  pbc.box = NULL,
  lwd = 2,
  lwd.xyz = lwd,
  lwd.abc = lwd,
  lwd.pbc.box = lwd,
  cex.xyz = 2,
  cex.abc = 2,
  col = NULL,
  bg = "#FAFAD2",
  radii = "rvdw",
  add = FALSE,
  windowRect = c(0, 0, 800, 600),
  FOV = 0,
  userMatrix = diag(4),
  ...
)

\method{visualize}{atoms}(
  x,
  crystal = NULL,
  connect = NULL,
  mode = NULL,
  type = "l",
  xyz = NULL,
  abc = NULL,
  pbc.box = NULL,
  lwd = 2,
  lwd.xyz = lwd,
  lwd.abc = lwd,
  lwd.pbc.box = lwd,
  cex.xyz = 2,
  cex.abc = 2,
  col = NULL,
  bg = "#FAFAD2",
  radii = "rvdw",
  scale.atoms = 1,
  add = FALSE,
  windowRect = NULL,
  FOV = 0,
  userMatrix = diag(4),
  alpha.box = 0.25,
  ...
)

\method{visualize}{pdb}(
  x,
  mode = NULL,
  type = "l",
  xyz = NULL,
  abc = NULL,
  pbc.box = NULL,
  lwd = 2,
  lwd.xyz = lwd,
  lwd.abc = lwd,
  lwd.pbc.box = lwd,
  cex.xyz = 2,
  cex.abc = 2,
  col = NULL,
  bg = "#FAFAD2",
  radii = "rvdw",
  scale.atoms = 1,
  add = FALSE,
  windowRect = NULL,
  FOV = 0,
  userMatrix = diag(4),
  ...
)

\method{visualize}{character}(
  x,
  mode = NULL,
  type = "l",
  xyz = NULL,
  abc = NULL,
  pbc.box = NULL,
  lwd = 2,
  lwd.xyz = lwd,
  lwd.abc = lwd,
  lwd.pbc.box = lwd,
  cex.xyz = 2,
  cex.abc = 2,
  col = NULL,
  bg = "#FAFAD2",
  radii = "rvdw",
  add = FALSE,
  windowRect = NULL,
  FOV = 0,
  userMatrix = diag(4),
  ...
)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{x}{an object or the name of a PDB file containing the molecular structure to visualize.}

\item{elename}{a character vector containing the atomic names used to chose atom colors and radii.}

\item{crystal}{an object of class \sQuote{crystal}. See \code{\link{crystal}}}

\item{connect}{an object of class \sQuote{connect}. See \code{\link{connect}}}

\item{mode}{a single element character vector indicating the visualization mode (See details).}

\item{type}{a character string indicating the visualization style (See details).}

\item{xyz}{a logical value indicating whether the x, y and z axes have to be added to the scene. See details.}

\item{abc}{a logical value indicating whether the a, b and c axes have to be added to the scene. See details.}

\item{pbc.box}{a logical value indicating whether the pbc box has to be added to the scene. See details.}

\item{lwd}{a numeric value indication the line width used to plot the axes, the pbc box and atomic bonds when \code{type = "l"} (see details).}

\item{lwd.xyz}{a numeric value indicating the line width used to plot the x, y and z axes.}

\item{lwd.abc}{a numeric value indicating the line width used to plot the a, b and c axes.}

\item{lwd.pbc.box}{a numeric value indicating the line width used to plot the pbc box.}

\item{cex.xyz}{a numeric value indicating the magnification used to plot the labels of the x, y and z axes.}

\item{cex.abc}{a numeric value indicating the magnification used to plot the labels of the a, b and c axes.}

\item{col}{a vector indicating the colors to use to plot each atom.}

\item{bg}{the color of the background}

\item{radii}{either a character string indicating the type of radii or a numeric vector specifying the radii of each atom to use to plot atoms as spheres (see details).}

\item{scale.atoms}{scalar value by which to scale the radii.}

\item{add}{a logical value indicating whether the plot has be to added to a existing scene (see \code{rgl.cur} and \code{open3d}).}

\item{windowRect}{a vector of four integers indicating the left, top, right and bottom of the displayed window in pixels (see \code{par3d}).}

\item{FOV}{the field of view. This controls the degree of parallax in the perspective view (see par3d).}

\item{userMatrix}{a 4 by 4 matrix describing user actions to display the scene (see \code{par3d}).}

\item{scale.box}{scalar value or numeric vector used to scale the PBC box.}

\item{scale.axis}{scalar value or numeric vector used to scale the axis-vectors.}

\item{alpha.box}{a logical value specifying the transparency of the box-lines;}
}
\value{
Return (using invisible) a two-column data.frame containing the IDs
and type indicators of the objects added to the scene.
}
\description{
Use the rgl library to visualize in 3D a molecular structure.
}
\details{
Three different visualization styles are allowed.
\itemize{
  \item   When \code{type="p"}: Points are drawn at each atomic positions (very light visualization mode).
  \item   When \code{type="l"}: Lines are drawn between bonded atoms. The connectivity of the system has to be specifyed.
  \item   When \code{type="s"}: Spheres are drawn at each atomic positions (heavy visualization mode).
  
  The radii of the spheres are given by \code{radii}.
  \itemize{
    \item   When \code{radii="rcov"}: Covalent radii, taken from the \code{elements} data set, are used.
    \item   When \code{radii="rvdw"}: Van der Waals radii, taken from the \code{elements} data set, are used.
    \item   When \code{radii} is a numeric vector: The numeric values are used to assign to each atom a radius. If \code{length(radii) != natom(pdb)} \code{radii} is recycled.
  }
}
When \code{xyz}, \code{abc} or \code{pbc.box} are \code{NULL}, the axis or pbc box are are added depending if a \sQuote{crystal} object can be found.\cr
Two different interactive visualization modes are avalable:
  \itemize{
    \item When \code{mode="measure"}: bond lengths, angles and dihedrals can be measured by \bold{right-clicing} on the atoms.
    \item When \code{mode="info"}: atomic labels can be added to the scene by \bold{right-clicing} on the atoms. The labels are as follow: "ResidResname:EleidElename"
  }
When \code{mode=NULL} the interactive mode is disabled. To escape the interactive mode press the ESC key.
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
visualize(x, type = "l", mode = NULL)
visualize(x, type = "s", radii = "rcov", mode = NULL)
visualize(x, type = "s", radii = "rvdw", mode = NULL)
visualize(x, type = "p", mode = NULL)
visualize(subset(x, resid != 1), type = "l", mode = NULL)
visualize(subset(x, resid == 1), type = "s", add = TRUE, mode = NULL)

### Protein BackBone + Surface
# x = some protein (1 chain);
# visualize(x, type="l", lwd=4)
# visualize(x, type="s", alpha = 0.05, add = TRUE)
}
\seealso{
\code{\link{addXYZ}}, \code{\link{addABC}}, \code{\link{addPBCBox}}, \code{par3d}, \code{select3d}, \code{measure}, \code{info3d}
}
\keyword{dynamic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distances.R
\name{distances}
\alias{distances}
\alias{dist.point}
\alias{distances.default}
\alias{distances.coords}
\alias{distances.atoms}
\alias{distances.connect}
\alias{distances.pdb}
\alias{dist.point.default}
\alias{is.distances}
\alias{norm}
\alias{norm.distances}
\alias{norm.connect}
\title{Inter-Atomic Distances}
\usage{
distances(...)

dist.point(...)

\method{distances}{default}(
  dx1 = numeric(0),
  dx2 = numeric(0),
  dx3 = numeric(0),
  basis = "xyz",
  ...
)

\method{distances}{coords}(x, sel1, sel2, ...)

\method{distances}{atoms}(x, sel1, sel2, ...)

\method{distances}{connect}(x, data, ...)

\method{distances}{pdb}(x, sel1, sel2, ...)

\method{dist.point}{default}(data, x, y = NULL, z = NULL, subset = NULL, ...)

is.distances(x)

norm(...)

\method{norm}{distances}(x, type = "xyz", ...)

\method{norm}{connect}(x, data, type = "xyz", ...)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{dx1, dx2, dx3}{numeric arrays containing the first, second and third components of the distance vectors.}

\item{basis}{a single element character vector indicating the type of basis vector used to express the coordinates.}

\item{x, y, z}{an R object containing atomic coordinates, or a a connect object (for the corresponding method);}

\item{sel1, sel2}{integer or logical vectors defining two atomic selections between which the distance vectors are computed.}

\item{data}{an object of type pdb, atoms or coords, containing atomic coordinates.}

\item{subset}{enables sub-setting the coords object;}

\item{type}{a single element character vector indicating how to project the distances vectors before computing the norms. See details.}
}
\value{
The \code{distances} and \code{dist.point} functions return an object
  of class \sQuote{distances} containing inter-atomic distance vectors.
The \code{norm} function return an array, with the same dimensions as
  the \code{dx1}, \code{dx2}, \code{dx3} components of the \sQuote{distances} object
  for which the norms have been computed, containing the norm of the distance vectors. 
\cr\cr
\code{is.distances} returns TRUE if x is an object of class \sQuote{distances}
and FALSE otherwise.
}
\description{
Computes inter-atomic distance vectors.
}
\details{
The purpose of the \sQuote{distances} class is to store the inter-atomic 
distance vectors and facilitate their manipulation when passing from the 
Cartesian to fractional references and vice versa.\cr
The default method of the \code{distances} function is actually a builder
  allowing to create a \sQuote{distances} object from its different components, i.e.:
  \code{dx1}, \code{dx2}, \code{dx3}, and \code{basis}.
  All the arguments have to be specified except 'basis' which
  by default is set to "xyz" (Cartesian reference).

For objects of class \sQuote{coords}, \sQuote{atoms}, \sQuote{pdb},
two sets of atomic coordinates, defined by \code{sel1} and \code{sel2},
are extracted and inter-atomic distance vectors are computed 
between these two sets.
For objects of class \sQuote{connect}, distances are computed for the atoms
  specified by the \sQuote{connect} object.

The method for the \sQuote{dist.point} function computes the inter-atomic distances
between the atoms and a specified point.

The method of the \code{norm} function for objects of class \sQuote{distances}
  computes the norm of the distances vectors.
\code{type} specify how to project the distance vectors before 
  computing the norms. By default no projection is perform.
  The three dx, dy, and dz components of the distance vectors
  are used to compute the norm. 
\code{type} can take the following values:
   \itemize{
   \item   x: The distance vectors are projected over x.
   \item   y: The distance vectors are projected over y.
   \item   z: The distance vectors are projected over z.
   \item  xy: The distance vectors are projected in the xy-plan.
   \item  yz: The distance vectors are projected in the yz-plan.
   \item  zx: The distance vectors are projected in the zx-plan.
   \item xyz: The distance vectors are not projected 
     (The three components of the distance vectors are used to compute the norm).}
\code{is.distances} tests if x is an object of class \sQuote{distances};
the test is limited to the class attribute.
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
is.DCB7 <- x$atoms$resname == "DCB" & x$atoms$resid == 7
is.DCB8 <- x$atoms$resname == "DCB" & x$atoms$resid == 8
d <- distances(x, is.DCB7, is.DCB8)
norm(d, type = "xyz")
norm(d, type = "xy")
norm(d, type = "x")
d <- dist.point(x, c(0,0,0), subset = is.DCB7)
norm(d, type = "xyz")

}
\seealso{
\code{\link{coords}}, \code{\link{basis}}, \code{\link{xyz2abc}}, \code{\link{abc2xyz}}
}
\keyword{classes}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chains.R
\name{chains}
\alias{chains}
\alias{chains.atoms}
\alias{chains.pdb}
\title{Extract Chain-Codes from a PDB Molecule
Extract the codes of all chains from a PDB object.
\code{chains} is a generic function to extract the ids
of the chains from a PDB molecule.}
\usage{
chains(x, ...)

\method{chains}{atoms}(x, sorted = TRUE, ...)

\method{chains}{pdb}(x, ...)
}
\arguments{
\item{x}{Object of class \code{pdb} or class \code{atoms};}

\item{\dots}{arguments passed to other methods.}

\item{sorted}{logical indicating if the returned chain-codes should be sorted;}
}
\value{
\code{chains} returns the codes of the individual chains in the molecule.
}
\description{
Extract Chain-Codes from a PDB Molecule
Extract the codes of all chains from a PDB object.
\code{chains} is a generic function to extract the ids
of the chains from a PDB molecule.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crystal.R
\name{crystal}
\alias{crystal}
\alias{cryst1}
\alias{crystal.default}
\alias{is.cryst1}
\alias{is.crystal}
\title{Create \sQuote{crystal} Object}
\usage{
cryst1(...)

\method{crystal}{default}(abc, abg = c(90, 90, 90), sgroup = "P1", ...)

is.cryst1(x)

is.crystal(x)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{abc}{a numeric vector of length 3 containing the norms of the lattice
vectors a, b and c.}

\item{abg}{a numeric vector of length 3 containing the angles between the
lattice vectors \eqn{\alpha}, \eqn{\beta} and \eqn{\gamma}.}

\item{sgroup}{a character string giving the Hermann-Mauguin symbol of the
space group.}

\item{x}{an R object to be tested.}
}
\value{
Function \code{crystal} returns a list of class \sQuote{crystal} with the
following components:
\item{abc}{a numeric vector of length 3 containing the norms of the lattice
  vectors a, b and c.}
\item{abg}{a numeric vector of length 3 containing the angles between the
  lattice vectors \eqn{\alpha}, \eqn{\beta} and \eqn{\gamma}.}
\item{sgroup}{a character string giving the Hermann-Mauguin symbol of the space group.}

Function \code{is.crystal} returns TRUE if \code{x} is an object of class \sQuote{crystal}
  and FALSE otherwise.
}
\description{
Create an object of class \sQuote{crystal} containing the unit cell parameters
and the name of the space group to associate with an object of class 
\sQuote{pdb}. Note: the \sQuote{cryst1} class will be deprecated.
}
\details{
\code{crystal} is a generic function to create objects of class 
\sQuote{crystal}. The purpose of this class is to store CRYST1 records from 
PDB files which contain the unit cell parameters and the name of the space 
group of a molecular system stored in a PDB file. The default method of the 
\code{crystal} function creates an object of class \sQuote{crystal} from its 
different components, i.e.: \code{abc}, \code{abg} and \code{sgroup}. At 
least \code{abc} has to be specified. \cr\cr \code{is.crystal} tests if an 
object is of class \sQuote{crystal}, i.e. if it has a \dQuote{class} attribute 
equal to \code{crystal}.
}
\examples{
x <- crystal(abc = c(10, 10, 10), abg = c(90,90,90), sgroup = "P1")
is.crystal(x)
 
}
\seealso{
\code{\link{cell.coords}}, \code{\link{pdb}}
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/toSymbols.R
\name{toSymbols}
\alias{toSymbols}
\alias{toSymbols.integer}
\alias{toSymbols.numeric}
\alias{toSymbols.character}
\title{Atomic Symbols Converter}
\usage{
toSymbols(x, ...)

\method{toSymbols}{integer}(x, ...)

\method{toSymbols}{numeric}(x, ...)

\method{toSymbols}{character}(x, nletters = 3, na = NA, ...)
}
\arguments{
\item{x}{a vector to be converted into atomic symbols.}

\item{\dots}{further arguments passed to or from other methods.}

\item{nletters}{an integer used to truncate the character strings before conversion.}

\item{na}{the default value to use for invalid or missing element symbols.}
}
\value{
a character vector containing atomic symbols
}
\description{
Converts character strings or atomic numbers into atomic symbols.
}
\details{
Each elements of \code{x} are converted into atomic symbols.\cr
When \code{x} is an integer (or numeric) vector, atomic number are search into the \code{elements} data set to find associated atomic symbols.\cr
When \code{x} is a character vector, \code{toSymbols} first removes all leading and trailing white spaces and numbers.
Then translates the first character of the character strings to uppercase and all the others to lowercase.
Finally, the character strings are tested for matching with element symbols provided by the \code{elements} data set.
NA are produced for no matching.
}
\examples{
x <- c(1:10)
toSymbols(x)

x <- c("C  "," o","h1","1h","UU","SI0","cR")
toSymbols(x)

# 'nletters' can be used to truncate the character strings before
# conversion, if needed:
toSymbols("SIL", nletters=3) # return NA
toSymbols("SIL", nletters=2) # return "Si"
toSymbols("SIL", nletters=1) # return "S"
toSymbols("SIL", nletters=3, na="X") # return "X"

}
\seealso{
\code{\link{elements}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mirrorHelpers.R
\name{mirrorHelpers}
\alias{mirrorHelpers}
\alias{Mxy}
\alias{Mxy.coords}
\alias{Mxy.pdb}
\alias{Myz}
\alias{Myz.coords}
\alias{Myz.pdb}
\alias{Mzx}
\alias{Mzx.coords}
\alias{Mzx.pdb}
\alias{Mab}
\alias{Mab.coords}
\alias{Mab.pdb}
\alias{Mbc}
\alias{Mbc.coords}
\alias{Mbc.pdb}
\alias{Mca}
\alias{Mca.coords}
\alias{Mca.pdb}
\title{Helper Functions for reflection of Atomic Coordinates}
\usage{
Mxy(...)

\method{Mxy}{coords}(x, mask = TRUE, crystal = NULL, ...)

\method{Mxy}{pdb}(x, mask = TRUE, crystal = x$crystal, ...)

Myz(...)

\method{Myz}{coords}(x, mask = TRUE, crystal = NULL, ...)

\method{Myz}{pdb}(x, mask = TRUE, crystal = x$crystal, ...)

Mzx(...)

\method{Mzx}{coords}(x, mask = TRUE, crystal = NULL, ...)

\method{Mzx}{pdb}(x, mask = TRUE, crystal = x$crystal, ...)

Mab(...)

\method{Mab}{coords}(x, crystal, mask = TRUE, ...)

\method{Mab}{pdb}(x, crystal = x$crystal, mask = TRUE, ...)

Mbc(...)

\method{Mbc}{coords}(x, crystal, mask = TRUE, ...)

\method{Mbc}{pdb}(x, crystal = x$crystal, mask = TRUE, ...)

Mca(...)

\method{Mca}{coords}(x, crystal, mask = TRUE, ...)

\method{Mca}{pdb}(x, crystal = x$crystal, mask = TRUE, ...)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{x}{an R object containing atomic coordinates.}

\item{mask}{a logical vector indicating the set of coordinates to which to
apply the reflection.}

\item{crystal}{an object of class \sQuote{crystal} used to convert fractional
into Cartesian coordinates (when needed).}
}
\value{
An object of the same class as \code{x} with reflected coordinates.
}
\description{
Reflection of atomic coordinates with respect to a specific Cartesian plane or
a plane defined by two lattice vectors.
}
\details{
These functions are helper functions to perform a reflection with respect to 
a specific Cartesian plan or a plan defined by two lattice vectors. All of 
them call the \code{mirror} function.
}
\examples{
# First lets read a pdb file
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
visualize(x, mode = NULL)

# Mirror operation with respect to the ab-plane
visualize(Mab(x), mode = NULL)
# Mirror operation with respect to the ab-plane for residue 1
visualize(Mab(x, mask = x$atoms$resid == 1), mode = NULL)

}
\seealso{
\code{\link{mirror}} and \code{\link{xyz2abc}},
\code{\link{abc2xyz}} for passing from Cartesian to fractional coordinates
(or Vice Versa).
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unsplit.R
\name{unsplit}
\alias{unsplit}
\alias{unsplit.default}
\title{Reassemble Groups}
\usage{
unsplit(value, f, drop = FALSE, ...)

\method{unsplit}{default}(value, f, drop = FALSE, ...)
}
\arguments{
\item{value}{a list of vectors or data frames compatible with a splitting of
\code{x}. Recycling applies if the lengths do not match.}

\item{f}{a \sQuote{factor} in the sense that \code{\link{as.factor}(f)}
defines the grouping, or a list of such factors in which case their
interaction is used for the grouping.}

\item{drop}{logical indicating if levels that do not occur should be dropped
(if \code{f} is a \code{factor} or a list).}

\item{\dots}{further potential arguments passed to methods.}
}
\value{
Returns a vector or data frame for which \code{split(x, f)} equals
  \code{value}
}
\description{
\code{unsplit} reverses the effect of \code{split}.
}
\details{
\code{unsplit} is a generic functions with a default method (Method dispatch 
takes place based on the class of the first element of \code{value}) working 
with lists of vectors or data frames (assumed to have compatible structure, 
as if created by \code{split}). It puts elements or rows back in the 
positions given by \code{f}. In the data frame case, row names are obtained 
by unsplitting the row name vectors from the elements of \code{value}. \cr 
\code{f} is recycled as necessary and if the length of \code{x} is not a 
multiple of the length of \code{f} a warning is printed. \cr Any missing 
values in \code{f} are dropped together with the corresponding values of 
\code{x}.
}
\examples{
require(stats); require(graphics)
n <- 10; nn <- 100
g <- factor(round(n * runif(n * nn)))
x <- rnorm(n * nn) + sqrt(as.numeric(g))
xg <- split(x, g)
boxplot(xg, col = "lavender", notch = TRUE, varwidth = TRUE)
sapply(xg, length)
sapply(xg, mean)

### Calculate 'z-scores' by group (standardize to mean zero, variance one)
z <- unsplit(lapply(split(x, g), scale), g)

# or

zz <- x
split(zz, g) <- lapply(split(x, g), scale)

# and check that the within-group std dev is indeed one
tapply(z, g, sd)
tapply(zz, g, sd)


### Data frame variation

# Notice that assignment form is not used since a variable is being added

g <- airquality$Month
l <- split(airquality, g)
l <- lapply(l, transform, Oz.Z = scale(Ozone))
aq2 <- unsplit(l, g)
head(aq2)
with(aq2, tapply(Oz.Z,  Month, sd, na.rm=TRUE))


### Split a matrix into a list by columns
ma <- cbind(x = 1:10, y = (-4:5)^2)
split(ma, col(ma))

split(1:10, 1:2)

}
\references{
Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988) \emph{The New S Language}. Wadsworth & Brooks/Cole.
}
\seealso{
\code{\link[base]{cut}} to categorize numeric values.
\cr
\code{\link[base]{strsplit}} to split strings.
}
\keyword{category}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.pdb.R
\name{read.pdb}
\alias{read.pdb}
\title{PDB File Reader}
\usage{
read.pdb(
  file,
  ATOM = TRUE,
  HETATM = TRUE,
  CRYSTAL = TRUE,
  CONNECT = TRUE,
  TITLE = TRUE,
  REMARK = TRUE,
  MODEL = 1,
  CRYST1 = NULL,
  resolution = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{file}{a single element character vector containing the name of the PDB file to be read.}

\item{ATOM}{a logical value indicating whether to read the ATOM records.}

\item{HETATM}{a logical value indicating whether to read the HETATM records.}

\item{CRYSTAL}{a logical value indicating whether to read the crystal cell parameters (from the CRYST1 pdb record).}

\item{CONNECT}{a logical value indicating whether to read the CONNECT records.}

\item{TITLE}{a logical value indicating whether to read the TITLE records.}

\item{REMARK}{a logical value indicating whether to read the REMARK records.}

\item{MODEL}{an integer vector containing the serial number of the MODEL sections to be read. Can also be equal to NULL to read all the MODEL sections or to NA to ignore MODEL records (see details).}

\item{CRYST1}{will be replaced by the CRYSTAL argument; existing code should be migrated to use the CRYSTAL argument.}

\item{resolution}{logical value indicating whether to extract the resolution (see details).}

\item{verbose}{logical value indicating whether to print additional information, e.g. number of models.}
}
\value{
When a single MODEL section is read, this function returns an object of class  \sQuote{pdb} (a list with a \code{class} attribute equal to \code{pdb}) with the following components:
\item{title}{a character vector containing the TITLE records found in the PDB file.}
\item{remark}{a character vector containing the REMARK records found in the PDB file.}
\item{crystal}{a list of class \sQuote{crystal} containing the first CRYSTAL record found in the PDB file. All others are ignored.}
\item{atoms}{a data.frame of class \sQuote{atoms} containing the ATOM and HETATM records found in the PDB file.}
\item{connect}{a data.frame of class \sQuote{connect} containing the CONNECT records found in the PDB file.}
When multiple MODEL sections are read, a list of object of class \sQuote{pdb} is returned.
}
\description{
Reads a Protein Data Bank (PDB) coordinates file.
}
\details{
The \code{read.pdb} function reads the TITLE, REMARK, ATOM, HETATM, CRYSTAL and CONECT records
  from a PDB file. Three different reading modes can be used depending on the value of \code{MODEL}: 
\itemize{
  \item When \code{MODEL} is a vector of integers, MODEL sections whose serial numbers
     match these integers are read.
  \item When \code{MODEL == NULL}, all MODEL sections are read.
  \item When \code{MODEL == NA}, MODEL records are ignored and all ATOM and/or HETATM records
    are merged together to return a single object.
}
When multiple models are specified, each of the models is actually stored
  as a separate pdb molecule in a list of pdb molecules.
If the \code{resolution} parameter is set, the function attempts to extract the resolution
  from the REMARKS field. Note: The resolution is only meaningful for X-ray crystallography.
}
\examples{
# Read a PDB file included with the package
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))

# Visualize the PDB file
visualize(x, mode = NULL)

# Write the 'pdb' object 'x' in file "Rpdb.pdb" into the current directory
write.pdb(x, file = "Rpdb.pdb")

# Cleanup
unlink("Rpdb.pdb")

}
\references{
PDB format has been taken from:
http://www.wwpdb.org/documentation/format33/v3.3.html
}
\seealso{
\code{\link{write.pdb}}, \code{\link{pdb}}, \code{\link{crystal}}, \code{\link{atoms}}, \code{\link{connect}}
}
\keyword{IO}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bond-angle-dihedral.R, R/measure.R
\name{bond-angle-dihedral}
\alias{bond-angle-dihedral}
\alias{bond}
\alias{bond.coords}
\alias{bond.pdb}
\alias{angle}
\alias{angle.coords}
\alias{angle.pdb}
\alias{dihedral}
\alias{dihedral.coords}
\alias{dihedral.pdb}
\alias{measure}
\alias{measure.default}
\alias{measure.coords}
\alias{measure.pdb}
\title{Atomic Bond Lengths, Angles and Dihedrals}
\usage{
bond(...)

\method{bond}{coords}(x, sel1, sel2, ...)

\method{bond}{pdb}(x, sel1, sel2, ...)

angle(...)

\method{angle}{coords}(x, sel1, sel2, sel3, ...)

\method{angle}{pdb}(x, sel1, sel2, sel3, ...)

dihedral(...)

\method{dihedral}{coords}(x, sel1, sel2, sel3, sel4, ...)

\method{dihedral}{pdb}(x, sel1, sel2, sel3, sel4, ...)

measure(...)

\method{measure}{default}(id = rgl::rgl.ids(), verbose = TRUE, ...)

\method{measure}{coords}(x, id = rgl::rgl.ids(), verbose = TRUE, ...)

\method{measure}{pdb}(x, id = rgl::rgl.ids(), verbose = TRUE, ...)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{x}{an R object containing atomic coordinates.}

\item{sel1, sel2, sel3, sel4}{an integer or logical vector used to select atoms 
defining bonds, angles or dihedrals. See details.}

\item{id}{vector of ID numbers of \sQuote{rgl} items, as returned by 
\code{rgl.ids}. The vertexes of these items are used to compute the bond 
lengths, angles or dihedrals.}

\item{verbose}{a logical value specifying if the information have to be 
printed to the terminal.}
}
\value{
A numeric vector containing atomic bond lengths (in Angstrom), angles
or dihedrals (in degrees)
}
\description{
Compute bond lengths, angles and dihedral from atomic coordinates.
}
\details{
The number of selected atoms with \code{sel1}, \code{sel2}, \code{sel3} and 
\code{sel4} must be the same. \code{sel1}, \code{sel2}, \code{sel3} and 
\code{sel4} respectively select the first, second, third and fouth atoms 
defining bonds, angles or dihedrals.\cr\cr\code{measure} activate an 
interactive mode to compute bond lengths, angles and dihedrals by selecting 
atoms by \bold{right-clicing} on the current \sQuote{rgl} scene. To escape 
the active mode press the ESC key.
}
\examples{
Pen <- read.pdb(system.file("examples/Pentacene.pdb", package="Rpdb"))
visualize(Pen, mode = NULL)
text3d(coords(Pen), texts = Pen$atoms$eleid)
bond(Pen, 3:4, 1:2)
angle(Pen, 3:4, 1:2, 5:6)
dihedral(Pen, 3:4, 1:2, 5:6, 6:5)

}
\seealso{
\code{\link{coords}}, \code{\link{pdb}}, \code{\link{info3d}},
  \code{\link{visualize}}
}
\keyword{dynamic}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rpdb-package.R
\name{Rpdb-package}
\alias{Rpdb-package}
\title{Read, Write, Visualize and Manipulate PDB Files}
\description{
Provides tools to read, write, visualize PDB files, and perform structural manipulations.
}
\details{
This package enables users, e.g. computational chemists,
to manipulate molecular structures stored in PDB files.
It enables users ro read, write and visualize PDB files.
Various basic structural manipulations are also supported.
Conversion of Cartesian coordinates into fractional coordinates.
Spliting a molecular structure into fragments.
Computation of centers-of-geometry and centers-of-mass.
Wrapping molecular structure using periodical boundary conditions.
Translation, rotation and reflection of atomic coordinates.
Calculate atomic bond lengths, angles and dihedrals.
}
\examples{
## Read a PDB file included in the package
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))

## Visualize the PDB file
visualize(x, mode = NULL)

## From Cartesian to fractional coordinates and vice versa
x <- xyz2abc(x)
basis(x)
natom(x, x$atoms$resid)
range(x)
centres(x)
x <- abc2xyz(x)
basis(x)
natom(x, x$atoms$resid)
range(x)
centres(x)

## Split and unsplit
F <- x$atoms$resid
x <-   split(x, F)
x <- unsplit(x, F)

## Subset and merge
x.PCB.only <- subset(x, resname == "PCB")
x.DCB.only <- subset(x, resname == "DCB")
x <- merge(x.PCB.only, x.DCB.only)

## Duplicate and wrap
x <- replicate(x, a.ind = -1:1, b.ind = -1:1, c.ind = -1:1)
x <- wrap(x)

## Write the 'pdb' object 'x' to a temporary file.
write.pdb(x, file = tempfile())

}
\references{
More information on the PDB format can be found here:\cr
http://www.wwpdb.org/documentation/format33/v3.3.html
}
\author{
Julien Idé \email{julien.ide.fr@gmail.com}\cr
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/viewAxis.R
\name{viewAxis}
\alias{viewAxis}
\alias{viewXY}
\alias{viewYZ}
\alias{viewZX}
\alias{viewAB}
\alias{viewBC}
\alias{viewCA}
\alias{viewInertia}
\title{Set the View of the \sQuote{rgl} Scene}
\usage{
viewAxis(V1, V2)

viewXY()

viewYZ()

viewZX()

viewAB(crystal)

viewBC(crystal)

viewCA(crystal)

viewInertia(x, m = NULL)
}
\arguments{
\item{V1}{a length 3 numeric vector.}

\item{V2}{a length 3 numeric vector.}

\item{crystal}{an object of class \sQuote{crystal}.}

\item{x}{an R object containing atomic coordinates.}

\item{m}{a numeric vector containing atomic masses.}
}
\value{
No return value, called for side effects.
}
\description{
Set the view of the current \sQuote{rgl} scene aligning one vector 
perpendicularly to the screen and placing another one in the horizontal plane.
}
\details{
\code{viewAxis} set the view of the current rgl scene (by setting
\code{UserMatrix}; see \code{\link[rgl]{par3d}} for more details) so that
\code{V1} is perpendicular to the screen and \code{V2} is in the horizontal
plane. The other functions documented here are helper functions calling
\code{viewAxis} to set the view using particular Cartesian or lattice
vectors. For functions \code{viewAB}, \code{viewBC} and \code{viewCA} a
\sQuote{crystal} object has to be specified to define the lattice vectors
used to set the view. The function \code{viewInertia} computes the inertia
tensor from the atomic coordinates and masses (see \code{\link{inertia}})
and sets the view to its eigen vectors basis set.
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
visualize(x, mode = NULL)
viewAB(x$crystal)

C70 <- read.pdb(system.file("examples/C70.pdb", package="Rpdb"))
visualize(C70, mode = NULL)
viewXY()
viewInertia(C70)

}
\seealso{
\code{\link{visualize}}, \code{\link{cell.coords}}, \code{\link[rgl]{par3d}}, \code{\link[rgl]{rgl.open}}
}
\keyword{dynamic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Graphics.Tools.R
\name{translate}
\alias{translate}
\alias{trans}
\title{Graphics Tools}
\usage{
trans(x, y = 0, z = 0, auto = FALSE)

translate(x, y = 0, z = 0, auto = FALSE)
}
\arguments{
\item{x, y, z}{amount to shift the observer in the x, y and z directions.}

\item{auto}{If TRUE, the location will be set automatically by RGL
to make the whole bounding box visible.
Unfortunately, it may fail, as it seems there are some bugs in rgl.}
}
\value{
\code{translate} returns invisibly the old position of the observer.
}
\description{
Tools to modify or set \code{rgl} objects, scenes and the observer.
\code{translate} performs a translation of the observer to a new position
  specified using relative shifts.
\code{trans} is an abbreviated alias to function \code{translate}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addAxes.R
\name{addAxes}
\alias{addAxes}
\alias{addABC}
\alias{addXYZ}
\alias{addPBCBox}
\alias{addBBox}
\alias{bbox.pdb}
\title{Add Axes or PBC Box to the \sQuote{rgl} Scene}
\usage{
addABC(x, lwd = 2, scale = NULL, labels = TRUE, cex = 2, adj.lab = 1.2)

addXYZ(lwd = 2, scale = NULL, labels = TRUE, cex = 2, dx = 5)

addPBCBox(x, scale = NULL, lwd = 2, col = "black", alpha = 0.25)

addBBox(x, lwd = 2, col = "black", alpha = 0.25)

bbox.pdb(x)
}
\arguments{
\item{x}{an object of class \sQuote{crystal} containing the unit cell parameters.}

\item{lwd}{a numeric value indicating the line width used to draw the axes or
the PBC box.}

\item{scale}{a scalar value, or a numeric vector of length 2 or 3 or 6, or a 3x3 matrix
used to scale the PBC box or the axes; a length 3 vector is converted to a diagonal matrix,
while vectors of length 2 or 4 or 6 are converted to a diagonal matrix for scaling
and a relative shift;}

\item{labels}{a logical value indicating whether to draw the labels of the axes.}

\item{cex}{a numeric value indicating the magnification used to draw the 
labels of the axes.}

\item{adj.lab}{numeric value performing relative adjustment of label position;}

\item{dx}{a numeric value specifying the absolute length of the xyz axes-vectors;}

\item{col}{colour used for the PBC box;}

\item{alpha}{a numeric value specifying the transparency of the lines;}
}
\value{
Returns invisibly a two-column data.frame containing the IDs 
  and type indicators of the objects added to the scene.

\code{bbox.pdb}: Returns the coordinates of the bounding box.\cr
}
\description{
Add lattice vectors, Cartesian axes or PBC box to the current \sQuote{rgl} scene.
}
\details{
\code{addABC}: Add the lattice vectors a, b and c to the current rgl device.\cr
\code{addXYZ}: Add the Cartesian axes x, y and z to the current rgl device.\cr
\code{addPBCBox}: Add a box representing the Periodic Boundary Conditions
  of a molecular system.\cr
\code{addBBox}: Add a bounding box around the PDB molecule
  constructed using the lattice vectors.\cr
\code{bbox.pdb}: Computes the bounding box around the PDB molecule.\cr
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
visualize(x, type = "l", xyz = FALSE, abc = FALSE, pbc.box = FALSE, mode = NULL)
addXYZ()
addABC(x$crystal)
addPBCBox(x$crystal)

}
\seealso{
\code{\link{visualize}}, \code{\link[rgl]{rgl.open}}, \code{\link[rgl]{par3d}},
  \code{\link{addLabels}}
}
\keyword{dynamic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/centres.roll.R
\name{centres.ppRoll}
\alias{centres.ppRoll}
\alias{centres.ppRoll.default}
\alias{centres.ppRoll.atoms}
\alias{centres.ppRoll.pdb}
\title{Centres-of-Geometry: Rolling Window}
\usage{
\method{centres}{ppRoll}(x, window, ...)

\method{centres.ppRoll}{default}(x, window = 34, na.rm = TRUE, ...)

\method{centres.ppRoll}{atoms}(x, window = 34, chain = NULL, na.rm = TRUE, ...)

\method{centres.ppRoll}{pdb}(x, window = 34, chain = NULL, na.rm = TRUE, ...)
}
\arguments{
\item{x}{an R object containing atomic coordinates (with additional class \code{ppRoll}).}

\item{window}{size of the rolling window, specified as number of amino-acids;}

\item{\dots}{further arguments passed to or from other methods.}

\item{na.rm}{a logical value indicating whether NA values should be stripped
before the computation proceeds.}

\item{chain}{apply only to the respective chains, by default all chains;}
}
\value{
Return an object of class \sQuote{coords} containing the coordinates
  of the centres.
}
\description{
Computes the geometric centres using a rolling window on a polypeptide chain
}
\details{
\sQuote{centres.ppRoll} is a generic function to compute the geometric centres
using a rolling window on an object containing atomic coordinates for
a polypeptide chain. Due to limitations by CRAN check, the function is further
referred as \sQuote{centres}.

The function may be useful to visualize the overall protein structure,
but in a less cluttered way. Unlike the protein backbone, the centres
may capture better the bulk of the protein.
}
\examples{
### Example 1: Toll-Like Receptor 2
# Download pdb file for TLR2: 2Z81;
# Read the pdb-file:
# x = read.pdb("pdb2z81.ent")

# Compute the centres:
# tmp = centres.ppRoll(x)

# Visualize the structure:
# visualize(x, type="p", pbc.box = FALSE)
# lines3d(tmp, lwd = 5, col = "red")

# Add Another "strand":
# tmp = centres.ppRoll(x, window = 8)
# lines3d(tmp, lwd = 5, col = "#FA3296")
}
\seealso{
\code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rotation.R
\name{rotation}
\alias{rotation}
\alias{R}
\alias{R.coords}
\alias{R.pdb}
\title{Rotation of Atomic Coordinates}
\usage{
R(...)

\method{R}{coords}(obj, angle = 0, x = 0, y = 0, z = 1, mask = TRUE, crystal = NULL, ...)

\method{R}{pdb}(obj, angle = 0, x = 0, y = 0, z = 1, mask = TRUE, crystal = obj$crystal, ...)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{obj}{an R object containing atomic coordinates.}

\item{angle}{the angle of the rotation in degrees.}

\item{x}{the x-component of the rotation vector.}

\item{y}{the y-component of the rotation vector.}

\item{z}{the z-component of the rotation vector.}

\item{mask}{a logical vector indicating the set of coordinates to which the rotation has to be applyed.}

\item{crystal}{an object of class \sQuote{crystal} used to convert fractional into Cartesian coordinates (when needed).}
}
\value{
An object of the same class as \code{x} with rotated coordinates.
}
\description{
Rotation of atomic coordinates around a given vector.
}
\details{
\code{R} is a generic function. Method for objects of class \sQuote{coords}
first convert the coordinates into Cartesian coordinates using \code{crystal}
if needed. Once rotated, the coordinates are reconverted back to the original
basis set using again \code{crystal}. Method for objects of class \sQuote{pdb}
first extracts coordinates from the object using the function \code{coords},
performs the rotation, and updates the coordinates of the \sQuote{pdb} object
using the function \code{coords<-}.
}
\examples{
# First lets read a pdb file
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
cell <- cell.coords(x)
visualize(x, mode = NULL)

# Rotation of the structure around the c-axis
visualize(R(x, 90, x=cell["x","c"], y=cell["y","c"], z=cell["z","c"]),
          mode = NULL)
# Rotation of the residue 1 around the c-axis
visualize(R(x, 90, x=cell["x","c"], y=cell["y","c"], z=cell["z","c"], mask = x$atoms$resid == 1),
          mode = NULL)
          
}
\seealso{
Helper functions for rotation around a given Cartesian vector:\cr
\code{\link{Rx}}, \code{\link{Ry}}, \code{\link{Rz}}\cr
Passing from Cartesian to fractional coordinates (or Vis Versa):\cr
\code{\link{xyz2abc}}, \code{\link{abc2xyz}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select.R
\name{select}
\alias{select}
\alias{select.pdb}
\alias{select.atoms}
\title{Select atoms}
\usage{
select(x, id, ...)

\method{select}{pdb}(x, id, residue, chain, ...)

\method{select}{atoms}(x, id, residue, chain, ...)
}
\arguments{
\item{x}{an R object containing atoms;}

\item{id}{an integer vector specifying the id of residues.}

\item{\dots}{arguments passed to methods.}

\item{residue}{a character vector specifying the residues to select.}

\item{chain}{character vector specifying which chains to process;}
}
\value{
\code{select} returns a vector with the element IDs.
}
\description{
Select groups of atoms from a PDB object.
}
\details{
\code{select} is a generic function, which enables
  the selection of element IDs from molecules.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write.pdb.R
\name{write.pdb}
\alias{write.pdb}
\title{PDB File Writer}
\usage{
write.pdb(x, file = "Rpdb.pdb")
}
\arguments{
\item{x}{an object, or a list of objects, of class \sQuote{pdb}.}

\item{file}{a single element character vector containing the name of the PDB file to be created.}
}
\value{
No return value, called for side effects.
}
\description{
Writes a Protein Data Bank (PDB) coordinate file from an object of class \sQuote{pdb}.
}
\details{
All data stored in the \sQuote{pdb} object are written to a PDB file. A list of objects of class \sQuote{pdb} can be provided to write multiple MODEL data into a single file. In this case, each \sQuote{pdb} object of the list must have the same \code{crystal} and \code{connect} components.
\cr
To write only a subset of a \sQuote{pdb} object see function \code{\link{subset.pdb}}.
}
\examples{
# Read a PDB file included with the package
pdb <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))

# Write the pdb object to file "Rpdb.pdb" in the current directory
write.pdb(pdb, file = "Rpdb.pdb")

# Cleanup
unlink("Rpdb.pdb")

}
\references{
PDB format is described at:
http://www.wwpdb.org/documentation/format33/v3.3.html
}
\seealso{
\code{\link{read.pdb}}, \code{\link{pdb}}, \code{\link{crystal}}, \code{\link{atoms}}, \code{\link{connect}}, \code{\link{subset.pdb}}
}
\keyword{IO}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellProperties.R
\name{cellProperties}
\alias{cellProperties}
\alias{cell.coords}
\alias{cell.coords.default}
\alias{cell.coords.crystal}
\alias{cell.coords.pdb}
\alias{cell.volume}
\alias{cell.volume.crystal}
\alias{cell.volume.pdb}
\alias{cell.density}
\alias{cell.density.default}
\alias{cell.density.pdb}
\title{Properties of a Unit Cell}
\usage{
cell.coords(...)

\method{cell.coords}{default}(abc, abg = c(90, 90, 90), digits = 3, ...)

\method{cell.coords}{crystal}(x, digits = 3, ...)

\method{cell.coords}{pdb}(x, digits = 3, ...)

cell.volume(...)

\method{cell.volume}{crystal}(x, ...)

\method{cell.volume}{pdb}(x, ...)

cell.density(...)

\method{cell.density}{default}(masses, volume, ...)

\method{cell.density}{pdb}(x, ...)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{abc}{a length 3 numeric vector containing the length of the a, b and c lattice vectors.}

\item{abg}{a length 3 numeric vector containing the angles (degrees) between the a, b and c lattice vectors (alpha, beta, gamma).}

\item{digits}{an integer used to round the lattice vectors coordinates.}

\item{x}{an R object containing lattice parameters.}

\item{masses}{a numeric vector containing atomic masses.}

\item{volume}{a single element numeric vector containing the volume of the unit cell in Angstrom cube.}
}
\value{
\code{cell.coords} returns a 3x3 matrix containing the Cartesian coordinates of lattice vectors arranged by columns.\cr
\code{cell.volume} returns a single element numeric vector containing the volume of the unit cell in Angstrom cube.\cr
\code{cell.density} returns a single element numeric vector containing the density of the unit cell in g.cm-3.
}
\description{
Compute the Cartesian coordinates of lattice vectors, the volume or the density of a unit cell.
}
\details{
\code{cell.coords} is a generic function which computes a 3x3 matrix whose columns contrain the Cartesian coordinates of lattice vectors.
The 'a' and 'b' vectors are assumed to be respectively along the x-axis and in the xy-plane.
The default method takes directly the lattice parameters as arguments.
For objects of class \code{\link{crystal}} the lattice parameters are first extracted from the object and then the default method is called.
For objects of class \code{\link{pdb}} the lattice parameters are extracted from their \code{crystal} component and the default method is called.
\cr
\cr
\code{cell.volume} is a generic function to compute the volume of a unit cell.
For objects of class \sQuote{crystal}, the unit cell parameters are directly used to compute the volume.
For objects of class \sQuote{pdb}, their \code{crystal} component is used.
\cr
\cr
\code{cell.density} is a generic function to compute the density of a unit cell.
For objects of class \sQuote{pdb}:
First the volume of the unit cell is calculated by calling the \code{cell.volume} function on the \code{crystal} component of the \sQuote{pdb} object.
Then the element names are converted into element symbols using the \code{toSymbols} function and their masses are taken from the \code{elements} data set.
Finally the density is calculated using the sum of the atomic masses and the volume of the unit cell.
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
cell.volume(x)
cell.density(x)
cell.coords(x)

}
\seealso{
\code{\link{crystal}}, \code{\link{pdb}}, \code{\link{xyz2abc}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addLabels.R
\name{addLabels}
\alias{addLabels}
\alias{addResLab}
\alias{addResLab.atoms}
\alias{addResLab.pdb}
\alias{addEleLab}
\alias{addEleLab.atoms}
\alias{addEleLab.pdb}
\alias{info3d}
\alias{info3d.atoms}
\alias{info3d.pdb}
\title{Add Labels to the \sQuote{rgl} Scene}
\usage{
addResLab(x, ...)

\method{addResLab}{atoms}(x, at.centre = TRUE, col = "black", ...)

\method{addResLab}{pdb}(x, at.centre = TRUE, col = "black", ...)

addEleLab(x, ...)

\method{addEleLab}{atoms}(x, eleid = FALSE, col = "black", ...)

\method{addEleLab}{pdb}(x, eleid = FALSE, col = "black", ...)

info3d(...)

\method{info3d}{atoms}(x, id = rgl::rgl.ids(), col = "black", verbose = TRUE, adj = 0, ...)

\method{info3d}{pdb}(x, id = rgl::rgl.ids(), col = "black", verbose = TRUE, adj = 0, ...)
}
\arguments{
\item{x}{an R object containing atomic coordinates.}

\item{\dots}{further arguments passed to or from other methods.}

\item{at.centre}{a single element logical vector indicating if residue labels
have to be added only at the position of the residue's centre-of-mass 
instead of at each atomic position.}

\item{col}{the colors used to display the labels.}

\item{eleid}{a single element logical vector indicating if the element ids 
have to be concatenated with the element names to prepare the labels.}

\item{id}{vector of ID numbers of \sQuote{rgl} items, as returned by 
\code{rgl.ids}. The vertexes of these items are used to display the labels.}

\item{verbose}{a logical value specifying if information have to be printed 
to the terminal.}

\item{adj}{one value specifying the horizontal adjustment, or two, specifying 
horizontal and vertical adjustment respectively. See 
\code{\link[rgl:text3d]{text3d}}}
}
\value{
\code{addResLab} and \code{addEleLab} return (using invisible) a
two-column data.frame containing the IDs and type indicators of the objects
added to the scene.
}
\description{
Add Labels to the current \sQuote{rgl} scene.
}
\details{
\code{addResLab} add residue labels to the scene. If \code{at.centre==TRUE} 
only one label per residue is added at the centre of the residue. Otherwise, 
residue labels are added at each atomic positions. \code{addEleLab} add 
element labels to the scene at each atomic positions. \code{info3d} activate 
an interactive mode to add labels by selecting atoms by \bold{right-clicking} 
on the current \sQuote{rgl} scene. To escape the interactive mode press the 
ESC key. The labels are as follow: "ResidResname:EleidElename"
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
visualize(x, type = "l", mode = NULL)
addResLab(x)
x <- read.pdb(system.file("examples/Pentacene.pdb", package="Rpdb"))
visualize(x, type = "l", mode = NULL)
addEleLab(x)

}
\seealso{
\code{\link{pdb}}, \code{\link{visualize}}, \code{\link{measure}}
}
\keyword{dynamic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xyz2abc.R
\name{xyz2abc}
\alias{xyz2abc}
\alias{xyz2abc.coords}
\alias{xyz2abc.atoms}
\alias{xyz2abc.pdb}
\alias{xyz2abc.distances}
\alias{abc2xyz}
\alias{abc2xyz.coords}
\alias{abc2xyz.atoms}
\alias{abc2xyz.pdb}
\alias{abc2xyz.distances}
\title{From Cartesian to Fractional Coordinates and Vice Versa}
\usage{
xyz2abc(...)

\method{xyz2abc}{coords}(x, crystal, ..., cryst1 = NULL)

\method{xyz2abc}{atoms}(x, crystal, ...)

\method{xyz2abc}{pdb}(x, crystal = x$crystal, ..., cryst1 = NULL)

\method{xyz2abc}{distances}(x, crystal, ...)

abc2xyz(...)

\method{abc2xyz}{coords}(x, crystal, ..., cryst1 = NULL)

\method{abc2xyz}{atoms}(x, crystal, ...)

\method{abc2xyz}{pdb}(x, crystal = x$crystal, ..., cryst1 = NULL)

\method{abc2xyz}{distances}(x, crystal, ...)
}
\arguments{
\item{\dots}{arguments passed to methods.}

\item{x}{an R object containing atomic coordinates.}

\item{crystal}{an object of class \code{\link{crystal}}.}

\item{cryst1}{will be deprecated; use \code{crystal} instead.}
}
\value{
Return an object of the same class as \code{x}, with atomic 
  coordinates expressed in a different basis set.
}
\description{
Converts Cartesian coordinates into fractional coordinates and vice versa.
}
\details{
For \code{\link{atoms}} and \code{\link{pdb}} objects, the atomic coordinates
are first extracted from \code{x} using the \code{\link{coords}} function. 
Then, using the periodic boundary conditions stored into \code{crystal}, the 
coordinates are converted from Cartesian to fractional (for the 
\code{xyz2abc} functions) or from fractional to Cartesian (for the 
\code{abc2xyz} functions) coordinates. Finally, for \code{\link{atoms}} and 
\code{\link{pdb}} objects, the new atomic coordinates are reassigned to the 
original \code{x} object using the \code{\link{coords<-}} function and 
\code{x} is returned.
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
basis(x)
x <- xyz2abc(x)
basis(x)
x <- abc2xyz(x)
basis(x)

\donttest{
  
  # This example returns an error because the coordinates stored
  # into the PDB file are already Cartesian coordinates.
  x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
  try(x <- abc2xyz(x))
}

}
\seealso{
\code{\link{basis}}, \code{\link{coords}}, \code{\link{atoms}},
\code{\link{pdb}}, \code{\link{crystal}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/masses.R
\name{masses}
\alias{masses}
\alias{masses.default}
\alias{masses.pdb}
\title{Mass of Chemical Elements}
\usage{
masses(...)

\method{masses}{default}(x, ...)

\method{masses}{pdb}(x, ...)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{x}{either a character or an integer vector containing element symbols
or atomic numbers, or an object of class \sQuote{pdb} from which element
symbols are determined (see details).}
}
\value{
Return a numeric vector containing the mass of chemical elements.
}
\description{
Determine the mass of chemical elements
}
\details{
\code{masses} is a generic function to determine the mass of chemical 
elements. \cr\cr For objects of class \sQuote{pdb}:
\itemize{ \item First the
element names are converted into element symbols using the \code{toSymbols} 
function. \item Then their masses are taken from the \code{elements} data 
set. } \code{NA} values are returned for unrecognized elements.
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb",package="Rpdb"))
masses(x)

masses(c("C","Cl",NA,"AA","N"))

}
\seealso{
\code{\link{toSymbols}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atoms.R
\name{atoms}
\alias{atoms}
\alias{atoms.pdb}
\alias{atoms.atoms}
\alias{atoms.default}
\alias{is.atoms}
\title{Create \sQuote{atoms} Object}
\usage{
atoms(...)

\method{atoms}{pdb}(x, ...)

\method{atoms}{atoms}(x, ...)

\method{atoms}{default}(
  recname,
  eleid,
  elename,
  alt,
  resname,
  chainid,
  resid,
  insert,
  x1,
  x2,
  x3,
  occ,
  temp,
  segid,
  basis = "xyz",
  symbol = NULL,
  isHetero = NULL,
  ...
)

is.atoms(x)
}
\arguments{
\item{\dots}{arguments passed to methods.}

\item{x}{an R object to be tested or from whom to extract the \code{atoms} component.}

\item{recname}{a character vector containing the record name for each element.}

\item{eleid}{a integer vector containing the element ID for each element.}

\item{elename}{a character vector containing the element name for each element.}

\item{alt}{a character vector containing the alternate location indicator for each element.}

\item{resname}{a character vector containing the residue name for each element.}

\item{chainid}{a character vector containing the chain ID for each element.}

\item{resid}{a integer vector containing the residue ID for each element.}

\item{insert}{a character vector containing the codes for insertion of residue of each element.}

\item{x1, x2, x3}{a numeric vector containing the first, second and third coordinate for each element.}

\item{occ}{a numeric vector containing the occupancie for each element.}

\item{temp}{a numeric vector containing the temperature factor for each element.}

\item{segid}{a character vector containing the segment ID for each element.}

\item{basis}{a single element character vector indicating the type of basis vector used to express the atomic coordinates.}

\item{symbol}{the chemical symbol of the atom.}

\item{isHetero}{logical indicating if the record corresponds to the non-protein ligands.}
}
\value{
\code{atoms} returns a data.frame of class \sQuote{atoms} with the following components:
\describe{
\item{recname}{a character vector containing the record name for each element.}
\item{eleid}{a integer vector containing the element ID for each element.}
\item{elename}{a character vector containing the element name for each element.}
\item{alt}{a character vector containing the alternate location indicator for each element.}
\item{resname}{a character vector containing the residue name for each element.}
\item{chainid}{a character vector containing the chain ID for each element.}
\item{resid}{a integer vector containing the residue ID for each element.}
\item{insert}{a character vector containing the codes for insertion of residue for each element.}
\item{x1, x2, x3}{a numeric vector containing the first, second and third coordinate for each element.}
\item{occ}{a numeric vector containing the occupencie for each element.}
\item{temp}{a numeric vector containing the temperature factor for each element.}
\item{segid}{a character vector containing the segment ID for each element.}
\item{basis}{a single element character vector indicating the type of basis vector used to express the atomic coordinates.}
}
\code{is.atoms} returns TRUE if \code{x} is an object of class \sQuote{atoms} and FALSE otherwise.
}
\description{
Creates an object of class \sQuote{atoms} containing the data related to ATOM
and HETATM records of a PDB file.
}
\details{
\code{atoms} is a generic function to create objects of class \sQuote{atoms}.
The purpose of this class is to store ATOM and HETATM records from PDB files.

The default method creates an \code{atoms} object from its different components, i.e.:
  \code{recname}, \code{eleid}, \code{elename}, \code{alt},
  \code{resname}, \code{chainid}, \code{resid}, \code{insert},
  \code{x1}, \code{x2}, \code{x3}, \code{occ}, \code{temp},
  \code{segid} and \code{basis}.
All the arguments have to be specified, except \code{basis}, which by default
  is set to "xyz" (Cartesian coordinates).\cr
\code{is.atoms} tests if an object is of class \sQuote{atoms}, i.e.
  it only checks that the object inherits the \dQuote{class} \code{atoms}.
}
\examples{
x <- atoms(recname = c("ATOM","ATOM"), eleid = 1:2, elename = c("H","H"), alt = "",
  resname = c("H2","H2"), chainid = "", resid = c(1,1), insert = "",
  x1 = c(0,0), x2 = c(0,0), x3 = c(0,1), occ = c(0.0,0.0), temp = c(1.0,1.0),
  segid = c("H2","H2"))
print(x)
is.atoms(x)

}
\seealso{
\code{\link{basis}}, \code{\link{coords}}, \code{\link{pdb}}
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrap.R
\name{wrap}
\alias{wrap}
\alias{wrap.coords}
\alias{wrap.atoms}
\alias{wrap.pdb}
\title{Wrap Atomic Coordinates}
\usage{
wrap(x, ...)

\method{wrap}{coords}(x, crystal = NULL, factor = NULL, ...)

\method{wrap}{atoms}(x, crystal = NULL, factor = NULL, ...)

\method{wrap}{pdb}(x, crystal = x$crystal, factor = NULL, ...)
}
\arguments{
\item{x}{an R object containing atomic coordinates to be wrapped.}

\item{\dots}{further arguments passed to or from other methods.}

\item{crystal}{an object of class \sQuote{crystal} containing periodic boundary
conditions used for wrapping.}

\item{factor}{a factor used to wrap the atoms by groups.}
}
\value{
Return an object of class \sQuote{pdb} with wrapped atomic
  coordinates.
}
\description{
Wraps atomic coordinates using periodic boundary conditions.
}
\details{
The \code{wrap} function translates all atoms out of the unit cell back into
the unit cell using periodic boundary conditions. To do so, the \code{wrap}
function first converts Cartesian into fractional coordinates. Then atoms
with fractional coordinates greater than 1 or lower than 0 are respectively
translated by -1 or +1. Finally, if the original atomic coordinates were
Cartesian coordinates their are reconverted into Cartesian coordinates.
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))

# Translation of the atoms along x-axis
x$atoms$x1 <- x$atoms$x1 + 10

# Wrapping the structure
y <- wrap(x)

}
\seealso{
\code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}},
\code{\link{crystal}}, \code{\link{centres.pdb}}, \code{\link{xyz2abc}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/split.pdb.R
\name{split.pdb}
\alias{split.pdb}
\alias{unsplit.pdb}
\title{Divide and Reassemble \sQuote{pdb} Objects}
\usage{
\method{split}{pdb}(x, f, drop = FALSE, ...)

\method{unsplit}{pdb}(value, f, drop = FALSE, ...)
}
\arguments{
\item{x}{an object of class \sQuote{pdb} to be divided into groups.}

\item{f}{a \sQuote{factor} in the sense that
\code{\link{as.factor}(f)} defines the grouping, or a list of such factors
in which case their interaction is used for the grouping.}

\item{drop}{logical indicating if levels that do not occur should be
dropped (if \code{f} is a \code{factor} or a list).}

\item{\dots}{further potential arguments passed to methods.}

\item{value}{a list of 'pdb' objects compatible with a splitting of
\code{x}. Recycling applies if the lengths do not match.}
}
\value{
The value returned from \code{split} is a list of \sQuote{pdb}
objects containing the data for the groups of atoms. The components of the
list are named by the levels of \code{f} (after converting to a factor, or if
already a factor and \code{drop=TRUE}, dropping unused levels). \cr 
\code{unsplit} returns a \sQuote{pdb} object for which \code{split(x, f)}
equals \code{value}.
}
\description{
\code{split} divides a \sQuote{pdb} object by groups of atoms defined by 
\code{f}. \code{unsplit} reverses the effect of \code{split}.
}
\details{
\code{split} produce a list of \sQuote{pdb} objects with the same 
\code{crystal}, \code{title} and \code{remark} components as \code{x}. Only 
its \code{atoms} component is split while its \code{connect} component is 
cleaned to keep only the meaningful connectivity for each \sQuote{pdb} object
of the list returned by the function. \code{unlist} produce a \sQuote{pdb} 
object with the same \code{crystal}, \code{title} and \code{remark} components
as the first element of \code{value}. The \code{atoms} and \code{connect} 
components of all the elements of \code{value} are combined by row.
}
\examples{
\donttest{
### Split a pdb file by residue IDs and write them into separate files
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))

file.names <- paste0(x$atoms$resname, "_", x$atoms$resid, ".pdb")
file.names <- unique(file.names)
pdb.resid  <- split(x, x$atoms$resid)
length(pdb.resid)
useless <- mapply(write.pdb, pdb.resid, file.names)

# Cleanup
unlink(file.names)
}

}
\seealso{
\code{\link[base]{split}}, \code{\link{unsplit}}, \code{\link{pdb}}
}
\keyword{category}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format.R
\name{format.pdb.title}
\alias{format.pdb.title}
\alias{format.pdb.title.character}
\title{Format "Title" Field of PDB File}
\usage{
\method{format.pdb}{title}(x, ...)

\method{format.pdb.title}{character}(x, ...)
}
\arguments{
\item{x}{a vector of strings;}

\item{\dots}{currently not used;}
}
\value{
properly formatted text string.
}
\description{
Formats the title field of a Protein Data Bank (PDB) file.
}
\details{
This function is mostly used internally, e.g. by \code{\link{write.pdb}},
but may be useful in other instances as well.
}
\examples{
format.pdb.title(c("Molecule 1", "is just an example"))
}
\seealso{
\code{\link{write.pdb}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rotationHelpers.R
\name{rotationHelpers}
\alias{rotationHelpers}
\alias{Rx}
\alias{Rx.coords}
\alias{Rx.pdb}
\alias{Ry}
\alias{Ry.coords}
\alias{Ry.pdb}
\alias{Rz}
\alias{Rz.coords}
\alias{Rz.pdb}
\title{Helper Functions for Rotation of Atomic Coordinates}
\usage{
Rx(...)

\method{Rx}{coords}(x, angle = 0, mask = TRUE, crystal = NULL, ...)

\method{Rx}{pdb}(x, angle = 0, mask = TRUE, crystal = x$crystal, ...)

Ry(...)

\method{Ry}{coords}(x, angle = 0, mask = TRUE, crystal = NULL, ...)

\method{Ry}{pdb}(x, angle = 0, mask = TRUE, crystal = x$crystal, ...)

Rz(...)

\method{Rz}{coords}(x, angle = 0, mask = TRUE, crystal = NULL, ...)

\method{Rz}{pdb}(x, angle = 0, mask = TRUE, crystal = x$crystal, ...)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{x}{an R object containing atomic coordinates.}

\item{angle}{the angle of the rotation in degrees.}

\item{mask}{a logical vector indicating the set of coordinates to which the
rotation has to be applied.}

\item{crystal}{an object of class \sQuote{crystal} used to convert fractional
into Cartesian coordinates when needed.}
}
\value{
An object of the same class as \code{x} with rotated coordinates.
}
\description{
Rotation of atomic coordinates along a specific Cartesian vector.
}
\details{
These functions are helper functions to perform a rotation around a specific 
Cartesian vector. All of them call the \code{R} function.
}
\examples{
# First lets read a pdb file
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
cell <- cell.coords(x)
visualize(x, mode = NULL)

# Rotation of the structure around the z-axis
visualize(Rz(x, 90), mode = NULL)
# Rotation of the residue 1 around the c-axis
visualize(Rz(x, 90, mask = x$atoms$resid == 1), mode = NULL)

}
\seealso{
\code{\link{R}} and \code{\link{xyz2abc}}, \code{\link{abc2xyz}} for
  passing from Cartesian to fractional coordinates (or Vis Versa).
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pdb.R
\name{pdb}
\alias{pdb}
\alias{pdb.default}
\alias{is.pdb}
\title{Create an Object of Class \sQuote{pdb}}
\usage{
pdb(...)

\method{pdb}{default}(
  atoms,
  crystal = NULL,
  connect = NULL,
  remark = NULL,
  title = NULL,
  resolution = NULL,
  ...,
  cryst1 = NULL
)

is.pdb(x)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{atoms}{a data.frame of class \code{atoms} containing ATOM and HETATM records use to create the \code{pdb} object.}

\item{crystal}{a list of class \code{crystal} containing the periodical boundary conditions and space group used to create the \code{pdb} object.}

\item{connect}{a data.frame of class \code{connect} containing CONECT records use to create the \code{pdb} object.}

\item{remark}{a character vector containing some REMARK records to be added to the \code{pdb} object.}

\item{title}{a character vector containing some TITLE records to be added to the \code{pdb} object.}

\item{resolution}{numeric value specifying the resolution; the unit should be specified as an attribute.}

\item{cryst1}{will be deprecated and replaced by argument crystal.}

\item{x}{an R object to be tested.}
}
\value{
\code{pdb} returns a list of class \sQuote{pdb} with the following components:
\item{title}{a character vector containing the TITLE records found in a PDB file.}
\item{remark}{a character vector containing the REMARK records found in a PDB file.}
\item{crystal}{a list of class \sQuote{crystal} containing the first CRYSTAL record
   found in a PDB file. All others are ignored.}
\item{atoms}{a data.frame of class \sQuote{atoms} containing the ATOM and HETATM records
   found in a PDB file.}
\item{connect}{a data.frame of class \sQuote{connect} containing the CONECT records found in a PDB file.}
\cr
\code{is.pdb} returns TRUE if x is an object of class \sQuote{pdb} and FALSE otherwise.
}
\description{
Creates an object of class 'pdb'.
}
\details{
This function is a generic function used to create objects of class \sQuote{pdb}.
  The purpose of this class is to store the data of molecular systems contained in PDB files.
The default method of the \code{pdb} function creates an object of class \sQuote{pdb}
  from its different components, i.e.:
  \code{title}, \code{remark}, \code{crystal}, \code{atoms} and \code{connect}. 
  At least an object of class \sQuote{atoms} has to be specified.\cr\cr

\code{is.pdb} tests if x is an object of class \sQuote{pdb},
  i.e. if x has a \dQuote{class} attribute equal to \code{pdb}.
}
\examples{
title  <- "This is just an example"
remark <- NULL
crystal <- crystal(c(10,10,10))
atoms <- atoms(recname = c("ATOM","ATOM"), eleid = 1:2, elename = c("H","H"), alt = "",
               resname = c("H2","H2"), chainid = "", resid = c(1,1), insert = "",
               x1 = c(0,0), x2 = c(0,0), x3 = c(0,1), occ = c(0.0,0.0), temp = c(1.0,1.0),
               segid = c("H2","H2"))
connect <- connect(eleid.1 = c(1), eleid.2 = c(2))
x <- pdb(atoms = atoms, crystal = crystal, connect = connect, remark = remark, title = title)
is.pdb(x)

}
\seealso{
\code{\link{atoms}}, \code{\link{coords}}, \code{\link{crystal}}, \code{\link{connect}} and \code{\link{read.pdb}}
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge.coords.R
\name{merge.coords}
\alias{merge.coords}
\alias{merge.atoms}
\alias{merge.pdb}
\alias{merge.pdb.hetero}
\alias{merge.pdb.structure}
\alias{merge.pdb.resolution}
\title{Merging Molecular Systems}
\usage{
\method{merge}{coords}(x, y, ...)

\method{merge}{atoms}(x, y, reindex = TRUE, ...)

\method{merge}{pdb}(x, y, reindex = TRUE, ...)

\method{merge}{pdb.hetero}(x, y, ...)

\method{merge}{pdb.structure}(x, y, ...)

\method{merge}{pdb.resolution}(x, y, ...)
}
\arguments{
\item{x, y}{objects of class \code{coords} to be merged.}

\item{\dots}{further arguments passed to or from other methods.}

\item{reindex}{a single element logical vector indicating if residue and
element IDs have to be reindexed after merging.}
}
\value{
Return an object of the same class as \code{x} and \code{y}
  merging \code{x} and \code{y}.
  Note: \code{x} and \code{y} must have the same \code{basis} attributes,
  otherwise an error is returned.
}
\description{
Merge two objects containing atomic coordinates
}
\details{
To merge \code{x} and \code{y} they must have the same \code{basis} 
attributes (see \code{\link{basis}}). \cr\cr
For objects of class \sQuote{coords} and \sQuote{atoms}
  the atomic coordinates are directly merged by row. \cr\cr
For objects of class \sQuote{pdb},
  the \code{atoms} and \code{connect} components of the two \code{pdb} objects
  are merged by row and the \code{crystal} component of \code{x} is used
  to build the returned object. \cr\cr
For objects of class \sQuote{atoms} and \sQuote{pdb}
  the residue and element IDs of \code{y} are shifted to avoid any confusion
  with those of \code{x}. If \code{reindex == TRUE} the \code{\link{reindex}} function
  is called to reinitialize the indexing of the returned object.
The functions of type \sQuote{pdb.hetero}, \sQuote{pdb.structure} and
  \sQuote{pdb.resolution} merge the corresponding \code{pdb} fields.
}
\examples{
c1 <- coords( 1:3 ,  4:6 ,  7:9 , basis = "xyz")
c2 <- coords(10:12, 13:15, 16:18, basis = "xyz")
merge(c1,c2)

\donttest{
## Merging objects with different basis sets returns an error.
c2 <- coords(9:11, 12:14, 15:17, basis = "abc")
try(merge(c1,c2))
}

## Prepare a Pentacene/C70 dimer
C70 <- read.pdb(system.file("examples/C70.pdb", package="Rpdb"))
Pen <- read.pdb(system.file("examples/Pentacene.pdb", package="Rpdb"))
x <- merge(Tz(C70, 3.5, thickness=0.5), Pen)
  
}
\seealso{
\code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}},
  \code{\link{basis}}, \code{merge}, \code{merge.data.frame}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/replicate.R
\name{replicate}
\alias{replicate}
\alias{replicate.coords}
\alias{replicate.atoms}
\alias{replicate.pdb}
\title{Replicate Atomic Coordinates}
\usage{
replicate(x, ...)

\method{replicate}{coords}(x, crystal = NULL, a.ind = 0, b.ind = 0, c.ind = 0, ...)

\method{replicate}{atoms}(x, crystal = NULL, a.ind = 0, b.ind = 0, c.ind = 0, ...)

\method{replicate}{pdb}(x, a.ind = 0, b.ind = 0, c.ind = 0, crystal = NULL, ...)
}
\arguments{
\item{x}{an R object containing atomic coordinates to be replicated.}

\item{\dots}{further arguments passed to or from other methods.}

\item{crystal}{an object of class \sQuote{crystal} containing periodical boundary conditions used for replicating.}

\item{a.ind}{a vector of integers indicating the positions of the replicated cells along the a-axis.}

\item{b.ind}{a vector of integers indicating the positions of the replicated cells along the b-axis.}

\item{c.ind}{a vector of integers indicating the positions of the replicated cells along the c-axis.}
}
\value{
Return an object of class \sQuote{pdb} with replicated atomic coordinates.
}
\description{
Replicate atomic coordinates using periodic boundary conditions.
}
\details{
The \code{replicate} function replicate a unit cell along the lattice vectors a, b and c
as as many times as indicated by the \code{a.ind}, \code{b.ind} and \code{c.ind} arguments.
Discontinuous integer vectors can be used for \code{a.ind}, \code{b.ind} and \code{c.ind}
to create layered supercells (See examples).
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))

# Create a 3x3 supercell
y <- replicate(x, a.ind = 0:2, b.ind = 0:2, c.ind = 0:2)

# Create a 3x3 supercell which might need to be wrapped (if some molecules are outside the cell)
y <- replicate(x, a.ind = -1:1, b.ind = -1:1, c.ind = -1:1)

# Create a layered supercell with a vacuum layer in the bc-plan
y <- replicate(x, a.ind = c(0,2), b.ind = 0:2, c.ind = 0:2)

}
\seealso{
\code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}}, \code{\link{crystal}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/translation.R
\name{translation}
\alias{translation}
\alias{Txyz}
\alias{Txyz.coords}
\alias{Txyz.pdb}
\alias{Tabc}
\alias{Tabc.coords}
\alias{Tabc.pdb}
\title{Translation of Atomic Coordinates}
\usage{
Txyz(...)

\method{Txyz}{coords}(
  obj,
  x = 0,
  y = 0,
  z = 0,
  mask = TRUE,
  thickness = NULL,
  crystal = NULL,
  ...
)

\method{Txyz}{pdb}(
  obj,
  x = 0,
  y = 0,
  z = 0,
  mask = TRUE,
  thickness = NULL,
  crystal = obj$crystal,
  ...
)

Tabc(...)

\method{Tabc}{coords}(
  obj,
  a = 0,
  b = 0,
  c = 0,
  mask = TRUE,
  thickness = NULL,
  crystal = NULL,
  ...
)

\method{Tabc}{pdb}(
  obj,
  a = 0,
  b = 0,
  c = 0,
  mask = TRUE,
  thickness = NULL,
  crystal = obj$crystal,
  ...
)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{obj}{an R object containing atomic coordinates.}

\item{x}{the x-component of the translation vector.}

\item{y}{the y-component of the translation vector.}

\item{z}{the z-component of the translation vector.}

\item{mask}{a logical vector indicating the set of coordinates to which to
apply the translation.}

\item{thickness}{a numeric value indicating the fraction of the thicknees of
the selected atom to be added to the translation vector (Usually 0, 0.5 or
1. See details).}

\item{crystal}{an object of class \sQuote{crystal} use to convert Cartesian
into fraction coordinates (or Vis Versa) when need.}

\item{a}{the a-component of the translation vector.}

\item{b}{the b-component of the translation vector.}

\item{c}{the c-component of the translation vector.}
}
\value{
An object of the same class as \code{x} with translated coordinates.
}
\description{
Translation of Cartesian or fractional coordinates.
}
\details{
\code{Txyz} and \code{Tabc} are generic functions. Method for objects of 
class \sQuote{coords} first convert the coordinates into Cartesian or 
fractional coordinates using \code{crystal} if needed to performed the 
translation. Once translated, the coordinates are reconverted back to the 
original basis set using again \code{crystal}. Method for objects of class 
\sQuote{pdb} first extract coordinates from the object using the function 
\code{coords}, perform the translation, and update the coordinates of the 
\sQuote{pdb} object using the function \code{coords<-}. The \code{thickness} 
argument can be use to translate selected atoms by a fraction of its 
thickness along the translation direction. This can be use when merging two 
fragments centered at the origin to build a dimer to avoid atomic overlap and
set the inter-fragment distance (see examples).
}
\examples{
# First lets read a pdb file
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
visualize(x, mode = NULL)
visualize(Txyz(x, y=10), mode = NULL)
visualize(Txyz(x, y=10,  mask = x$atoms$resid==1), mode = NULL)
visualize(Tabc(x, b= 1), mode = NULL)
visualize(Tabc(x, b= 1,  mask = x$atoms$resid==1), mode = NULL)

# Lets build a C70/Pentacene dimer with an inter-molecular distance equal to 3.5
C70 <- read.pdb(system.file("examples/C70.pdb", package="Rpdb"))
Pen <- read.pdb(system.file("examples/Pentacene.pdb", package="Rpdb"))
x <- merge(C70, Pen)
visualize(x, mode = NULL)
viewXY()
visualize(Txyz(x, x=0, y=0, z=3.5, mask = x$atoms$resname == "C70", thickness=0.5), mode = NULL)
viewXY()

}
\seealso{
Helper functions for translation along given Cartesian or lattice vector:\cr
\code{\link{Tx}}, \code{\link{Ty}}, \code{\link{Tz}}, \code{\link{Ta}}, \code{\link{Tb}}, \code{\link{Tc}}\cr
Passing from Cartesian to fractional coordinates (or Vis Versa):\cr
\code{\link{xyz2abc}}, \code{\link{abc2xyz}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/range.coords.R
\name{range.coords}
\alias{range.coords}
\alias{range.atoms}
\alias{range.pdb}
\title{Range of Atomic Coordinates}
\usage{
\method{range}{coords}(x, na.rm = FALSE, finite = FALSE, ...)

\method{range}{atoms}(x, na.rm = FALSE, finite = FALSE, ...)

\method{range}{pdb}(x, na.rm = FALSE, finite = FALSE, ...)
}
\arguments{
\item{x}{an R object containing atomic coordinates.}

\item{na.rm}{logical, indicating if \code{\link{NA}}'s should be omitted.}

\item{finite}{logical, indicating if all non-finite elements should be omitted.}

\item{\dots}{further arguments passed to or from other methods.}
}
\value{
Return a \code{\link{data.frame}} whose columns contain the range of
  the first, second and third coordinates of \code{x}.
}
\description{
Determines the range of atomic coordinates.
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
range(x)
range(range(x))

}
\seealso{
\code{range}, \code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/translationHelpers.R
\name{translationHelpers}
\alias{translationHelpers}
\alias{Tx}
\alias{Tx.coords}
\alias{Tx.pdb}
\alias{Ty}
\alias{Ty.coords}
\alias{Ty.pdb}
\alias{Tz}
\alias{Tz.coords}
\alias{Tz.pdb}
\alias{Ta}
\alias{Ta.coords}
\alias{Ta.pdb}
\alias{Tb}
\alias{Tb.coords}
\alias{Tb.pdb}
\alias{Tc}
\alias{Tc.coords}
\alias{Tc.pdb}
\title{Helper Functions for Translation of Atomic Coordinates}
\usage{
Tx(...)

\method{Tx}{coords}(obj, x = 0, mask = TRUE, thickness = NULL, crystal = NULL, ...)

\method{Tx}{pdb}(obj, x = 0, mask = TRUE, thickness = NULL, crystal = obj$crystal, ...)

Ty(...)

\method{Ty}{coords}(obj, y = 0, mask = TRUE, thickness = NULL, crystal = NULL, ...)

\method{Ty}{pdb}(obj, y = 0, mask = TRUE, thickness = NULL, crystal = obj$crystal, ...)

Tz(...)

\method{Tz}{coords}(obj, z = 0, mask = TRUE, thickness = NULL, crystal = NULL, ...)

\method{Tz}{pdb}(obj, z = 0, mask = TRUE, thickness = NULL, crystal = obj$crystal, ...)

Ta(...)

\method{Ta}{coords}(obj, a = 0, mask = TRUE, crystal = NULL, ...)

\method{Ta}{pdb}(obj, a = 0, mask = TRUE, crystal = obj$crystal, ...)

Tb(...)

\method{Tb}{coords}(obj, b = 0, mask = TRUE, crystal = NULL, ...)

\method{Tb}{pdb}(obj, b = 0, mask = TRUE, crystal = obj$crystal, ...)

Tc(...)

\method{Tc}{coords}(obj, c = 0, mask = TRUE, crystal = NULL, ...)

\method{Tc}{pdb}(obj, c = 0, mask = TRUE, crystal = obj$crystal, ...)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{obj}{an R object containing atomic coordinates.}

\item{x}{the x-component of the translation vector.}

\item{mask}{a logical vector indicating the set of coordinates to which to apply the translation.}

\item{thickness}{a numeric value indicating the fraction of the thicknees of the selected atom to be added to the translation vector (Usually 0, 0.5 or 1. See details).}

\item{crystal}{an object of class \sQuote{crystal} used to convert Cartesian into fractional coordinates (or Vice Versa) when need.}

\item{y}{the y-component of the translation vector.}

\item{z}{the z-component of the translation vector.}

\item{a}{the a-component of the translation vector.}

\item{b}{the b-component of the translation vector.}

\item{c}{the c-component of the translation vector.}
}
\value{
An object of the same class as \code{x} with translated coordinates.
}
\description{
Translation of atomic coordinates along a specific Cartesian or lattice
vector.
}
\details{
These functions are helper functions to perform a translation along a
specific Cartesian or lattice vector. All of them call either the \code{Txyz}
or \code{Tabc} function.
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
visualize(x, mode = NULL)
visualize(Ty(x, 10), mode = NULL)
visualize(Ty(x, 10, mask = x$atoms$resid==1), mode = NULL)
visualize(Tb(x, 1 ), mode = NULL)
visualize(Tb(x, 1 , mask = x$atoms$resid==1), mode = NULL)

# Lets build a C70/Pentacene dimer with an inter-molecular distance equal to 3.5
C70 <- read.pdb(system.file("examples/C70.pdb", package="Rpdb"))
Pen <- read.pdb(system.file("examples/Pentacene.pdb", package="Rpdb"))
x <- merge(C70, Pen)
visualize(x, mode = NULL)
viewXY()
visualize(Tz(x, z=3.5, mask = x$atoms$resname == "C70", thickness=0.5), mode = NULL)
viewXY()

}
\seealso{
\code{\link{Txyz}}, \code{\link{Tabc}}\cr
Passing from Cartesian to fractional coordinates (or Vis Versa):\cr
\code{\link{xyz2abc}}, \code{\link{abc2xyz}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset.atoms.R
\name{subset.atoms}
\alias{subset.atoms}
\alias{subset.pdb}
\title{Subsetting \sQuote{atoms} and \sQuote{pdb} Objects}
\usage{
\method{subset}{atoms}(x, subset, drop = FALSE, reindex.all = TRUE, ...)

\method{subset}{pdb}(x, subset, drop = FALSE, reindex.all = TRUE, ...)
}
\arguments{
\item{x}{object to be subsetted.}

\item{subset}{logical expression indicating elements or rows to keep: missing values are taken as false.}

\item{drop}{passed on to [ indexing operator.}

\item{reindex.all}{a single element logical vector indicating if residues and elements IDs have to be reindexed after subsetting.}

\item{\dots}{further arguments to be passed to or from other methods.}
}
\value{
Return a subsetted object of the same class as \code{x}.
}
\description{
Return subsets of \sQuote{atoms} or \sQuote{pdb} objects which meet
conditions.
}
\details{
For an \sQuote{atoms} object the method is similar to the data.frame method
(see \code{\link{subset}}) but allow to directly reindex the elements and
residues IDs. For a \sQuote{pdb} object subsetting is applied on the
\code{atoms} and \code{connect} components of the object in a consistent way.
First the \code{atoms} component is subsetted and then the \code{connect}
component is filtered to keep only the connectivity for the subset.
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
y <- subset(x, x$atoms$eleid \%in\% sample(x$atoms$eleid, 10))
is(y)
y <- subset(x$atoms, x$atoms$eleid \%in\% sample(x$atoms$eleid, 10))
is(y)
x <- coords(x)
y <- subset(x, x < 0)
is(y)

}
\seealso{
\code{\link[base]{subset}}, \code{\link{pdb}}, \code{\link{atoms}}, \code{\link{reindex}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vectorialOperations.R
\name{vectorialOperations}
\alias{vectorialOperations}
\alias{dotProd}
\alias{vectNorm}
\alias{rotVect}
\alias{vectProd}
\title{Basic Vectorial Operations}
\usage{
dotProd(U, V)

vectNorm(U)

rotVect(U, n = 1)

vectProd(U, V)
}
\arguments{
\item{U}{a numeric vector of length 3.}

\item{V}{a numeric vector of length 3.}

\item{n}{an integer.}
}
\value{
\itemize{
\item{\code{dotProd}  return a single element numeric vector.}
\item{\code{vectNorm} return a single element numeric vector.}
\item{\code{rotVect}  return a numeric vector of length 3.}
\item{\code{vectProd} return a numeric vector of length 3.}
}
}
\description{
Basic vectorial operations such as scalar product and vectorial product
}
\examples{
Vx <- c(3,0,0)
vectNorm(Vx)
Vx <- Vx / vectNorm(Vx)
Vy <- c(0,1,0)
Vz <- vectProd(Vx, Vy)
print(Vz)

}
\seealso{
\code{\link[base]{matmult}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/connect.NucleicAcids.R
\name{connect.nucleic}
\alias{connect.nucleic}
\alias{connect.nucleic.pdb}
\alias{connect.nucleic.atoms}
\title{Connects atoms in Nucleic acids}
\usage{
\method{connect}{nucleic}(x, ...)

\method{connect.nucleic}{pdb}(x, ...)

\method{connect.nucleic}{atoms}(x, ...)
}
\arguments{
\item{x}{a \code{pdb} of \code{atoms} object containing atomic identifiers.}

\item{\dots}{arguments passed to methods.}
}
\value{
\code{connect.nucleic} returns an object of class \code{connect} containing
  the explicit connections between all nucleotide atoms.
Note: the detailed format of the returned value may change in the future.
}
\description{
Builds the skeleton for a nucleic acid, by connecting the individual atoms
for each nucleotide.
}
\details{
\code{connect.nucleic} is a generic function with methods for
  \code{pdb} and \code{atoms} objects.
It creates object of class \sQuote{connect} with the skeleton of each nucleotide.
}
\examples{
# Download some pdb files:
#   7EQJ = tRNA Val;
# x = read.pdb("9J4O.pdb")
# visualize(atoms(x), connect = connect.nucleic(x), type="l", pbc = FALSE);
}
\seealso{
\code{\link{pdb}}, \code{\link{connect}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/universalConstants.R
\name{universalConstants}
\alias{universalConstants}
\title{Universal Constants}
\format{
A data frame containing for each universal constant the following information.
\describe{
  \item{\code{Quantity}}{a character vector containing a short description of the constants.}
  \item{\code{Value}}{a numeric vector containing the value of the constants.}
  \item{\code{Unit}}{a character vector indicating the unit of the constants.}
}
}
\source{
http://www.ebyte.it/library/educards/constants/ConstantsOfPhysicsAndMath.html
}
\description{
This data set provides various universal constants
}
\examples{
# Data for the speed of light
universalConstants["c",]

# Return the speed of light in m.s-1
universalConstants["c","Value"]

# Return the Planck constant in J.s
universalConstants["h","Value"]

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/centres.R
\name{centres}
\alias{centres}
\alias{centres.coords}
\alias{centres.atoms}
\alias{centres.pdb}
\title{Centres-of-Geometry and Centres-of-Mass}
\usage{
centres(x, ...)

\method{centres}{coords}(x, factor = NULL, weights = NULL, unsplit = FALSE, na.rm = FALSE, ...)

\method{centres}{atoms}(x, factor = NULL, weights = NULL, unsplit = FALSE, na.rm = FALSE, ...)

\method{centres}{pdb}(x, factor = NULL, weights = NULL, unsplit = FALSE, na.rm = FALSE, ...)
}
\arguments{
\item{x}{an R object containing atomic coordinates.}

\item{\dots}{further arguments passed to or from other methods.}

\item{factor}{a factor used to split the atomic coordinates by groups to
compute multiple centres.}

\item{weights}{a numerical vector containing atomic weights used to compute
centres-of-mass.}

\item{unsplit}{a logical value indicating whether the coordinates of the
centres have to be unsplit to repeat their coordinates for each atom used
for their calculation (used for wrapping by groups).}

\item{na.rm}{a logical value indicating whether NA values should be stripped
before the computation proceeds.}
}
\value{
Return an object of class \sQuote{coords} containing the coordinates
  of centres.
}
\description{
Computes centres-of-geometry and centres-of-mass of groups of atoms.
}
\details{
\code{centres} is a generic function to compute centres-of-geometry and 
centres-of-mass from an object containing atomic coordinates. For objects of 
class \sQuote{coords}, \sQuote{atoms} and \sQuote{pdb}, the coordinates of 
\code{x} are first splitted into groups defined by \code{factor} using the 
\code{\link{split}} function. For each group, the weighted mean of the 
\code{x1}, \code{x2} and \code{x3} components of \code{x} are calculated 
using \code{weights}. By default all atoms are assumed to have the same 
weight (calculation of centres-of-geometry). Finally, if \code{unplit = TRUE}
the coordinates of the centres are unsplitted using the \code{\link{unsplit}}
function to assign to each atom the coordinates of the centre to which they 
are attached (used for wrapping by groups). \cr \cr For objects of class 
\sQuote{atoms} and \sQuote{pdb} by default \code{factor} is set to 
\code{x$resid} and \code{x$coordinates$resid}, respectively, to compute the 
centre-of-geometry of the different resdiues. Notice that coordinates can be 
neglected for the calculation of the centres using NA values in 
\code{factor}.
}
\examples{
# First lets read a pdb file
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))

# Centres-of-geometry of the residues
centres(x)

# Centre-of-geometry of the whole structure
centres(x, factor = rep(1, natom(x)))
# or
centres(coords(x))

# Centres-of-geometry of the PCB and DCB residues
centres(x, factor = x$atoms$resname)

# Knowing the name of the elements forming
# the C60 of the PCBM molecules (PCB residues)
# we can compute the centres-of-geometry of
# the C60 by neglecting the other atoms of the
# PCB residues.
C60.elename <- paste0("C", sprintf("\%0.3d", 1:60))

is.PCB <- x$atoms$resname == "PCB" # Produce a mask to select only the PCB residues
is.C60 <- is.PCB & x$atoms$elename \%in\% C60.elename # Produce a mask to keep only the C60

F <- x$atoms$resid # We use the residue IDs to split the coordinates
F[!is.C60] <- NA # We keep only the atoms of the C60

C60.centres <- centres(x, factor = F)

# Lets check the position of the C60 centres
visualize(x , mode = NULL)
spheres3d(C60.centres)
text3d(Ty(C60.centres, 2), text=paste0("PCB_", rownames(C60.centres)), cex=2)

# Centres-of-mass of the resdiues
symb <- toSymbols(x$atoms$elename) # Convert elename into elemental symbols
# Find the mass of the element in the periodic table
w <- elements[match(symb, elements[,"symb"]), "mass"] 
centres(x, weights = w)

}
\seealso{
\code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}},
\code{\link{elements}} \cr\cr and \code{\link{split}}, \code{\link{unsplit}},
\code{\link[base]{factor}} for details about splitting data sets.
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coords.R
\name{coords}
\alias{coords}
\alias{coords<-}
\alias{as.coords}
\alias{coords.default}
\alias{coords.data.frame}
\alias{coords.matrix}
\alias{coords.coords}
\alias{coords.atoms}
\alias{coords<-.atoms}
\alias{coords.pdb}
\alias{coords<-.pdb}
\alias{is.coords}
\title{The Atomic Coordinates of an Object}
\usage{
coords(...)

coords(x) <- value

as.coords(...)

\method{coords}{default}(x1, x2, x3, basis = "xyz", ...)

\method{coords}{data.frame}(x, basis = NULL, ...)

\method{coords}{matrix}(x, basis = NULL, ...)

\method{coords}{coords}(x, ...)

\method{coords}{atoms}(x, ...)

\method{coords}{atoms}(x) <- value

\method{coords}{pdb}(x, ...)

\method{coords}{pdb}(x) <- value

is.coords(x)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{x}{an R object containing atomic coordinates.}

\item{value}{an object of class \sQuote{coords} used for replacement}

\item{x1, x2, x3}{numeric vectors containing the first, second and third coordinates.}

\item{basis}{a single element character vector indicating the type of basis vector
used to express the atomic coordinates.}
}
\value{
The accessor function returns a data.frame of class \sQuote{coords}
whose columns contain the three coordinates of the atoms of a molecular
system. The coordinates can either be Cartesian (\code{basis} attribute equal
to \code{"xyz"}) or fractional coordinates (\code{basis} attribute equal to
\code{"abc"}). \cr\cr The replacement function returns an object of the same
class as \code{x} with updated coordinates. \cr\cr \code{is.coords} returns
TRUE if x is an object of class \sQuote{coords} and FALSE otherwise
}
\description{
Get or set the atomic coordinates (either Cartesian or fractional 
coordinates) of an object.
}
\details{
The purpose of the \sQuote{coords} class is to store the coordinates of a 
molecular system and facilitate their manipulation when passing from the 
Cartesian to fractional coordinates and vice versa.\cr
\code{coords} and \code{coords<-} are generic accessor and replacement
functions. \code{as.coords} is an alias to \code{coords}.\cr
The default method of the \code{coords} function is actually a builder allowing 
to create a \sQuote{coords} object from its different components, i.e.: 
\code{x1}, \code{x2}, \code{x3}, and \code{basis}. All the arguments have to 
be specified except 'basis' which by default is set to "xyz" (Cartesian 
coordinates). \cr\cr
For an object of class \sQuote{atoms}, the accessor function
extracts its \code{x1}, \code{x2} and \code{x3} components as well 
as its \code{basis} attribute to create a \sQuote{coords} object.

The replacement function sets \code{x1}, \code{x2} and \code{x3} components,
as well as the \code{basis} attribute. \cr\cr
For an object of class \sQuote{coords}, the accessor function
returns the \sQuote{coords} object as is.

For an object of class \sQuote{pdb}, the accessor function extracts the \code{x1},
\code{x2} and \code{x3} components, as well as the \code{basis} attribute of its 
\code{atoms} component to create a \sQuote{coords} object. The replacement 
function sets the \code{x1}, \code{x2} and \code{x3} components as well as the
\code{basis} attribute of its \code{atoms} component. \cr\cr

For \sQuote{matrix} and \sQuote{data.frame} objects, when \code{basis == NULL}
this function searches x, y, z or a, b, c columns in \code{x}.\cr
If x, y, z columns are found, they are used to set the first, second and
third coordinates of the returned \sQuote{coords} object.
In that case the basis set of \code{x} is set to \code{"xyz"}.\cr
If a, b, c columns are found they are used to a 
set the first, second and third coordinates of the returned \sQuote{coords} 
object. In that case the basis set of \code{x} is set to \code{"abc"}.\cr If 
the function doesn't found neither the x, y, z nor the a, b, c columns an 
error is returned.\cr When \code{basis!=NULL} it has to be equal to 
\code{"xyz"} or \code{"abc"} and \code{x} must have exactly 3 columns. \cr\cr
\code{is.coords} tests if x is an object of class \sQuote{coords}, i.e. if x 
has a \dQuote{class} attribute equal to \code{coords}.
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
is.coords(x)
is.coords(x$atoms)

## Replace the coordinates of x by translated coordinates
coords(x) <- coords(Tz(x, 10))
coords(x)

}
\seealso{
\code{\link{basis}}
}
\keyword{classes}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reindex.R
\name{reindex}
\alias{reindex}
\alias{reindex.atoms}
\alias{reindex.pdb}
\title{Reinitialize Object Indexing}
\usage{
reindex(...)

\method{reindex}{atoms}(x, eleid = TRUE, resid = TRUE, ...)

\method{reindex}{pdb}(x, eleid = TRUE, resid = TRUE, ...)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{x}{an R object.}

\item{eleid}{a single element logical vector indicating if elements IDs have to reindexed.}

\item{resid}{a single element logical vector indicating if residues IDs have to reindexed.}
}
\value{
Return an object of the same class as \code{x} with updated indexes.
}
\description{
Reinitialize the indexing of an object.
}
\details{
\code{reindex} is a generic function to reinitialize the indexing of an object or its components.
The methods for objects of class \sQuote{atoms} reinitialize the residue and element IDs starting
from 1 and avoiding gaps in the indexes. For objects of class \sQuote{pdb} their \code{atoms} and
\code{connect} components are reindexed consistently.
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
x <- subset(x, x$atoms$eleid \%in\% sample(x$atoms$eleid, 10))
print(x)
x <- reindex(x)
print(x)

}
\seealso{
\code{\link{pdb}}, \code{\link{atoms}}, \code{\link{subset.atoms}}, \code{\link{subset.pdb}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inertia.R
\name{inertia}
\alias{inertia}
\alias{inertia.coords}
\alias{inertia.atoms}
\alias{inertia.pdb}
\title{Moment of Inertia of a Molecular System}
\usage{
inertia(...)

\method{inertia}{coords}(x, m = NULL, ...)

\method{inertia}{atoms}(x, m = NULL, ...)

\method{inertia}{pdb}(x, m = NULL, ...)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{x}{an R object containing atomic coordinates.}

\item{m}{a numeric vector containing atomic masses.}
}
\value{
Return the inertia tensor in a 3x3 matrix.
}
\description{
Computes the inertia tensor of a molecular system from atomic coordinates and
masses.
}
\details{
\code{inertia} is a generic function to compute the inertia tensor of a
molecular system. For object of class \sQuote{coords} both atomic coordinates
and masses have to be speifyed. For object of class \sQuote{atoms} the masses
are determined from the \code{elename} component of the object (see
\code{\link{toSymbols}} and \code{\link{masses}}). For object of class
\sQuote{pdb} the \code{atoms} component is used.
}
\examples{
C70 <- read.pdb(system.file("examples/C70.pdb", package="Rpdb"))
inertia(C70)
visualize(C70, mode = NULL)
viewXY()
viewInertia(C70)

}
\seealso{
\code{\link{toSymbols}}, \code{\link{masses}}, \code{\link{viewInertia}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proj.R
\name{proj.line3d}
\alias{proj.line3d}
\alias{proj.line3d.numeric}
\alias{proj.line3d.matrix}
\title{Projections of Objects}
\usage{
proj.line3d(p, x, y, z, ...)

\method{proj}{line3d.numeric}(p, x, y = NULL, z = NULL, ...)

\method{proj}{line3d.matrix}(p, x, y = NULL, z = NULL, ...)
}
\arguments{
\item{p}{numeric array or matrix with the x,y,z coordinates of the point or collection of points.}

\item{x, y, z}{an R object containing the coordinates of the end-points of the line segment; can be specified either as individual values or as a matrix.}

\item{\dots}{currently not used.}
}
\value{
The \code{proj.line3d} function returns an object
of class \sQuote{proj} containing the 3-dimensional coordinates of
the projected point, as well as a scalar value representing the fraction
of the path on the line segment.
}
\description{
Computes 3D Projections
}
\details{
The purpose of the \sQuote{proj} helper functions is to compute
the spatial projections of various objects on other geometrical structures.
Currently, only projection of points on a line segment are implemented.

The methods for the \sQuote{proj.line3d} function compute the projections
of 1 point or a collection of points on a line segment defined by the 2
delimiting points.

The \sQuote{numeric} method projects a point, while the \sQuote{matrix} method
projects a collection of points on another geometric object.
}
\examples{
p = c(1,2,3)
line = matrix(c(0,5,2,3,1,4), nrow = 2)
proj.line3d(p, line)

}
\keyword{classes}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/connect.R
\name{connect}
\alias{connect}
\alias{conect.default}
\alias{connect.default}
\alias{connect.coords}
\alias{connect.atoms}
\alias{connect.pdb}
\alias{connect.character}
\alias{is.connect}
\title{Create \sQuote{connect} Object}
\usage{
conect.default(eleid.1, eleid.2, ...)

connect(...)

\method{connect}{default}(eleid.1, eleid.2, ...)

\method{connect}{coords}(x, radii = 0.75, safety = 1.2, by.block = FALSE, maxLimit = 3200, ...)

\method{connect}{atoms}(x, safety = 1.2, by.block = FALSE, ...)

\method{connect}{pdb}(x, safety = 1.2, by.block = FALSE, ...)

\method{connect}{character}(pdbRec, atoms, ...)

is.connect(x)
}
\arguments{
\item{eleid.1}{a integer vector containing the IDs of bonded atoms.}

\item{eleid.2}{a integer vector containing the IDs of bonded atoms.}

\item{\dots}{arguments passed to methods.}

\item{x, atoms}{an R object containing atomic coordinates.}

\item{radii}{a numeric vector containing atomic radii used to find neigbours.}

\item{safety}{a numeric value used to extend the atomic radii.}

\item{by.block}{a logical value indicating whether the connectivity has to be
determine by block (see details).}

\item{maxLimit}{integer value specifying the maximum number of atoms permitted
to try connecting by distance.}

\item{pdbRec}{the raw text lines from the pdb file.}
}
\value{
\code{connect} returns a two-column data.frame of class \sQuote{connect}
  whose rows contain the IDs of bonded atoms. The columns of this data.frame
  are described below:
\item{eleid.1}{an integer vector containing the elements IDs
   defining the connectivity of the system.} 
\item{eleid.2}{an integer vector containing the elements IDs
   defining the connectivity of the system.}\cr\cr

\code{is.connect} returns TRUE if \code{x} is an object of class \sQuote{coords}
  and FALSE otherwise.
}
\description{
Creates an object of class \sQuote{connect} containing the IDs of bonded atoms
defining the connectivity of a molecular system.
}
\details{
\code{connect} is a generic function to create objects of class \sQuote{connect}.
  The purpose of this class is to store CONNECT records from PDB files,
  indicating the connectivity of a molecular system.\cr
The default method creates a \code{connect} object from its different components, i.e.: 
  \code{eleid.1} and \code{eleid.2}. Both arguments have to be specified.\cr 
The S3 method for an object of class \sQuote{coords} determines the connectivity 
  from atomic coordinates. A distance matrix is computed, then, for each pair 
  of atoms the distance is compared to a bounding distance computed from atomic 
  radii. If this distance is lower than the bounding distance then the atoms 
  are assumed to be connected.\cr
The S3 method for object of class \sQuote{pdb} first uses the element names
  to search for atomic radii in the \code{elements} data set.
  Then atomic coordinates and radii are passed to \code{connect.coords}.\cr
  If \code{by.block == TRUE}, a grid is defined to determined the connectivity by block.
  The method is slow but allows to deal with very large systems.\cr
The S3 method for object of class \sQuote{character} converts the raw string
  from a pdb file into a \code{connect} object.

\code{is.connect} tests if an object is of class \sQuote{connect},
  i.e. if it has a \dQuote{class} attribute equal to \code{connect}.
}
\examples{
# If atom 1 is connected to atom 2, 3, 4 and 5
# then we can prepare the following 'connect' object:
x <- connect(rep(1,4), 2:5)
print(x)
is.connect(x)

# Compute connectivity from coordinates
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"), CONNECT = FALSE)
x$connect
x$connect <- connect(x)
x$connect

}
\seealso{
\code{\link{pdb}}
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/basis.R
\name{basis}
\alias{basis}
\alias{basis.default}
\alias{basis<-}
\alias{basis<-.default}
\alias{basis.pdb}
\alias{basis<-.pdb}
\title{The Basis of an Object}
\usage{
basis(x)

\method{basis}{default}(x)

basis(x) <- value

\method{basis}{default}(x) <- value

\method{basis}{pdb}(x)

\method{basis}{pdb}(x) <- value
}
\arguments{
\item{x}{an R object containing atomic coordinates.}

\item{value}{a single element character vector use to set the basis of 
\code{x}.}
}
\value{
\describe{ \item{For \code{basis}:}{NULL or a single element
character vector. (NULL is given if the object has no \code{basis}
attribute.)} \item{For \code{basis<-}:}{the updated object. (Note that the
value of \code{basis(x) <- value} is that of the assignment, value, not the
return value from the left-hand side.)} }
}
\description{
Functions to get or set the basis of an object containing atomic coordinates.
}
\details{
\code{basis} and \code{basis<-} are respectively generic accessor and 
replacement functions. The default methods get and set the \code{basis} 
attribute of an object containing atomic coordinates. This attribute indicate
the type basis vector used to express atomic coordinates.\cr \code{value} 
must be equal to \code{"xyz"}, for Cartesian, or \code{"abc"}, for fractional
coordinates.\cr The methods for objects of class \sQuote{pdb} get and set the
\code{basis} attribute of its \code{atoms} component.
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
basis(x)
x <- xyz2abc(x)
basis(x)

}
\seealso{
\code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}}
}
\keyword{attribute}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/natom.R
\name{natom}
\alias{natom}
\alias{natom.coords}
\alias{natom.atoms}
\alias{natom.pdb}
\title{Number of Atoms in an Object Containing Atomic Coordinates}
\usage{
natom(x, ...)

\method{natom}{coords}(x, factor = NULL, ...)

\method{natom}{atoms}(x, factor = NULL, ATOM = TRUE, HETATM = TRUE, ...)

\method{natom}{pdb}(x, factor = NULL, ATOM = TRUE, HETATM = TRUE, ...)
}
\arguments{
\item{x}{an R object containing atomic coordinates.}

\item{\dots}{further arguments passed to or from other methods.}

\item{factor}{a factor used to split the object and evaluate the number of atoms
in each group.}

\item{ATOM}{a single element logical vector indicating if ATOM records have
to be considered or not.}

\item{HETATM}{a single element logical vector indicating if HETATM records
have to be considered or not.}
}
\value{
Return an integer or a vector of integer of lenght equal to 
  \code{nlevels(factor)} (if \code{factor} is specify) indication the number
  of atoms in the object or in the groups defined by \code{factor}.
}
\description{
Evaluates the number of atoms in an object containing atomic coordinates.
}
\details{
\code{natom} is a generic function to evalute the number of atom in an object
containing atomic coordinates. The atomic coordinates of the object are first
filtered to keep ATOM and/or HETATM records as indicated by the 'ATOM' and 
'HETATM' arguments. Then, if \code{factor} is specify, the object is splitted
to evalute the number of atoms in each group defined by \code{factor}. If 
\code{factor} is not specify then the total number of atoms in the object is 
return.
}
\examples{
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))

natom(x)
natom(x, x$atoms$resid)
natom(x, x$atoms$resname)
natom(x, HETATM = FALSE)
  
}
\seealso{
\code{\link{coords}}, \code{\link{atoms}}, \code{\link{pdb}},
  \code{\link[base]{factor}}, \code{\link[base]{split}}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mirror.R
\name{mirror}
\alias{mirror}
\alias{mirror.coords}
\alias{mirror.pdb}
\title{Reflection of Atomic Coordinates}
\usage{
mirror(...)

\method{mirror}{coords}(x, p1, p2 = NULL, p3 = NULL, mask = TRUE, crystal = NULL, ...)

\method{mirror}{pdb}(x, p1, p2 = NULL, p3 = NULL, mask = TRUE, crystal = x$crystal, ...)
}
\arguments{
\item{\dots}{further arguments passed to or from other methods.}

\item{x}{an R object containing atomic coordinates.}

\item{p1}{a numeric vector of length 3 containing the coordinates of the
first point defining the reflexion plan. Can also be a 3x3 matrix or
data.frame containing by row \code{p1}, \code{p2} and \code{p3}.}

\item{p2}{a numeric vector of length 3 containing the coordinates of the
second point defining the reflexion plane.}

\item{p3}{a numeric vector of length 3 containing the coordinates of the
third point defining the reflexion plane.}

\item{mask}{a logical vector indicating the set of coordinates to which to
apply the reflexion.}

\item{crystal}{an object of class \sQuote{crystal} used to convert fractional
into Cartesian coordinates (when needed).}
}
\value{
An object of the same class as \code{x} with reflected coordinates.
}
\description{
Perform a reflection (or mirror) operation on atomic coordinates with respect 
to a given reflection plane.
}
\details{
\code{mirror} is a generic function.
Method for objects of class \sQuote{coords} first convert the coordinates
into Cartesian coordinates using \code{crystal} if needed.
Once reflected, the coordinates are reconverted back to the original basis set
using again \code{crystal}.

Method for objects of class \sQuote{pdb} first extract coordinates from the object 
using the function \code{coords}, perform the reflection, and update the 
coordinates of the \sQuote{pdb} object using the function \code{coords<-}.
}
\examples{
# First lets read a pdb file
x <- read.pdb(system.file("examples/PCBM_ODCB.pdb", package="Rpdb"))
cell <- cell.coords(x)
visualize(x, mode = NULL)

# Mirror operation with respect to the ab-plane
visualize(mirror(x, rep(0,3), p1=cell[, "a"], p2=cell[, "b"]), mode = NULL)
# Mirror operation with respect to the ab-plane for residue 1
visualize(mirror(x, rep(0,3), p1=cell[, "a"], p2=cell[, "b"],
   mask = x$atoms$resid == 1), mode = NULL)

}
\seealso{
Helper functions for reflection with respect to a given Cartesian
plane or a plane defined by two lattice vectors:\cr \code{\link{Mxy}},
\code{\link{Myz}}, \code{\link{Mzx}}, \code{\link{Mab}}, \code{\link{Mbc}},
\code{\link{Mca}}\cr Passing from Cartesian to fractional coordinates (or Vis
Versa):\cr \code{\link{xyz2abc}}, \code{\link{abc2xyz}}
}
\keyword{manip}
